/*
 * FILE NAME rtl_gpio.c
 *
 * BRIEF MODULE DESCRIPTION
 *  GPIO For Flash Reload Default
 *
 *  Author: jimmylin@realtek.com.tw
 *
 * Copyright 2005 Realtek Semiconductor Corp.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 *  THIS  SOFTWARE  IS PROVIDED   ``AS  IS'' AND   ANY  EXPRESS OR IMPLIED
 *  WARRANTIES,   INCLUDING, BUT NOT  LIMITED  TO, THE IMPLIED WARRANTIES OF
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN
 *  NO  EVENT  SHALL   THE AUTHOR  BE	LIABLE FOR ANY   DIRECT, INDIRECT,
 *  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 *  NOT LIMITED   TO, PROCUREMENT OF  SUBSTITUTE GOODS  OR SERVICES; LOSS OF
 *  USE, DATA,  OR PROFITS; OR  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 *  ANY THEORY OF LIABILITY, WHETHER IN  CONTRACT, STRICT LIABILITY, OR TORT
 *  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 *  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *  You should have received a copy of the  GNU General Public License along
 *  with this program; if not, write  to the Free Software Foundation, Inc.,
 *  675 Mass Ave, Cambridge, MA 02139, USA.
 */



#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <asm/errno.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <linux/miscdevice.h>
#include <linux/pci.h>
#include <linux/delay.h>
#include <linux/slab.h>
#include <linux/sched.h>
#include <linux/init.h>
#include <linux/types.h>
#include <linux/reboot.h>
#include <linux/kmod.h>
#include <linux/proc_fs.h>
#include <asm/rtl865x/rtl865xc_asicregs.h>

#define RTL_GPIO_PABCNR		PABCD_CNR
#define RTL_GPIO_PABDIR		PABCD_DIR
#define RTL_GPIO_PABDATA	PABCD_DAT
#define RTL_GPIO_PEFCNR		PEFGH_CNR
#define RTL_GPIO_PEFDIR		PEFGH_DIR
#define RTL_GPIO_PEFDATA	PEFGH_DAT

#define RS232_RTS_PIN		26  // RS232 RTS			//GPIO_D[2]
#define RS232_DTR_PIN		30  // RS232 DTR			//GPIO_D[6]

#define RS232_DCD_PIN		1   // RS232 DCD			//GPIO_E[1]
#define RS232_CTS_PIN		2   // RS232 CTS			//GPIO_E[2]
#define RS232_DSR_PIN		3   // RS232 DSR			//GPIO_E[3]

#define RTL_R32(addr)		(*(volatile unsigned long *)(addr))
#define RTL_W32(addr, l)	((*(volatile unsigned long*)(addr)) = (l))
#define RTL_R8(addr)		(*(volatile unsigned char*)(addr))
#define RTL_W8(addr, l)		((*(volatile unsigned char*)(addr)) = (l))

#define ON		1
#define OFF		0
#define SHARE_PIN_MUX		0xB8000030

static unsigned int RS232_RTS_FLAG = OFF;
static unsigned int RS232_DTR_FLAG = OFF;
static unsigned int RS232_DCD_FLAG = OFF;
static unsigned int RS232_CTS_FLAG = OFF;
static unsigned int RS232_DSR_FLAG = OFF;

static int rs232_read_proc(char *page, char **start, off_t off, int count, int *eof, void *data)
{
	int len=0, rts=0, dtr=0, dcd=0, cts=0, dsr=0;

	if(!(RTL_R32(RTL_GPIO_PABDATA) & (1 << RS232_RTS_PIN)))
		rts = 1;

	if(!(RTL_R32(RTL_GPIO_PABDATA) & (1 << RS232_DTR_PIN)))
		dtr = 1;

	if(!(RTL_R32(RTL_GPIO_PEFDATA) & (1 << RS232_DCD_PIN)))
		dcd = 1;

	if(!(RTL_R32(RTL_GPIO_PEFDATA) & (1 << RS232_CTS_PIN)))
		cts = 1;

	if(!(RTL_R32(RTL_GPIO_PEFDATA) & (1 << RS232_DSR_PIN)))
		dsr = 1;

	len += sprintf(page+len, "RS232_RTS=%d\n", rts);
	len += sprintf(page+len, "RS232_DTR=%d\n", dtr);
	len += sprintf(page+len, "RS232_DCD=%d\n", dcd);
	len += sprintf(page+len, "RS232_CTS=%d\n", cts);
	len += sprintf(page+len, "RS232_DSR=%d\n", dsr);

	return len;
}

static int rs232_write_proc(struct file *file, const char *buffer, unsigned long count, void *data)
{
	unsigned char rs232_sw[4];
	int ret = -EFAULT;

	memset(rs232_sw, 0, sizeof(rs232_sw));

	if (buffer && !copy_from_user(rs232_sw, buffer, 4)) {

		if(!memcmp(rs232_sw, "RTS", 3)) { //RTS

			if(rs232_sw[3] == '0') {

				RS232_RTS_FLAG = OFF;
				RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) | (1 << RS232_RTS_PIN)));

			} else if(rs232_sw[3] == '1') {

				RS232_RTS_FLAG = ON;
				RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) & (~(1 << RS232_RTS_PIN))));
			}

			ret = count;

		} else if(!memcmp(rs232_sw, "DTR", 3)) { //DTR

			if(rs232_sw[3] == '0') {

				RS232_DTR_FLAG = OFF;
				RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) | (1 << RS232_DTR_PIN)));

			} else if(rs232_sw[3] == '1') {

				RS232_DTR_FLAG = ON;
				RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) & (~(1 << RS232_DTR_PIN))));
			}

			ret = count;
#if 0
		} else if(!memcmp(rs232_sw, "DCD", 3)) { //DCD

			if(rs232_sw[3] == '0') {

				RS232_DCD_FLAG = OFF;
				RTL_W32(RTL_GPIO_PEFDATA, (RTL_R32(RTL_GPIO_PEFDATA) | (1 << RS232_DCD_PIN)));

			} else if(rs232_sw[3] == '1') {

				RS232_DCD_FLAG = ON;
				RTL_W32(RTL_GPIO_PEFDATA, (RTL_R32(RTL_GPIO_PEFDATA) & (~(1 << RS232_DCD_PIN))));
			}

			ret = count;

		} else if(!memcmp(rs232_sw, "CTS", 3)) { //CTS

			if(rs232_sw[3] == '0') {

				RS232_CTS_FLAG = OFF;
				RTL_W32(RTL_GPIO_PEFDATA, (RTL_R32(RTL_GPIO_PEFDATA) | (1 << RS232_CTS_PIN)));
				RTL_W32(RTL_GPIO_PEFDATA, (RTL_R32(RTL_GPIO_PEFDATA) & (~(1 << RS232_CTS_PIN))));

			} else if(rs232_sw[3] == '1') {

				RS232_CTS_FLAG = ON;
				RTL_W32(RTL_GPIO_PEFDATA, (RTL_R32(RTL_GPIO_PEFDATA) | (1 << RS232_CTS_PIN)));
				RTL_W32(RTL_GPIO_PEFDATA, (RTL_R32(RTL_GPIO_PEFDATA) & (~(1 << RS232_CTS_PIN))));
			}

			ret = count;

		} else if(!memcmp(rs232_sw, "DSR", 3)) { //DSR

			if(rs232_sw[3] == '0') {

				RS232_DSR_FLAG = OFF;
				RTL_W32(RTL_GPIO_PEFDATA, (RTL_R32(RTL_GPIO_PEFDATA) | (1 << RS232_DSR_PIN)));

			} else if(rs232_sw[3] == '1') {

				RS232_DSR_FLAG = ON;
				RTL_W32(RTL_GPIO_PEFDATA, (RTL_R32(RTL_GPIO_PEFDATA) & (~(1 << RS232_DSR_PIN))));
			}

			ret = count;
#endif
		}
	}

	return ret;
}

static void rs232_init_gpio (void)
{
	// GPIO pin: 0, Parallel: 1
	// input pin: 0, output pin: 1
	
	/*	Configure share PIN to GPIO	*/
	RTL_W32(SHARE_PIN_MUX,(RTL_R32(SHARE_PIN_MUX) | 0x3 ));	// Program Share pin (P0-MII) as GPIO
 
	// RS232 RTS
    RTL_W32(RTL_GPIO_PABCNR,(RTL_R32(RTL_GPIO_PABCNR) & (~(1 << RS232_RTS_PIN))));
    RTL_W32(RTL_GPIO_PABDIR,(RTL_R32(RTL_GPIO_PABDIR) | (1 << RS232_RTS_PIN)));
	RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) | (1 << RS232_RTS_PIN)));

	// RS232 DTR
    RTL_W32(RTL_GPIO_PABCNR,(RTL_R32(RTL_GPIO_PABCNR) & (~(1 << RS232_DTR_PIN))));
    RTL_W32(RTL_GPIO_PABDIR,(RTL_R32(RTL_GPIO_PABDIR) | (1 << RS232_DTR_PIN)));
	RTL_W32(RTL_GPIO_PABDATA, (RTL_R32(RTL_GPIO_PABDATA) | (1 << RS232_DTR_PIN)));

	// RS232 DCD
    RTL_W32(RTL_GPIO_PEFCNR,(RTL_R32(RTL_GPIO_PEFCNR) & (~(1 << RS232_DCD_PIN))));
    RTL_W32(RTL_GPIO_PEFDIR,(RTL_R32(RTL_GPIO_PEFDIR) & (~(1 << RS232_DCD_PIN))));

	// RS232 CTS
    RTL_W32(RTL_GPIO_PEFCNR,(RTL_R32(RTL_GPIO_PEFCNR) & (~(1 << RS232_CTS_PIN))));
    RTL_W32(RTL_GPIO_PEFDIR,(RTL_R32(RTL_GPIO_PEFDIR) & (~(1 << RS232_CTS_PIN))));

	// RS232 DSR
    RTL_W32(RTL_GPIO_PEFCNR,(RTL_R32(RTL_GPIO_PEFCNR) & (~(1 << RS232_DSR_PIN))));
    RTL_W32(RTL_GPIO_PEFDIR,(RTL_R32(RTL_GPIO_PEFDIR) & (~(1 << RS232_DSR_PIN))));

}

int __init rs232_init(void)
{
	struct proc_dir_entry *res=NULL;

	printk("GPIO Driver initial\n");

	// Initial GPIO
	rs232_init_gpio();

	res = create_proc_entry("rs232", 0, NULL);
	if (res) {
		res->read_proc = rs232_read_proc;
		res->write_proc = rs232_write_proc;
	}
	else {
		printk("GPIO Driver, create proc failed!\n");
	}

	return 0;
}


static void __exit rs232_exit(void)
{
	printk("Unload GPIO Driver \n");
}


module_exit(rs232_exit);
module_init(rs232_init);

MODULE_LICENSE("GPL");
MODULE_DESCRIPTION("GPIO driver for RS232");
