#ifndef __RTL8651_CRYPTO_IF_H
#define __RTL8651_CRYPTO_IF_H
#include "rtl8651_crypto.h"


/* Commands */
#define RTL_CRYPT_NO_OP		0x0000
#define RTL_CRYPT_ENCODE	0x0100
#define RTL_CRYPT_DECODE	0x0080 
#define RTL_CRYPT_DES		0x0001	// default CBC mode
#define RTL_CRYPT_3DES		0x0002	// default CBC mode
#define RTL_CRYPT_AES		0x0020	// default CBC mode
#define RTL_CRYPT_ECB		0x0010	// Meaningful only for [3]DES/AES, default CBC mode 
#define RTL_CRYPT_HMAC_MD5	0x0004
#define RTL_CRYPT_HMAC_SHA1	0x0008
#define RTL_CRYPT_APPEND96	0x0040	// Meaningful only for HMAC, append ICV to output 


/* Return codes */
#define RTL_CRYPT_DONE			1
#define RTL_CRYPT_OK			0
#define RTL_CRYPT_ERROR			-1
#define RTL_CRYPT_NO_RESOURCE 	-2


struct rtl8651_crypto_request
{
	unsigned int  id;	// Automatically assigned
	char 	*inbuf;		/* Input buffer */
	char 	*outbuf;	/* Output buffer */
	int 	inlen;		/* Length of input buffer (authenticated range) */
	int 	outlen;		/* On input, max size. On Output, real size of output */

	unsigned int flags;					

	volatile unsigned int cmds;	/* Operation to perform */
	int	crypt_offset;	// En/Decrypt Offset, from inbuf, to start operation 
	int crypt_len;		// En/Decrypt Length of operation 

	unsigned char *crypt_key;	// K1(0-7) K2(8-15) K3(16-24)
	unsigned char *crypt_key_d;	// for AES decryption
	unsigned char *crypt_iv;
	unsigned int  keys_len;
	unsigned char *hmac_key_pad; // OPAD and IPAD	
	unsigned char hmac_result[20];	/* On output, contains the hash result */
	unsigned char hmac_key_len;		// 20: SHA1, 16: MD5
	unsigned char hmac_reslen;		// 12

	void (*callback)(struct rtl8651_crypto_request *);	/* function to call when done request. This struct * is passed as argument. */
	void *priv;					/* Private to requester */
	
	volatile unsigned char is_done;
};


/* Prototypes */

extern struct rtl8651_crypto_request *rtl8651_crypto_alloc_request(int flag);
extern int rtl8651_crypto_execute(struct rtl8651_crypto_request *req);	
extern int rtl8651_crypto_query_request(struct rtl8651_crypto_request *req);
extern void rtl8651_crypto_free_request(struct rtl8651_crypto_request *r);
// void rtl8651_crypto_set_hmac_key(struct rtl8651_crypto_request *r, char * hmac_key, int len);

#endif

