#if defined(MODVERSIONS)
#include <linux/modversions.h>
#endif
#include <linux/module.h>
#include <linux/version.h>
#include <linux/types.h>
#include <linux/skbuff.h>
#include <net/udp.h>
#include <net/tcp.h>
#include <linux/netfilter/xt_webstr.h>
#include <linux/netfilter/x_tables.h>
#include <linux/netfilter_ipv4/ip_tables.h>

#define split(word, wordlist, next, delim) \
    for (next = wordlist, \
	strncpy(word, next, sizeof(word)), \
	word[(next=strstr(next, delim)) ? strstr(word, delim) - word : sizeof(word) - 1] = '\0', \
	next = next ? next + sizeof(delim) - 1 : NULL ; \
	strlen(word); \
	next = next ? : "", \
	strncpy(word, next, sizeof(word)), \
	word[(next=strstr(next, delim)) ? strstr(word, delim) - word : sizeof(word) - 1] = '\0', \
	next = next ? next + sizeof(delim) - 1 : NULL)

#define BUFSIZE 	1024

/* Flags for get_http_info() */
#define HTTP_HOST	0x01
#define HTTP_URL	0x02

/* Flags for mangle_http_header() */
#define HTTP_COOKIE	0x04

#if 0
#define SPARQ_LOG       printk
#else
#define SPARQ_LOG(format, args...)
#endif

typedef struct httpinfo {
    char host[BUFSIZE + 1];
    int hostlen;
    char url[BUFSIZE + 1];
    int urllen;
    char refer[BUFSIZE + 1];
    int referlen;
} httpinfo_t;

/* Return 1 for match, 0 for accept, -1 for partial. */
static int find_pattern2(const char *data, size_t dlen,
	const char *pattern, size_t plen, char term,
	unsigned int *numoff, unsigned int *numlen)
{
    size_t i, j, k;
    int state = 0;
    *numoff = *numlen = 0;

    SPARQ_LOG("%s: pattern = '%s', dlen = %u\n",__FUNCTION__, pattern, dlen);
    if (dlen == 0)
        return 0;

    if (dlen <= plen) {	/* Short packet: try for partial? */
        if (strnicmp(data, pattern, dlen) == 0)
            return -1;
        else 
            return 0;
    }

    for (i = 0; i <= (dlen - plen); i++) {
        /* DFA : \r\n\r\n :: 1234 */
        if (*(data + i) == '\r') {
            if (!(state % 2)) 
                ++state;	    /* forwarding move */
            else 
                state = 0;		/* reset */
        }
        else if (*(data + i) == '\n') {
            if (state % 2) 
                ++state;
            else 
                state = 0;
        }
        else state = 0;

        if (state >= 4)
            break;

        /* pattern compare */
        if (memcmp(data + i, pattern, plen ) != 0)
            continue;

        /* Here, it means patten match!! */
        *numoff=i + plen;
        for (j = *numoff, k = 0; data[j] != term; j++, k++)
            if (j > dlen) 
                return -1 ;	/* no terminal char */

        *numlen = k;
        return 1;
    }
    return 0;
}                                                  

static int get_http_info(const struct sk_buff *skb, int flags, httpinfo_t *info)
{
    struct iphdr *iph = (skb)->nh.iph;
    struct tcphdr *tcph = (void *)iph + iph->ihl*4;
    unsigned char *data = (void *)tcph + tcph->doff*4;
    unsigned int datalen = (skb)->len - (iph->ihl*4) - (tcph->doff*4);

    int found, offset;
    int hostlen, pathlen;
    int ret = 0;

    SPARQ_LOG("%s: seq=%u\n", __FUNCTION__, ntohl(tcph->seq));

    /* Basic checking, is it HTTP packet? */
    if (datalen < 10)
        return ret;	/* Not enough length, ignore it */

    if (memcmp(data, "GET ", sizeof("GET ") - 1) != 0 &&
        memcmp(data, "POST ", sizeof("POST ") - 1) != 0 &&
        memcmp(data, "HEAD ", sizeof("HEAD ") - 1) != 0) //zg add 2006.09.28 for cdrouter3.3 item 186(cdrouter_urlfilter_15)
        return ret;	/* Pass it */

    if (!(flags & (HTTP_HOST | HTTP_URL)))
        return ret;

    /* find the 'Host: ' value */
    found = find_pattern2(data, datalen, "Host: ", sizeof("Host: ") - 1, '\r', &offset, &hostlen);
    SPARQ_LOG("Host found=%d\n", found);

    if (!found || !hostlen)
        return ret;                                     

    ret++;	/* Host found, increase the return value */
    hostlen = (hostlen < BUFSIZE) ? hostlen : BUFSIZE;
    strncpy(info->host, data + offset, hostlen);
    *(info->host + hostlen) = 0;		/* null-terminated */
    info->hostlen = hostlen;
    SPARQ_LOG("HOST=%s, hostlen=%d\n", info->host, info->hostlen);

    if (!(flags & HTTP_URL))
        return ret;

    /* find the 'GET ' or 'POST ' or 'HEAD ' value */
    found = find_pattern2(data, datalen, "GET ", sizeof("GET ") - 1, '\r', &offset, &pathlen);
    if (!found)
        found = find_pattern2(data, datalen, "POST ", sizeof("POST ") - 1, '\r', &offset, &pathlen);

    /******* zg add 2006.09.28 for cdrouter3.3 item 186(cdrouter_urlfilter_15) ******/
    if (!found)
        found = find_pattern2(data, datalen, "HEAD ", sizeof("HEAD ") - 1, '\r', &offset, &pathlen);

    /************************* zg end 2006.09.28 ****************************/
    SPARQ_LOG("GET/POST found=%d\n", found);
    if (!found || (pathlen -= (sizeof(" HTTP/x.x") - 1)) <= 0)/* ignor this field */
        return ret;

    ret++;	/* GET/POST/HEAD found, increase the return value */
    pathlen = ((pathlen + hostlen) < BUFSIZE) ? pathlen : BUFSIZE - hostlen;
    strncpy(info->url, info->host, hostlen);
    strncpy(info->url + hostlen, data + offset, pathlen);
    *(info->url + hostlen + pathlen) = 0;	/* null-terminated */
    info->urllen = hostlen + pathlen;
    SPARQ_LOG("URL=%s, urllen=%d\n", info->url, info->urllen);

    found = find_pattern2(data, datalen, "Referer: ", sizeof("Referer: ") - 1, '\r', &offset, &pathlen);
    SPARQ_LOG("Referer found=%d, offset %d, pathlen %d\n", found, offset, pathlen);
    if(!found){
        info->referlen = 0;
        return ret;
    }
    
    ret++;
    strncpy(info->refer, data + offset, pathlen);
    *(info->refer + pathlen) = 0;	/* null-terminated */
    SPARQ_LOG("Referer: %s\n", info->refer);    

    return ret;
}

static int
match(const struct sk_buff *skb,
      const struct net_device *in,
      const struct net_device *out,
      const struct xt_match *match,
      const void *matchinfo,
      int offset,
      unsigned int protoff,
      int *hotdrop)
{
	const struct ipt_webstr_info *info = matchinfo;
	struct iphdr *ip = skb->nh.iph;

	char token[] = "<&nbsp;>";
	char *wordlist = (char *)&info->string;
	httpinfo_t htinfo;
	int flags = 0;
	int found = 0;
	long int opt = 0;

	if (!ip || info->len < 1)
	    return 0;

	SPARQ_LOG("\n************************************************\n"
		"%s: type=%s\n", __FUNCTION__, (info->type == IPT_WEBSTR_URL) 
		? "IPT_WEBSTR_URL"  : (info->type == IPT_WEBSTR_HOST) 
		? "IPT_WEBSTR_HOST" : "IPT_WEBSTR_CONTENT" );
	
	/* Determine the flags value for get_http_info(), and mangle packet 
	 * if needed. */
	switch(info->type) {
        case IPT_WEBSTR_URL:	/* fall through */
            flags |= HTTP_URL;

        case IPT_WEBSTR_HOST:
            flags |= HTTP_HOST;
            break;

        case IPT_WEBSTR_CONTENT:
            opt = simple_strtol(wordlist, (char **)NULL, 10);
            SPARQ_LOG("%s: string=%s, opt=%#lx\n", __FUNCTION__, wordlist, opt);
            if (opt & (BLK_JAVA | BLK_ACTIVE | BLK_PROXY))
                flags |= HTTP_URL;

            if (opt & BLK_PROXY)
                flags |= HTTP_HOST;
#if 0
            // Could we modify the packet payload in a "match" module?  --YY@Ralink
            if (opt & BLK_COOKIE)
                mangle_http_header(skb, HTTP_COOKIE);
#endif
		break;

        default:
            printk("%s: Sorry! Cannot find this match option.\n", __FILE__);
            return 0;
	}

	/* Get the http header info */
	if (get_http_info(skb, flags, &htinfo) < 1)
	    return 0;
 
    /* For URL White List */
    if(info->string[0] == '*' || info->string[1] == '*')
        return 1;

	/* Check if the http header content contains the forbidden keyword */
	if (info->type == IPT_WEBSTR_HOST || info->type == IPT_WEBSTR_URL) {
	    int nlen = 0, hlen = 0, hlen_more;
	    char needle[BUFSIZE], *haystack = NULL, *haystack_more = NULL;
	    char *next;

	    if (info->type == IPT_WEBSTR_HOST) {
            haystack = htinfo.host;
            hlen = htinfo.hostlen;
	    }
	    else {
            haystack = htinfo.url;
            hlen = htinfo.urllen;
            if(htinfo.referlen > 0){
                haystack_more = htinfo.refer;
                hlen_more = htinfo.referlen;
            }
	    }
	    split(needle, wordlist, next, token) {
            nlen = strlen(needle);
            SPARQ_LOG("keyword=%s, nlen=%d, hlen=%d\n", needle, nlen, hlen);
            if (!nlen || !hlen || nlen > hlen) 
                continue;

            if (haystack != NULL && strstr(haystack, needle) != NULL){
                found = 1;
                break;
            }
        
            if (haystack_more != NULL && strstr(haystack_more, needle) != NULL){
                found = 1;
                break;
            }
        }
	}
	else {		/* IPT_WEBSTR_CONTENT */
	    int vicelen;
	    if (opt & BLK_JAVA) {
            vicelen = sizeof(".js") - 1;
            if (strnicmp(htinfo.url + htinfo.urllen - vicelen, ".js", vicelen) == 0) {
                SPARQ_LOG("%s: MATCH....java\n", __FUNCTION__);
                found = 1;
                goto match_ret;
            }

            vicelen = sizeof(".class") - 1;
            if (strnicmp(htinfo.url + htinfo.urllen - vicelen, ".class", vicelen) == 0) {
                SPARQ_LOG("%s: MATCH....java\n", __FUNCTION__);
                found = 1;
                goto match_ret;
            }
	    }

	    if (opt & BLK_ACTIVE){
            vicelen = sizeof(".ocx") - 1;
            if (strnicmp(htinfo.url + htinfo.urllen - vicelen, ".ocx", vicelen) == 0) {
                SPARQ_LOG("%s: MATCH....activex\n", __FUNCTION__);
                found = 1;
                goto match_ret;
            }

            vicelen = sizeof(".cab") - 1;
            if (strnicmp(htinfo.url + htinfo.urllen - vicelen, ".cab", vicelen) == 0) {
                SPARQ_LOG("%s: MATCH....activex\n", __FUNCTION__);
                found = 1;
                goto match_ret;
            }
	    }

	    if (opt & BLK_PROXY){
            if (strnicmp(htinfo.url + htinfo.hostlen, "http://", sizeof("http://") - 1) == 0) {
                SPARQ_LOG("%s: MATCH....proxy\n", __FUNCTION__);
                found = 1;
                goto match_ret;
            }
	    }
	}

match_ret:
	SPARQ_LOG("%s: Verdict =======> %s \n",__FUNCTION__ , found ? "DROP" : "ACCEPT");
	return (found ^ info->invert);
}
          
static int checkentry(const char *tablename, 
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,17)
			const void *ip_void,
			const struct xt_match *match,
#else
			const struct ipt_ip *ip,
#endif
			void *matchinfo,
			unsigned int matchsize,
			unsigned int hook_mask)
{
#if 0	
	if (matchsize != IPT_ALIGN(sizeof(struct ipt_webstr_info)))
		return 0;
#endif
	return 1;
}

static struct xt_match webstr_match = { 
#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
	{ NULL, NULL },
	"webstr",
	&match,
	&checkentry,
	NULL,
	THIS_MODULE
#endif
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,0)
	.name		= "webstr",
	.family		= AF_INET,
	.match		= match,
#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,17)
	.matchsize     = sizeof(struct ipt_webstr_info),
#endif
	.checkentry	= checkentry,
	.me		= THIS_MODULE,
#endif
};

static int __init init(void)
{
	return xt_register_match(&webstr_match);
}

static void __exit fini(void)
{
	xt_unregister_match(&webstr_match);
}

module_init(init);
module_exit(fini);

MODULE_AUTHOR("JasonJiang AMIT,inc.");
MODULE_DESCRIPTION("A module to match URL.");
MODULE_ALIAS("ipt_webstr");
