/* dhcpd.c
 *
 * udhcp Server
 * Copyright (C) 1999 Matthew Ramsay <matthewr@moreton.com.au>
 *			Chris Trew <ctrew@moreton.com.au>
 *
 * Rewrite by Russ Dill <Russ.Dill@asu.edu> July 2001
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <fcntl.h>
#include <string.h>
#include <stdlib.h>
#include <sys/wait.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <unistd.h>
#include <signal.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <time.h>
#include <sys/time.h>

#include "dhcpd.h"
#include "arpping.h"
#include "socket.h"
#include "options.h"
#include "files.h"
#include "serverpacket.h"
#include "common.h"
#include "signalpipe.h"
#include "static_leases.h"

#include "sync_list.h"

/* globals */
struct dhcpOfferedAddr *leases;
struct server_config_t server_config;

#if defined(AMIT_ENABLE_SDMZ)
/* super dmz */
u_int8_t sdmz_chaddr[16]; //add by jack 2007/12/07
#endif

#if defined(_AMIT_ENABLE_AIRLIVE_IPCAM)
u_int8_t airLiveCamMac[2]={0x00,0x4f};
u_int8_t zeroMac[6]={0x00,0x00,0x00,0x00,0x00,0x00};
static int checkAirliveIpcam(struct dhcpMessage *oldpacket);
#endif

//mike.chiu Workaround wrong ip pool setting+{
struct ip4_addr
{
	unsigned char a, b, c, d;
};

char* TrToInAddr(uint32_t addrIn)
{
	struct in_addr addr; 	
	addr.s_addr = addrIn;		
	return inet_ntoa(addr);
}
//mike.chiu Workaround wrong ip pool setting+}

//mike.chiu DHCP_List_Error_After_Changing_System_Time +{
struct SystemTimrChange {
	u_int32_t oldsystemtime;		/* host order */
	u_int32_t newsystemtime;	/* host order */
};

#define TEMP_CHANGE_TIME_FILE		"/var/run/changetime"
//mike.chiu DHCP_List_Error_After_Changing_System_Time +}

//mike.chiu Workaround wrong ip pool setting+{
struct ip4_addr dotted_quad_to_ip (char *dottedquad)
{
	struct ip4_addr ip;
	int a, b, c, d;

	sscanf (dottedquad, "%d.%d.%d.%d", &a, &b, &c, &d);
	ip.a = a;
	ip.b = b;
	ip.c = c;
	ip.d = d;

	return ip;
}

/*Check that if the ip pool can match lan ip*/
void CheckIPPool()
{
	struct ip4_addr LAN_IP, IP_START, IP_END;
	
	unsigned char temp_dhcp_ip_start[30];
	unsigned char temp_dhcp_ip_end[30];
	unsigned char temp_lan_ip[30];
	
	unsigned char new_dhcp_ip_start_cmd[64];
	unsigned char new_dhcp_ip_end_cmd[64];	

	LAN_IP 		= dotted_quad_to_ip (TrToInAddr(server_config.server));	
	IP_START 	= dotted_quad_to_ip (TrToInAddr(server_config.start));
	IP_END 		= dotted_quad_to_ip (TrToInAddr(server_config.end));
	
	sprintf(temp_dhcp_ip_start, "%d.%d.%d", IP_START.a, IP_START.b, IP_START.c);
	sprintf(temp_dhcp_ip_end, "%d.%d.%d", IP_END.a, IP_END.b, IP_END.c);
	sprintf(temp_lan_ip, "%d.%d.%d", LAN_IP.a, LAN_IP.b, LAN_IP.c);
	
	if(strcmp(temp_dhcp_ip_start,temp_dhcp_ip_end)==0 && strcmp(temp_dhcp_ip_start,temp_lan_ip)!=0 && strcmp(temp_dhcp_ip_end,temp_lan_ip)!=0)
	{
		/*Wrong ip pool, for example 
		lan ip	=192.168.20.1
		start ip	=192.168.15.101
		end ip	=192.168.15.200
		*/
		LOG(LOG_ERR,"Wrong IP pool! DHCP server will reset IP pool, please check csman of CSID_C_DHCP_SVR_IPPOOL_START & CSID_C_DHCP_SVR_IPPOOL_END \n");
		sprintf(new_dhcp_ip_start_cmd, "wrcsman '0x00030003 {%d.%d.%d.%d}'", LAN_IP.a, LAN_IP.b, LAN_IP.c, IP_START.d);//reset csman CSID_C_DHCP_SVR_IPPOOL_START
		sprintf(new_dhcp_ip_end_cmd, "wrcsman '0x00030004 {%d.%d.%d.%d}'", LAN_IP.a, LAN_IP.b, LAN_IP.c, IP_END.d);//reset csman CSID_C_DHCP_SVR_IPPOOL_END
		system(new_dhcp_ip_start_cmd);
		system(new_dhcp_ip_end_cmd);	    	
		
		sprintf(temp_dhcp_ip_start, "%d.%d.%d.%d", LAN_IP.a, LAN_IP.b, LAN_IP.c, IP_START.d);
		sprintf(temp_dhcp_ip_end, "%d.%d.%d.%d", LAN_IP.a, LAN_IP.b, LAN_IP.c, IP_END.d);
		inet_aton(temp_dhcp_ip_start,&(server_config.start));
		inet_aton(temp_dhcp_ip_end,&(server_config.end));
		
		LOG(LOG_ERR, "Reset: IP pool start = %s, end = %s\n", temp_dhcp_ip_start, temp_dhcp_ip_end);		
	}	
}
//mike.chiu Workaround wrong ip pool setting+}

//mike.chiu DHCP_List_Error_After_Changing_System_Time +{
/*After changing system time, we need to reset the expires of leases[]*/
void ResetExpiresOfLeases()
{
 	FILE *fp;
	struct SystemTimrChange timedata;

	if (!(fp = fopen(TEMP_CHANGE_TIME_FILE, "r")))
	{
		LOG(LOG_ERR, "Unable to open %sfor reading",TEMP_CHANGE_TIME_FILE);
		return;
	}
	
	if ((fread(&timedata, sizeof timedata, 1, fp) == 1)) {			
		LOG(LOG_DEBUG, "oldsystemtime=%ld newsystemtime=%ld",timedata.oldsystemtime,timedata.newsystemtime);
		int i;
		for (i = 0; i < server_config.max_leases; i++)
		{
			if (leases[i].yiaddr != 0 && leases[i].expires != 0) {						
				leases[i].expires = leases[i].expires+(timedata.newsystemtime-timedata.oldsystemtime);
			}
		}				
	}			
	
	fclose(fp);
	remove(TEMP_CHANGE_TIME_FILE);	
}
//mike.chiu DHCP_List_Error_After_Changing_System_Time +}

#ifdef COMBINED_BINARY
int udhcpd_main(int argc, char *argv[])
#else
int main(int argc, char *argv[])
#endif
{
	fd_set rfds;
	struct timeval tv;
	int server_socket = -1;
	int bytes, retval;
	struct dhcpMessage packet;
	uint8_t *state;
	uint8_t *server_id, *requested;
	uint32_t server_id_align, requested_align;
	unsigned long timeout_end;
	struct option_set *option;
	struct dhcpOfferedAddr *lease;
	struct dhcpOfferedAddr static_lease;
	int max_sock;
	unsigned long num_ips;

	uint32_t static_lease_ip;
	unsigned long max_leases_config=0;//MikeChiu Enable_Leases_File 2010/12/17+

	memset(&server_config, 0, sizeof(struct server_config_t));
	read_config(argc < 2 ? DHCPD_CONF_FILE : argv[1]);
	CheckIPPool();//mike.chiu Workaround wrong ip pool setting+
    //MikeChiu Enable_Leases_File 2010/12/17+{
	/*Because we need to handle leases which are not in range(server_config.start to server_config.end)
      ,we must increase the server_config.max_leases*/
        max_leases_config=server_config.max_leases;      
        server_config.max_leases=max_leases_config+read_out_range_leases_num(server_config.lease_file);	
        //MikeChiu Enable_Leases_File 2010/12/17+}
	/* Start the log, sanitize fd's, and write a pid file */
	start_log_and_pid("udhcpd", server_config.pidfile);
	
#if defined(AMIT_ENABLE_SDMZ)
	//add by jack 2007/08/07
	if(server_config.sdmz_ip)
	{
		memset(sdmz_chaddr, 0, sizeof(sdmz_chaddr));
		if(server_config.sdmz_mac)
			sscanf(server_config.sdmz_mac, "%hhx:%hhx:%hhx:%hhx:%hhx:%hhx", &sdmz_chaddr[0], &sdmz_chaddr[1], &sdmz_chaddr[2], &sdmz_chaddr[3], &sdmz_chaddr[4] ,&sdmz_chaddr[5]);
	}
	//end by jack
#endif

	if ((option = find_option(server_config.options, DHCP_LEASE_TIME))) {
		memcpy(&server_config.lease, option->data + 2, 4);
		server_config.lease = ntohl(server_config.lease);
	}
	else server_config.lease = LEASE_TIME;

    //MikeChiu Enable_Leases_File 2010/12/17+{
	if(!max_leases_config)
	{
    //MikeChiu Enable_Leases_File 2010/12/17+}
		/* Sanity check */
		num_ips = ntohl(server_config.end) - ntohl(server_config.start) + 1;
		if (server_config.max_leases > num_ips) {
			LOG(LOG_ERR, "max_leases value (%lu) not sane, "
				"setting to %lu instead",
				server_config.max_leases, num_ips);
			server_config.max_leases = num_ips;
		}
	}//MikeChiu Enable_Leases_File 2010/12/17+

	leases = xcalloc(server_config.max_leases, sizeof(struct dhcpOfferedAddr));
	memset(leases, 0, server_config.max_leases*sizeof(struct dhcpOfferedAddr));//init the leases
	
	read_leases(server_config.lease_file);

	if (read_interface(server_config.interface, &server_config.ifindex,
			   &server_config.server, server_config.arp) < 0)
		return 1;

#ifndef UDHCP_DEBUG
	background(server_config.pidfile); /* hold lock during fork. */
#endif

	/* Setup the signal pipe */
	udhcp_sp_setup();

	timeout_end = time(0) + server_config.auto_time;	
	while(1) { /* loop until universe collapses */

		if (server_socket < 0)
			if ((server_socket = listen_socket(INADDR_ANY, SERVER_PORT, server_config.interface)) < 0) {
				LOG(LOG_ERR, "FATAL: couldn't create server socket, %m");
				return 2;
			}

		max_sock = udhcp_sp_fd_set(&rfds, server_socket);
		if (server_config.auto_time) {
			tv.tv_sec = timeout_end - time(0);
			tv.tv_usec = 0;
		}
		if (!server_config.auto_time || tv.tv_sec > 0) {
			retval = select(max_sock + 1, &rfds, NULL, NULL,
					server_config.auto_time ? &tv : NULL);
		} else retval = 0; /* If we already timed out, fall through */

		if (retval == 0) {
			write_leases();
			timeout_end = time(0) + server_config.auto_time;
			continue;
		} else if (retval < 0 && errno != EINTR) {
			DEBUG(LOG_INFO, "error on select");
			continue;
		}

		switch (udhcp_sp_read(&rfds)) {
		case SIGUSR1:
		//	LOG(LOG_INFO, "Received a SIGUSR1");
			sync_list();
			write_leases();
			/* why not just reset the timeout, eh */
			timeout_end = time(0) + server_config.auto_time;
			continue;
		case SIGTERM:
			write_leases();//MikeChiu Enable_Leases_File 2010/12/17+
			LOG(LOG_INFO, "Received a SIGTERM");
			return 0;
		//mike.chiu DHCP_List_Error_After_Changing_System_Time +{
		/*System time has been changed */
		case SIGUSR2:			
			LOG(LOG_DEBUG, "Received a SIGUSR2");	
			ResetExpiresOfLeases();
			write_leases();
			continue;
		//mike.chiu DHCP_List_Error_After_Changing_System_Time +}
		case 0: break;		/* no signal */
		default: continue;	/* signal or error (probably EINTR) */
		}

		if ((bytes = get_packet(&packet, server_socket)) < 0) { /* this waits for a packet - idle */
			if (bytes == -1 && errno != EINTR) {
				DEBUG(LOG_INFO, "error on read, %m, reopening socket");
				close(server_socket);
				server_socket = -1;
			}
			continue;
		}

		if ((state = get_option(&packet, DHCP_MESSAGE_TYPE)) == NULL) {
			DEBUG(LOG_ERR, "couldn't get option from packet, ignoring");
			continue;
		}

		/* Look for a static lease */
		static_lease_ip = getIpByMac(server_config.static_leases, &packet.chaddr);

		if(static_lease_ip)
		{
			printf("Found static lease: %x\n", static_lease_ip);

			memcpy(&static_lease.chaddr, &packet.chaddr, 16);
			static_lease.yiaddr = static_lease_ip;
			static_lease.expires = 0;

			lease = &static_lease;

		}
		else
		{
		lease = find_lease_by_chaddr(packet.chaddr);
		}

		switch (state[0]) {
		case DHCPDISCOVER:
#if defined(_AMIT_ENABLE_AIRLIVE_IPCAM)
#if AMIT_DEBUG_AIRLIVE
	udhcp_logging(LOG_INFO,"%s<%d> =====Received DHCP Discover==\n",__FUNCTION__,__LINE__);
#endif
			if( 2 == checkAirliveIpcam(&packet) )
			{
				break;
			}


#endif
			DEBUG(LOG_INFO,"received DISCOVER");
			if (sendOffer(&packet) < 0) {
				LOG(LOG_ERR, "send OFFER failed");
			}

			break;
		case DHCPREQUEST:
			DEBUG(LOG_INFO, "received REQUEST");


			requested = get_option(&packet, DHCP_REQUESTED_IP);
			server_id = get_option(&packet, DHCP_SERVER_ID);

			if (requested) memcpy(&requested_align, requested, 4);
			if (server_id) memcpy(&server_id_align, server_id, 4);

			if (lease) {
				if (server_id) {
					/* SELECTING State */
					DEBUG(LOG_INFO, "server_id = %08x", ntohl(server_id_align));
#if defined(AMIT_ENABLE_SDMZ)
					//add by jack 2007/12/07
					if (!memcmp(packet.chaddr, sdmz_chaddr, 16)) {
						if (requested && requested_align == lease->yiaddr) {
							sendACK(&packet, lease->yiaddr);
						}
					}
					//end by jack
					else
#endif
					{
						if (server_id_align == server_config.server && requested && 
					    		requested_align == lease->yiaddr) {
							sendACK(&packet, lease->yiaddr);
						}
					}
				} else {
					if (requested) {
						/* INIT-REBOOT State */
						if (lease->yiaddr == requested_align)
							sendACK(&packet, lease->yiaddr);
						else sendNAK(&packet);
					} else {
						/* RENEWING or REBINDING State */
						if (lease->yiaddr == packet.ciaddr)
							sendACK(&packet, lease->yiaddr);
						else {
							/* don't know what to do!!!! */
							sendNAK(&packet);
						}
					}
				}

			/* what to do if we have no record of the client */
			} else if (server_id) {
				/* SELECTING State */

			} else if (requested) {
				/* INIT-REBOOT State */
				if ((lease = find_lease_by_yiaddr(requested_align))) {
					if (lease_expired(lease)) {
						/* probably best if we drop this lease */
						memset(lease->chaddr, 0, 16);
					/* make some contention for this address */
					} else sendNAK(&packet);
				} else if (ntohl(requested_align) < ntohl(server_config.start) ||
					   ntohl(requested_align) > ntohl(server_config.end)) {
					sendNAK(&packet);
				} /* else remain silent */
				/*  modify by ken 2008/12/31  */
				else  
				//mike.chiu  SendACK_if_requested_IP_is_legal +{
				{					
					if (	/* exclude Router Lan IP */
						ntohl(requested_align) != ntohl(server_config.server))
					{
						sendACK(&packet, requested_align);						
					}
					else
					{						
						sendNAK(&packet);											
					}							
				}
				//mike.chiu  SendACK_if_requested_IP_is_legal  +}	
				/*end by ken */  
			} else {
				 /* RENEWING or REBINDING State */
				 //mike.chiu  SendACK_if_requested_IP_is_legal *{
				 if (ntohl(packet.ciaddr) >= ntohl(server_config.start) && ntohl(packet.ciaddr) <= ntohl(server_config.end) && ntohl(packet.ciaddr) != ntohl(server_config.server))				
				 	sendACK(&packet, packet.ciaddr);				
				 else
					sendNAK(&packet);    // modify by ken 2008/12/31
				//mike.chiu  SendACK_if_requested_IP_is_legal *}
			   	
			}
		
			break;
		case DHCPDECLINE:
			DEBUG(LOG_INFO,"received DECLINE");
			if (lease) {
				memset(lease->chaddr, 0, 16);
				lease->expires = time(0) + server_config.decline_time;
			}
			break;
		case DHCPRELEASE:
			DEBUG(LOG_INFO,"received RELEASE");
			if (lease) lease->expires = time(0);
			break;
		case DHCPINFORM:
			DEBUG(LOG_INFO,"received INFORM");
			send_inform(&packet);
			break;
		default:
			LOG(LOG_WARNING, "unsupported DHCP message (%02x) -- ignoring", state[0]);
		}
	}

	return 0;
}
#if defined(_AMIT_ENABLE_AIRLIVE_IPCAM)
//-1 :Unexpect error
// 0 :Not airlive cam
// 1 :AirCam but already in static leases List
// 2 :AirCam but NOT in static leases List
static int checkAirliveIpcam(struct dhcpMessage *oldpacket)
{
#if AMIT_DEBUG_AIRLIVE
	udhcp_logging(LOG_INFO,"%s<%d>\n",__FUNCTION__,__LINE__);
#endif
	int  ret = 0,csman_fd , i ,enable = 0;
	unsigned char mac[6];


	bzero(mac,sizeof(mac));
	if( memcmp(oldpacket->chaddr,airLiveCamMac,2) )//not 0 , not airLive cam Mac
	{
#if AMIT_DEBUG_AIRLIVE
	udhcp_logging(LOG_INFO,"%s<%d> =====NO Match MAC==\n",__FUNCTION__,__LINE__);
#endif
		return ret;
	}
	csman_fd = open_csman(NULL,0);
	for(i=0 ; i < AIRLIVE_CAM_NUMBER ;i++)
	{
		if( read_csman(csman_fd ,(CSID_C_IPCAM_AIRLIVE_MAC+i), mac, sizeof(mac), CSM_R_ZERO) < 0 )
		{ //read csman failed
			ret = -1;
			break;
		}
		//stop loop when meet empty entry ,which index the number of entry available
		if(memcmp(mac,zeroMac,6) == 0 )
		{

#if AMIT_DEBUG_AIRLIVE
			udhcp_logging(LOG_INFO,"%s<%d> XXXXX Mac of zero length\n",__FUNCTION__,__LINE__);
#endif
			break;	
		}
		else if( memcmp(oldpacket->chaddr,mac,6) == 0 )//found  a exist entry
		{
			ret  = 1;
			break;
		}

	}
	//Sanity check , exhaust all 8 sets but no empty slot
	if(AIRLIVE_CAM_NUMBER == i )
	{
#if AMIT_DEBUG_AIRLIVE
		udhcp_logging(LOG_INFO,"%s<%d>  Exhaust 8 sets\n",__FUNCTION__,__LINE__);
#endif
		close_csman(csman_fd);
		return ret;
	}


	//i index the empty slot so we can store the 
	//new IPCam stats and set the notify to acknowledge the cmder 
	//to restart the udhcpd to valid the new static leases
	if( 1 != ret)
	{

#if AMIT_DEBUG_AIRLIVE
		udhcp_logging(LOG_INFO,"%s<%d> ==== LOG CAM INFO ==== check it now\n",__FUNCTION__,__LINE__);
#endif
		uint32_t addr;
		struct in_addr ip;
		addr = ntohl(server_config.start);//to host order
		addr = (addr & 0xFFFFFF00 | 0xC9) + i;
		ip.s_addr = htonl(addr);

#if AMIT_DEBUG_AIRLIVE
		udhcp_logging(LOG_INFO,"%s<%d> ====addr =%lu  mac=%d:%d:%d----\n",__FUNCTION__,__LINE__,addr,(oldpacket->chaddr)[0],(oldpacket->chaddr)[1],(oldpacket->chaddr)[2]);
#endif

				write_csman(csman_fd, CSID_C_IPCAM_AIRLIVE_IP + i , &ip ,sizeof(struct in_addr) ,0 );
				write_csman(csman_fd,CSID_C_IPCAM_AIRLIVE_MAC + i ,oldpacket->chaddr,6,0 );
		int notifyDhcpsrv = 1;
		write_csman(csman_fd, CSID_S_DHCP_SVR_ALTERED, &notifyDhcpsrv, 4, 0);
		ret = 2;
	}
	close_csman(csman_fd);
#if AMIT_DEBUG_AIRLIVE
	udhcp_logging(LOG_INFO,"%s<%d> ret=%d ----END \n",__FUNCTION__,__LINE__,ret);
#endif
	return ret;
}
#endif
