/*
 * Copyright c                Realtek Semiconductor Corporation, 2003
 * All rights reserved.                                                    
 * 
 * $Header: /home/cvsroot/linux-2.6.19/linux-2.6.x/drivers/net/re865x/rtl_nic.c,v 1.22 2008/04/11 10:49:14 bo_zhao Exp $
 *
 * $Author: bo_zhao $
 *
 * Abstract: Pure L2 NIC driver, without RTL865X's advanced L3/4 features.
 *
 *   re865x_nic.c: NIC driver for the RealTek 865* 
 *
 */

#define DRV_RELDATE		"Mar 25, 2004"
#include <linux/config.h>
#include <linux/autoconf.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/compiler.h>
#include <linux/netdevice.h>
#include <linux/inetdevice.h>
#include <linux/etherdevice.h>
#include <linux/init.h>
#include <linux/pci.h>
#include <linux/delay.h>
#include <linux/ethtool.h>
#include <linux/mii.h>
#include <linux/if_vlan.h>
#include <linux/crc32.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/rtl865x/interrupt.h>
#include <asm/rtl865x/re865x.h>
#include <linux/slab.h>
#include <linux/signal.h>
#include <linux/proc_fs.h>
#include <linux/time.h>
#include <linux/rtc.h>

#ifdef CONFIG_RTK_VLAN_SUPPORT
#include <linux/rtk_vlan.h>
#endif

#if	defined(CONFIG_RTL8196_RTL8366) && defined(CONFIG_RTL865X_IGMP_SNOOPING)
#undef	CONFIG_RTL865X_IGMP_SNOOPING
#endif

#include "version.h"

#include "rtl_nic.h"

#include "common/rtl_types.h"
#include "common/mbuf.h"
#include "common/rtl_glue.h"

#include "AsicDriver/asicRegs.h"
#include "common/rtl_queue.h"
#ifdef CONFIG_RTL865X_LAYERED_ASIC_DRIVER
#include "common/rtl_queue.h"
#include "AsicDriver/rtl865x_asicCom.h"
#include "AsicDriver/rtl865x_asicL2.h"
#ifdef CONFIG_RTL865X_LAYERED_ASIC_DRIVER_L3
#include "AsicDriver/rtl865x_asicL3.h"
#endif

#else
#include "common/rtl8651_aclLocal.h"
#include "AsicDriver/rtl8651_tblAsicDrv.h"
#endif

#include "common/rtl_errno.h"
#include "common/rtl8651_tblDrvProto.h"

#include "swNic2.h"
#include "assert.h"
#include "rtl865xc_swNic.h"

/*common*/
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
#include "common/rtl865x_vlan.h"
#include "common/rtl865x_netif.h"
#include "common/rtl865x_netif_local.h"
#else
#include "common/rtl865x_common.h"
#include "common/rtl865x_common_local.h"
#endif

/*l2*/
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
#ifdef CONFIG_RTL865X_LAYERED_DRIVER_L2
#include "l2Driver/rtl865x_fdb.h"
#endif
#else
#include "l2Driver/rtl865x_layer2.h"
#include "l2Driver/rtl865x_layer2_local.h"
#endif

/*l3*/
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
#ifdef CONFIG_RTL865X_LAYERED_DRIVER_L3
#include "l3Driver/rtl865x_ip.h"
#include "l3Driver/rtl865x_nexthop.h"
#include "l3Driver/rtl865x_ppp.h"
#include "l3Driver/rtl865x_route.h"
#include "l3Driver/rtl865x_arp.h"
#include "l4Driver/rtl865x_nat.h"
#endif
#else
#include "l3Driver/rtl865x_layer3.h"
#include "l3Driver/rtl865x_layer3_local.h"
#include "l4Driver/rtl865x_layer4.h"
#include "l4Driver/rtl865x_layer4_local.h"
#endif

/*l4*/


#undef CONFIG_RTL865X_ROMEPERF
#ifdef	CONFIG_RTL865X_ROMEPERF
#include "romeperf.h"
#endif

#ifdef RTL865X_DRIVER_DEBUG_FLAG
#include "common/rtl865x_proc_debug.h"
#endif

#if defined(CONFIG_PROC_FS) && defined(CONFIG_NET_QOS) && defined(CONFIG_RTL865X_LAYERED_DRIVER)
#if defined(CONFIG_RTL865X_HW_QOS_SUPPORT)
#include <l2Driver/rtl865x_outputQueue.h>
#endif
#endif

#ifdef CONFIG_RTL865X_HW_PPTPL2TP
#include <linux/netfilter_ipv4/ip_conntrack.h>
#include <linux/netfilter_ipv4/ip_conntrack_pptp.h>
#endif

#ifdef CONFIG_RTL_STP
#include "l2Driver/rtl865x_stp.h"
#endif

#ifdef CONFIG_RTL865X_IGMP_SNOOPING
#include "igmpsnooping/rtl865x_igmpsnooping.h"
#include "linux/ip.h"
//uint32 nicIgmpModuleIndex=0xFFFFFFFF;
extern uint32 brIgmpModuleIndex;
static unsigned int curLinkPortMask=0;
#endif

#ifdef CONFIG_RTL865X_HARDWARE_MULTICAST
#include "l3Driver/rtl865x_multicast.h"
#endif

#ifdef CONFIG_RTL8196_RTL8366
#include "RTL8366RB_DRIVER/gpio.h"
#include "RTL8366RB_DRIVER/rtl8366rb_apiBasic.h"
#endif

#ifdef CONFIG_RTL8186_GR
static int oldStatus = 0;
#endif

#if 0
#define DEBUG_ERR printk
#else
#define DEBUG_ERR(format, args...)
#endif

#ifdef CONFIG_RTL865X_LAYERED_DRIVER

#else
extern struct lr_cpu_stats 					lr_stats;
extern struct vlan_table					vlan_tbl;
extern struct acl_table						acl_tbl;
extern struct if_table						if_tbl;

extern struct natip_table					natip_tbl;
extern struct rt_table						rt_tbl;
extern struct arp_table						arpt_tbl;
extern struct pppoe_table					pppoe_tbl;

extern struct fdb_table						fdb_tbl;
extern struct port_attribute					port_attr;

#endif

extern struct nat_table						nat_tbl;


static int32 __865X_Config;
//#define RX_TASKLET
#define TX_TASKLET
#if defined(CONFIG_NET_WIRELESS_AGN) || (CONFIG_RTL8192SE)
#define BR_SHORTCUT
#endif

#define RTK_QUE

#define BIT(x)	(1<<(x))

#ifdef BR_SHORTCUT
static int eth_flag=12; // 0 dynamic tasklet, 1 - disable tasklet, 2 - always tasklet , bit2 - bridge shortcut enabled
static int pkt_cnt=0, enable_brsc=0;
__DRAM_FWD  unsigned char cached_eth_addr[6];
__DRAM_FWD  struct net_device *cached_dev;
#endif

#if defined (CONFIG_RTL_UNKOWN_UNICAST_CONTROL)
#define	RTL_MAC_RECORD_NUM		4	/* Must be 2's orders */
#define	RTL_MAC_THRESHOLD		128	/* threshold for block unkown unicast */
#define	RTL_MAC_TIMEOUT			10*HZ	/* unkown unicast restriction time */
#define	RTL_MAC_REFILL_TOKEN		25000	/* per second refill token number */
typedef	struct __rtlMacRecord
{
	uint8		enable;
	uint8		cnt;
	uint8		mac[ETHER_ADDR_LEN];
	struct timer_list	timer;
}	rtlMacRecord;

static rtlMacRecord	macRecord[RTL_MAC_RECORD_NUM];
static uint32	macRecordIdx;
static uint8	lanIfName[NETIF_NUMBER];
static void	rtl_unkownUnicastUpdate(uint8 *mac);
static void rtl_unkownUnicastTimer(unsigned long data);
#endif

#if defined(DYNAMIC_ADJUST_TASKLET) || defined(CONFIG_RTL8186_TR) || defined(BR_SHORTCUT)
static void one_sec_timer(unsigned long task_priv);
#endif

#if 0
#define MAX_PRE_ALLOC_RX_SKB		256
#define NUM_RX_PKTHDR_DESC		512
#define NUM_TX_PKTHDR_DESC		1024
#else
	#ifdef DELAY_REFILL_ETH_RX_BUF
	#define MAX_PRE_ALLOC_RX_SKB	64
	#else
	#define MAX_PRE_ALLOC_RX_SKB	160
	#endif
#define NUM_RX_PKTHDR_DESC		256
#define NUM_TX_PKTHDR_DESC		512
#if defined(RTL_OUTPUTQUEUE_SUPPORT)
#define NUM_TX_PKTHDR_DESC1		64
#else
#define NUM_TX_PKTHDR_DESC1		0
#endif
#endif

#ifdef CONFIG_RTL865X_ETH_PRIV_SKB
static struct sk_buff *dev_alloc_skb_priv_eth(unsigned int size);
static void init_priv_eth_skb_buf(void);
#endif

struct ring_que {
	int qlen;
	int qmax;	
	int head;
	int tail;
	struct sk_buff **ring;
};
__DRAM_FWD static struct ring_que rx_skb_queue;
int skb_num=0;

#ifdef CONFIG_RTL865X_LAYERED_DRIVER
	int32 rtl865x_init(void);
	int32 rtl865x_config(struct rtl865x_vlanConfig vlanconfig[]);
#else
enum RTL_RESULT rtl865x_init(void);
enum RTL_RESULT rtl865x_config(struct rtl865x_vlanConfig vlanconfig[]);
#endif

/* These identify the driver base version and may not be removed. */
MODULE_DESCRIPTION("RealTek RTL-8650 series 10/100 Ethernet driver");
MODULE_LICENSE("GPL");

#ifdef CONFIG_DEFAULTS_KERNEL_2_6
static char* multicast_filter_limit = "maximum number of filtered multicast addresses";
module_param (multicast_filter_limit,charp, S_IRUGO);
#else
/* Maximum number of multicast addresses to filter (vs. Rx-all-multicast).
   The RTL chips use a 64 element hash table based on the Ethernet CRC.  */
MODULE_PARM (multicast_filter_limit, "i");
MODULE_PARM_DESC (multicast_filter_limit, "maximum number of filtered multicast addresses");
#endif

#define DRV_NAME		"re865x_nic"
#define PFX			DRV_NAME ": "
#define DRV_VERSION		"0.1"
#define TX_TIMEOUT		(10*HZ)
#define BDINFO_ADDR 0xbe3fc000

#define	LOCK_RX_TX		1

#if	defined(LOCK_RX_TX)
static DEFINE_SPINLOCK(rtl_nic_spinlock);
#define	RTL_NIC_LOCK_INIT(__rtl_lock__)		spin_lock_init(&__rtl_lock__)
#define	RTL_NIC_LOCK(__rtl_lock__)			spin_lock(&__rtl_lock__)
#define	RTL_NIC_UNLOCK(__rtl_lock__)		spin_unlock(&__rtl_lock__)
#endif

#define NEXT_DEV(cp)			(cp->dev_next ? cp->dev_next : cp->dev_prev)
#define NEXT_CP(cp)			((struct dev_priv *)NEXT_DEV(cp)->priv)
#define IS_FIRST_DEV(cp)	(NEXT_CP(cp)->opened ? 0 : 1)
#define GET_IRQ_OWNER(cp) (cp->irq_owner ? cp->dev : NEXT_DEV(cp))

#define MAX_PORT_NUM 9
#ifdef RTL865X_MULTIPLE_RX_RING
#define NUM_RX_PKTHDR_DESC5 32
#else
#define NUM_RX_PKTHDR_DESC5 0
#endif

static unsigned int rxRingSize[RTL865X_SWNIC_RXRING_MAX_PKTDESC] = {NUM_RX_PKTHDR_DESC, 0, 0, 0, 0, NUM_RX_PKTHDR_DESC5};
static unsigned int txRingSize[RTL865X_SWNIC_TXRING_MAX_PKTDESC] = {NUM_TX_PKTHDR_DESC, NUM_TX_PKTHDR_DESC1};

#if defined(CONFIG_RTK_VLAN_SUPPORT)
#define ETH_INTF_NUM	5
#else
#define ETH_INTF_NUM	2
#endif

#ifdef CONFIG_RTL865X_LAYERED_DRIVER
static struct rtl865x_vlanConfig vlanconfig[] = {
/*      	ifName  W/L      If type		VID	 FID	   Member Port	UntagSet		mtu		MAC Addr	is_slave								*/
/*		=====  ===   =======	===	 ===   =========   =======	====	====================================	*/

#ifdef CONFIG_BRIDGE
//Hyking: so strange!!
//eth0<--->physical port 4
//eth4<--->physical port 1
//just for sync with 2.4 -_ -!
#if defined(CONFIG_RTK_VLAN_SUPPORT)
	{ 	"br0",	 0,   IF_ETHER, 	RTL_LANVLANID, 	   0, 		RTL_LANPORT_MASK_4,		RTL_LANPORT_MASK_4,		1500, 	{ { 0x00, 0x12, 0x34, 0x56, 0x78, 0x90 } }, 0	},
#else
	{ 	"br0",	 0,   IF_ETHER, 	RTL_LANVLANID, 	   0, 		RTL_LANPORT_MASK,		RTL_LANPORT_MASK,		1500, 	{ { 0x00, 0x12, 0x34, 0x56, 0x78, 0x90 } }, 0	},
#endif
	{	"eth1",	 1,   IF_ETHER,		RTL_WANVLANID,	   0,		RTL_WANPORT_MASK,		RTL_WANPORT_MASK,		1500,	{ { 0x00, 0x12, 0x34, 0x56, 0x78, 0x91 } }, 0	},
	//{ 	"eth0",	 0,   IF_NONE, 	RTL_LANVLANID, 	   0, 		RTL_LANPORT_MASK, 		RTL_LANPORT_MASK,		1500, 	{ { 0x00, 0x12, 0x34, 0x56, 0x78, 0x90 } }, 1	},
#else
	{ 	"eth0",	 0,   IF_ETHER, 	RTL_LANVLANID, 	   0, 		RTL_LANPORT_MASK, 		RTL_LANPORT_MASK,		1500, 	{ { 0x00, 0x12, 0x34, 0x56, 0x78, 0x90 } }, 0	},
	{	"eth1",	 1,   IF_ETHER,		RTL_WANVLANID,	   0,		RTL_WANPORT_MASK,		RTL_WANPORT_MASK,		1500,	{ { 0x00, 0x12, 0x34, 0x56, 0x78, 0x91 } }, 0	},
#endif	
#if defined(CONFIG_RTK_VLAN_SUPPORT)
	{	"eth2",	0,   IF_ETHER, 		RTL_LANVLANID_2, 	0,		RTL_LANPORT_MASK_1, 	RTL_LANPORT_MASK_1,		1500, 	{ { 0x00, 0x12, 0x34, 0x56, 0x78, 0x92 } }, 0	},
	{	"eth3", 	0,   IF_ETHER, 	RTL_LANVLANID_3, 	0, 		RTL_LANPORT_MASK_2, 	RTL_LANPORT_MASK_2,		1500, 	{ { 0x00, 0x12, 0x34, 0x56, 0x78, 0x93 } }, 0	},
	{	"eth4", 	0,   IF_ETHER, 	RTL_LANVLANID_4, 	0, 		RTL_LANPORT_MASK_3, 	RTL_LANPORT_MASK_3,		1500, 	{ { 0x00, 0x12, 0x34, 0x56, 0x78, 0x94 } }, 0	},
#endif
	{	"ppp0", 1,   IF_PPPOE,    	RTL_WANVLANID,		0,    	RTL_WANPORT_MASK,		RTL_WANPORT_MASK,		1500, 	{ { 0x00, 0x12, 0x34, 0x56, 0x78, 0x91 } }, 1 },
	RTL865X_CONFIG_END,
};

#else

static struct rtl865x_vlanConfig vlanconfig[] = {
/*      	ifName  W/L      If type		VID	 FID	   Member Port	UntagSet		mtu		MAC Addr									*/
/*		=====  ===   =======	===	 ===   =========   =======	====	====================================	*/

#ifdef CONFIG_BRIDGE
	{ 	"br0",	 0,   IF_ETHER, 	RTL_LANVLANID, 	   0, 		RTL_LANPORT_MASK, 		RTL_LANPORT_MASK,		1500, 	{ { 0x00, 0x12, 0x34, 0x56, 0x78, 0x90 } }	},
	{	"eth1",	 1,   IF_ETHER,		RTL_WANVLANID,	   0,		RTL_WANPORT_MASK,		RTL_WANPORT_MASK,		1500,	{ { 0x00, 0x12, 0x34, 0x56, 0x78, 0x91 } }	},
#else
	{ 	"eth0",	 0,   IF_ETHER, 	RTL_LANVLANID, 	   0, 		RTL_LANPORT_MASK, 		RTL_LANPORT_MASK,		1500, 	{ { 0x00, 0x12, 0x34, 0x56, 0x78, 0x90 } }	},
	{	"eth1",	 1,   IF_ETHER,		RTL_WANVLANID,	   0,		RTL_WANPORT_MASK,		RTL_WANPORT_MASK,		1500,	{ { 0x00, 0x12, 0x34, 0x56, 0x78, 0x91 } }	},
#endif
	{	"ppp0", 1,   IF_PPPOE,    		RTL_WANVLANID,    0,    RTL_WANPORT_MASK,         RTL_WANPORT_MASK,     1500, { { 0x00, 0x12, 0x34, 0x56, 0x78, 0x91 } } },
	LRCONFIG_END,
};

#endif

#if defined(CONFIG_PROC_FS) && defined(CONFIG_NET_QOS) && defined(CONFIG_RTL865X_LAYERED_DRIVER)
static uint8	netIfName[NETIF_NUMBER][IFNAMSIZ] = {{0}};
#endif

#ifdef CONFIG_RTL865X_ETH_PRIV_SKB
	#ifdef CONFIG_RTL8196B
		#ifdef DELAY_REFILL_ETH_RX_BUF
		#define MAX_ETH_SKB_NUM	(NUM_RX_PKTHDR_DESC + MAX_PRE_ALLOC_RX_SKB + 400)
		#else
		#define MAX_ETH_SKB_NUM	(NUM_RX_PKTHDR_DESC + MAX_PRE_ALLOC_RX_SKB + 400)
		#endif

	#else
	#define MAX_ETH_SKB_NUM	(NUM_RX_PKTHDR_DESC + MAX_PRE_ALLOC_RX_SKB + 256)
	#endif

#define ETH_SKB_BUF_SIZE	(CROSS_LAN_MBUF_LEN+sizeof(struct skb_shared_info)+160)
//#define ETH_SKB_BUF_SIZE	2048
#define ETH_MAGIC_CODE		"865x"

/*	The following structure's field orders was arranged for special purpose, 
	it should NOT be modify	*/
struct priv_skb_buf2 {
	unsigned char magic[4];
	struct list_head	list;	
	unsigned char buf[ETH_SKB_BUF_SIZE];
};

static struct priv_skb_buf2 eth_skb_buf[MAX_ETH_SKB_NUM+1];
static struct list_head eth_skbbuf_list;

int eth_skb_free_num=MAX_ETH_SKB_NUM;
extern struct sk_buff *dev_alloc_8190_skb(unsigned char *data, int size);
#endif

#if	defined(CONFIG_RTL865X_EXTPORT)
struct net_device *cachedWlanDev = NULL;
#endif

int rtl865x_curOpMode=GATEWAY_MODE;


 struct re865x_priv
{
	u16			ready;
	u16			addIF;
	u16			devnum;
	u32			sec_count;
	u32			sec;
#if defined(CONFIG_RTK_VLAN_SUPPORT)
	struct	net_device	*dev[ETH_INTF_NUM];
#else
	struct	net_device	*dev[NETIF_NUMBER];
#endif
#ifdef CONFIG_RTL_STP
	struct	net_device	*stp_port[MAX_RE865X_STP_PORT];
#endif
	spinlock_t		lock;
	void			*regs;
	struct tasklet_struct	rx_tasklet;
	struct timer_list timer;	/* Media monitoring timer. */
	unsigned long		linkchg;	
};

__DRAM_FWD static  struct re865x_priv _rtl86xx_dev; 

struct dev_priv {
	u32			id;            /* VLAN id, not vlan index */
	u32			portmask;     /* member port mask */
	u32			portnum;     	/* number of member ports */
	u32			netinit;
	struct net_device	*dev;
	struct net_device   *dev_prev;
	struct net_device   *dev_next;
#ifdef RX_TASKLET
	struct tasklet_struct   rx_dsr_tasklet;
#endif

#ifdef TX_TASKLET
	struct tasklet_struct   tx_dsr_tasklet;
#endif

#ifdef CONFIG_RTK_VLAN_SUPPORT
	struct vlan_info	vlan_setting;
#endif
#ifdef CONFIG_RTL865X_IGMP_SNOOPING
	u32	igmpModuleIndex;
#endif
	//struct re865x_priv	*priv;
#ifdef CP_VLAN_TAG_USED
	struct vlan_group	*vlgrp;
#endif
	spinlock_t			lock;
	u32			msg_enable;
	u32 			opened;
	u32			irq_owner; //record which dev request IRQ
	struct net_device_stats net_stats;
#if defined(DYNAMIC_ADJUST_TASKLET) || defined(CONFIG_RTL8186_TR) || defined(BR_SHORTCUT)
    struct timer_list expire_timer; 
#endif

#if 0
	int			ip;
	int			mask;
	int			dns;
	int			pppoe_status;
	int			gateway;
	int			setup;
//	struct mii_if_info	mii_if;
#endif
};

#ifdef CONFIG_RTL_STP
static unsigned char STPmac[] = { 1, 0x80, 0xc2, 0,0,0};
#ifdef CONFIG_RTK_MESH
int8 STP_PortDev_Mapping[MAX_RE865X_STP_PORT] ={NO_MAPPING, NO_MAPPING, NO_MAPPING, NO_MAPPING, NO_MAPPING, WLAN_PSEUDO_IF_INDEX, WLAN_MESH_PSEUDO_IF_INDEX};
#else
int8 STP_PortDev_Mapping[MAX_RE865X_STP_PORT] ={NO_MAPPING, NO_MAPPING, NO_MAPPING, NO_MAPPING, NO_MAPPING, WLAN_PSEUDO_IF_INDEX};
#endif
static int re865x_stp_get_pseudodevno(uint32 port_num);
#endif

static int re865x_ioctl (struct net_device *dev, struct ifreq *rq, int cmd);

#ifdef CONFIG_RTK_VLAN_SUPPORT
static int read_proc_vlan(char *page, char **start, off_t off,int count, int *eof, void *data);
static int write_proc_vlan(struct file *file, const char *buffer,unsigned long count, void *data);

static int32 rtk_vlan_support_read( char *page, char **start, off_t off, int count, int *eof, void *data );
static int32 rtk_vlan_support_write( struct file *filp, const char *buff,unsigned long len, void *data );	
__DRAM_FWD int rtk_vlan_support_enable;

//static int rtk_vlan_netmask_read(char *page, char **start, off_t off,int count, int *eof, void *data);
//static int rtk_vlan_netmask_write(struct file *file, const char *buffer,unsigned long count, void *data);

static int read_vlan_num(char *page, char **start, off_t off,int count, int *eof, void *data);
static int write_vlan_num(struct file *file, const char *buffer,unsigned long count, void *data);
int vlan_num;
#endif

extern void br_signal_igmpProxy(void);

/* Set or clear the multicast filter for this adaptor.
 * This routine is not state sensitive and need not be SMP locked.
 */
static void re865x_set_rx_mode (struct net_device *dev){
	unsigned long flags;
	spin_lock_irqsave (&_rtl86xx_dev.lock, flags);
	//Not yet implemented.
	spin_unlock_irqrestore (&_rtl86xx_dev.lock, flags);
}

static struct net_device_stats *re865x_get_stats(struct net_device *dev){
	struct dev_priv  *dp = dev->priv;
	return &dp->net_stats;
}

static void rtl865x_disableDevPortForward(struct net_device *dev, struct dev_priv *cp)
{
	int port;
	for(port=0;port<RTL8651_AGGREGATOR_NUMBER;port++)
	{
		if((1<<port) & cp->portmask)
		{
#if 1
//			WRITE_MEM32(PCRP0+(port<<2), ((READ_MEM32(PCRP0+(port<<2)))&(~EnablePHYIf))
			REG32(PCRP0+(port<<2))= ((REG32(PCRP0+(port<<2)))&(~EnablePHYIf));
#else
			rtl865xC_setAsicEthernetForceModeRegs(port, TRUE, FALSE, 1, TRUE);
#endif
		}
	}
	
}

static void rtl865x_enableDevPortForward(struct net_device *dev, struct dev_priv *cp)
{
	int port;
	for(port=0;port<RTL8651_AGGREGATOR_NUMBER;port++)
	{
		if((1<<port) & cp->portmask)
		{
#if 1
			REG32(PCRP0+(port<<2))= ((REG32(PCRP0+(port<<2)))|(EnablePHYIf));
#else
			rtl865xC_setAsicEthernetForceModeRegs(port, FALSE, TRUE, 1, TRUE);
			rtl8651_restartAsicEthernetPHYNway(port);
#endif
		}
	}
	
}

static void rtl865x_disableInterrupt(void)
{
	REG32(CPUICR) = 0; 
	REG32(CPUIIMR) = 0;      
}

static void rtk_queue_init(struct ring_que *que)
{
#if	defined(LOCK_RX_TX)
	RTL_NIC_LOCK(rtl_nic_spinlock);
#endif
	memset(que, 0, sizeof(struct ring_que));
	que->ring = (struct sk_buff **)kmalloc(
		(sizeof(struct skb_buff*)*(rtl865x_maxPreAllocRxSkb+1))
		,GFP_KERNEL);
	memset(que->ring, 0, (sizeof(struct sk_buff *))*(rtl865x_maxPreAllocRxSkb+1));
	que->qmax = rtl865x_maxPreAllocRxSkb;	
#if	defined(LOCK_RX_TX)
	RTL_NIC_UNLOCK(rtl_nic_spinlock);
#endif
}

static int rtk_queue_tail(struct ring_que *que, struct sk_buff *skb)
{
	int next;
	uint32 flags;

	local_irq_save(flags);

#if	defined(LOCK_RX_TX)
	RTL_NIC_LOCK(rtl_nic_spinlock);
#endif
	if (que->head == que->qmax)
		next = 0;
	else
		next = que->head + 1;
	
	if (que->qlen >= que->qmax || next == que->tail) {
#if	defined(LOCK_RX_TX)
		RTL_NIC_UNLOCK(rtl_nic_spinlock);
#endif
		local_irq_restore(flags);							
		return 0;
	}	
	
	que->ring[que->head] = skb;
	que->head = next;
	que->qlen++;

#if	defined(LOCK_RX_TX)
	RTL_NIC_UNLOCK(rtl_nic_spinlock);
#endif
	local_irq_restore(flags);					
	
	return 1;
}

static struct sk_buff *rtk_dequeue(struct ring_que *que)
{
	struct sk_buff *skb;
	uint32 flags;

	local_irq_save(flags);
#if	defined(LOCK_RX_TX)
	RTL_NIC_LOCK(rtl_nic_spinlock);
#endif

	if (que->qlen <= 0 || que->tail == que->head) {
#if	defined(LOCK_RX_TX)
		RTL_NIC_UNLOCK(rtl_nic_spinlock);
#endif
		local_irq_restore(flags);							
		return NULL;
	}

	skb = que->ring[que->tail];
		
	if (que->tail == que->qmax)
		que->tail  = 0;
	else
		que->tail++;

	que->qlen--;	

#if	defined(LOCK_RX_TX)
	RTL_NIC_UNLOCK(rtl_nic_spinlock);
#endif
	local_irq_restore(flags);					
	
	return (struct sk_buff *)skb;
}

__MIPS16
__IRAM_FWD
static void refill_rx_skb(void)
{
	struct sk_buff *skb;

	while (rx_skb_queue.qlen < rtl865x_maxPreAllocRxSkb) {
#ifdef CONFIG_RTL865X_ETH_PRIV_SKB
		skb = dev_alloc_skb_priv_eth(CROSS_LAN_MBUF_LEN);
#else
		skb = dev_alloc_skb(CROSS_LAN_MBUF_LEN);
#endif

		if (skb == NULL) { 
			DEBUG_ERR("EthDrv: dev_alloc_skb() failed!\n");		
			return;
		}
		skb_reserve(skb, RX_OFFSET);	
#ifdef RTK_QUE
		rtk_queue_tail(&rx_skb_queue, skb);
#else		
		__skb_queue_tail(&rx_skb_queue, skb);
#endif
	}
}

//---------------------------------------------------------------------------
static void free_rx_skb(void)
{
	struct sk_buff *skb;

	swNic_freeRxBuf();

	while  (rx_skb_queue.qlen > 0) {
#ifdef RTK_QUE
		skb = rtk_dequeue(&rx_skb_queue);
#else
		skb = __skb_dequeue(&rx_skb_queue);
#endif
		dev_kfree_skb_any(skb);
	}
}

//---------------------------------------------------------------------------
void *UNCACHED_MALLOC(int size) 
{
	return ((void *)(((uint32)kmalloc(size, GFP_KERNEL)) | UNCACHE_MASK));	
}

//---------------------------------------------------------------------------
unsigned char *alloc_rx_buf(void **skb, int buflen)
{
	struct sk_buff *new_skb;

	if (rx_skb_queue.qlen == 0) {
#ifdef CONFIG_RTL865X_ETH_PRIV_SKB
		new_skb = dev_alloc_skb_priv_eth(CROSS_LAN_MBUF_LEN);
#else        
		new_skb = dev_alloc_skb(CROSS_LAN_MBUF_LEN);
#endif
		if (new_skb == NULL) { 
			DEBUG_ERR("EthDrv: dev_alloc_skb() failed!\n");		
		}
		else 
			skb_reserve(new_skb, RX_OFFSET);
	}
	else {
#ifdef RTK_QUE
		new_skb = rtk_dequeue(&rx_skb_queue);
#else
		new_skb = __skb_dequeue(&rx_skb_queue);
#endif
	}
	
	if (new_skb == NULL) 
		return NULL;	
	*skb = new_skb;

	#ifdef RTL865X_RX_RUNOUT_BUG
	/* store the skb pointer in a DW in front of  new_skb->data, it will be used in swNic_receive() */
	*(uint32 *)(new_skb->data-6) = (uint32)(new_skb);
	#endif

	return new_skb->data;		
}

//---------------------------------------------------------------------------
void free_rx_buf(void *skb)
{
	dev_kfree_skb_any((struct sk_buff *)skb);
}

//---------------------------------------------------------------------------
void tx_done_callback(void *skb)
{
//	dev_kfree_skb_irq((struct sk_buff *)skb);	
	dev_kfree_skb_any((struct sk_buff *)skb);
}

#if CONFIG_RTL_DYNAMIC_IRAM_MAPPING_FOR_WAPI
extern int32 (*p_swNic_send)(void *skb, void * output, uint32 len,rtl_nicTx_info *nicTx);
extern int32 (*p_swNic_receive)(void** input, uint32* pLen, unsigned int *vid, unsigned int *pid);
extern int (*p_interrupt_isr)(int irq, void *dev_instance);
extern int32 (*p_swNic_send)(void *skb, void * output, uint32 len,rtl_nicTx_info *nicTx);
#endif

#ifdef CONFIG_RTL865X_IGMP_SNOOPING				
static inline struct iphdr * re865x_getIpv4Header(uint8 *macFrame)
{
	uint8 *ptr;
	struct iphdr *iph=NULL;
	
	ptr=macFrame+12;
	if(*(int16 *)(ptr)==(int16)htons(0x8100))
	{
		ptr=ptr+4;
	}

	/*it's not ipv4 packet*/
	if(*(int16 *)(ptr)!=(int16)htons(0x0800))
	{
		return NULL;
	}
	
	iph=(struct iphdr *)(ptr+2);

	return iph;
}

static inline void re865x_relayTrappedMCast(struct sk_buff *skb, unsigned int vid, unsigned int mcastFwdPortMask, unsigned int keepOrigSkb)
{
	rtl_nicTx_info	nicTx;
 	struct sk_buff *skb2=NULL;
	if(keepOrigSkb==TRUE)
	{
		skb2= skb_clone(skb, GFP_ATOMIC);
	}
	else
	{
		skb2=skb;
	}
	
       if(skb2!=NULL)
       {
		nicTx.vid = vid; 
		nicTx.portlist = mcastFwdPortMask;
		nicTx.srcExtPort = 0;
		nicTx.flags = (PKTHDR_USED|PKT_OUTGOING);
#if CONFIG_RTL_DYNAMIC_IRAM_MAPPING_FOR_WAPI
		#if defined(CONFIG_RTL_DYNAMIC_IRAM_MAPPING_FOR_WAPI_DEBUG)
		FUNCTION_CHECK(p_swNic_send);		
		#endif
		if(p_swNic_send((void *)skb2, skb2->data, skb2->len, &nicTx) < 0)
#else
		if (swNic_send((void *)skb2, skb2->data, skb2->len, &nicTx) < 0) 
#endif
		{
			dev_kfree_skb_any(skb2);			
		}
	
	}
	return;	
}
#endif

int bridgemode;// for ADSL get log and upgrade FW 
__MIPS16
__IRAM_FWD
static void interrupt_dsr_rx(unsigned long task_priv)
{
	struct dev_priv *cp = (struct dev_priv *)task_priv;
	struct dev_priv *cp_next;
	struct dev_priv *cp_this;
	struct sk_buff *skb;
#if defined(CONFIG_RTK_VLAN_SUPPORT)
	int i;
#endif
 	int len;
 	unsigned int vid,pid;
 	unsigned rx_work = rtl865x_rxSkbPktHdrDescNum;
#ifdef BR_SHORTCUT
	extern struct net_device *get_shortcut_dev(unsigned char *da);
	struct net_device *dev = NULL;
#endif	
#ifdef CONFIG_RTL_STP
	int32 dev_no;
#endif
	cp_next = NEXT_CP(cp);
	while (rx_work--) 
	{
#if CONFIG_RTL_DYNAMIC_IRAM_MAPPING_FOR_WAPI
	#if defined(CONFIG_RTL_DYNAMIC_IRAM_MAPPING_FOR_WAPI_DEBUG)
		FUNCTION_CHECK(p_swNic_receive);
	#endif
		if(p_swNic_receive((void **)&skb, &len, &vid, &pid) != 0)
#else		
		if (swNic_receive((void **)&skb, &len, &vid, &pid) !=  0)
#endif
	        	break;

		skb->tail = skb->data;
		skb->len = 0;
		skb_put(skb, len);

#if	defined(CONFIG_RTL8196_RTL8366) || defined(CONFIG_RTL_QOS_8021P_SUPPORT)
		if (*((unsigned short *)(skb->data+(ETH_ALEN<<1))) == __constant_htons(ETH_P_8021Q)) 
		{
			uint8 *data = skb->data;

			vid = *((unsigned short *)(data+(ETH_ALEN<<1)+2));
#if	defined(CONFIG_RTL_QOS_8021P_SUPPORT)
			skb->srcVlanPriority = (vid>>13)&0x7;
#endif
			vid &= 0x0fff;

			memmove(data + VLAN_HLEN, data, VLAN_ETH_ALEN<<1);
			skb->len -= 4;
			skb->data += 4;			
			//skb_pull_rcsum(skb, VLAN_HLEN);
		}
  #if	defined(CONFIG_RTL8196_RTL8366)
		else
		{
			dev_kfree_skb_any(skb);
			continue;
		}
  #elif defined(CONFIG_RTL_QOS_8021P_SUPPORT)
  		else
  		{
  			skb->srcVlanPriority = 0;
  		}
  #endif
#endif


#if defined( CONFIG_RTL865X_HARDWARE_MULTICAST) || defined(CONFIG_RTL865X_LANPORT_RESTRICTION)
		if(vid!=PKTHDR_EXTPORT_MAGIC)
		{
			skb->srcPort=(uint16)pid;
			skb->srcVlanId=(uint16)vid;	
		}
		else
		{
			skb->srcPort=0xFFFF;
			skb->srcVlanId=0;
		}
#endif
		DEBUG_ERR("RX from port 0x%x, vid 0x%x.\n", pid, vid);
		DEBUG_ERR("RX skb 0x%p\n", skb);
		DEBUG_ERR("RX data 0x%p\n", skb->data);
		DEBUG_ERR("%x:%x:%x:%x:%x:%x ==> %x:%x:%x:%x:%x:%x \n", 
			skb->data[6], skb->data[7], skb->data[8],
			skb->data[9], skb->data[10], skb->data[11], 
			skb->data[0], skb->data[1], skb->data[2], 
			skb->data[3], skb->data[4], skb->data[5]);
		DEBUG_ERR("EtherType: 0x%x\n", *((uint16*)&skb->data[12]));
		
#if defined(CONFIG_TRENDCHIP_ADSL_VLAN)
		if(*((uint16*)&skb->data[12])==0x8100) //Jase add,remove vlan tag
		{
			int i;
			for(i=11;i>=0;i--)
			{
				skb->data[i+4]=skb->data[i];
			}
			skb->data+=4;
			skb->len-=4;
		}
#endif

		{
						
/*************************begin special device decision process*****************************************/
			/*	extension port process	*/
#if	defined(CONFIG_RTL865X_EXTPORT)		
			if (pid==PKTHDR_EXTPORT_P1)
			{
				if (vid == PKTHDR_EXTPORT_MAGIC)
				{
					skb->dev = cachedWlanDev;
					cachedWlanDev->hard_start_xmit(skb, cachedWlanDev);
					continue;
				}
				else
				{
					cachedWlanDev->last_rx = jiffies;
					skb->dev = cachedWlanDev;
				}
			}
			else
#endif
#ifdef CONFIG_RTL865X_HW_PPTPL2TP
			if (pid==PKTHDR_EXTPORT_P2)
			{
				if(vid == PKTHDR_EXTPORT_MAGIC)
				{
					memset(skb->cb, 'P', 3);
				}
				skb->dev = pptpAccInfo.pppDev;
			} 
			else if (vid==RTL_PPTPL2TP_VLANID&&((1<<pid)&vlanconfig[1].memPort))
			{
				memset(skb->cb, 'N', 3);
//				skb->cb[0] = 'N';
//				skb->cb[1] = 'A';
//				skb->cb[2] = 'T';
				skb->dev = pptpAccInfo.wanDev;
			}
			else
#endif
#ifdef CONFIG_RTL_STP
			if ((skb->data[0]&0x01) && !memcmp(&(skb->data[0]), STPmac, 5) && !(skb->data[5] & 0xF0))
			{
				/* It's a BPDU */
				dev_no = re865x_stp_get_pseudodevno(pid);
				if (dev_no != NO_MAPPING)
				{
					skb->dev = _rtl86xx_dev.stp_port[dev_no];
					goto RxToPs;
	/*				printk("receieved BPDU packet in nic form %s\n", skb->dev->name);	*/
				}
				else
				{
					dev_kfree_skb_any(skb);
					continue;
				}
			}
			else
#endif		/*****************end of special device decision ***************************************************/
			{	/*warning don't remove this braces*/
		
//normal device desicion, according to pid, decide net_device...
#if defined(CONFIG_RTK_VLAN_SUPPORT)
if(bridgemode!=1)
{
				{	
					cp_this = NULL;
					for(i = 0; i < ETH_INTF_NUM; i++)
					{
						cp = ((struct dev_priv *)_rtl86xx_dev.dev[i]->priv);
						if(cp && cp->opened && (cp->portmask & (1<<pid)))
						{
							cp_this = cp;
							break;
						}
						
					}
#ifdef CONFIG_RTL865X_HW_PPTPL2TP
					if (pid & BIT(7))
					{
						if(cp_this == NULL)
							cp_this = (struct dev_priv *)_rtl86xx_dev.dev[0]->priv;
					}
#endif
					if(cp_this == NULL)
					{
						dev_kfree_skb_any(skb);
						continue;
					}			

				#if defined(CONFIG_RTK_VLAN_SUPPORT)&& defined(CONFIG_TRENDCHIP_ADSL_VLAN)
					rtk_vlan_support_enable=0;
				#endif
					if (rtk_vlan_support_enable && cp_this->vlan_setting.global_vlan) 
					{
						if (rx_vlan_process(cp_this->dev, &cp_this->vlan_setting, skb))
						{
							cp_this->net_stats.rx_dropped++;
							dev_kfree_skb_any(skb);		
							continue;
						}
					}
				}
}
else
{
	#ifdef CONFIG_RTL865X_HARDWARE_NAT
#if	defined(CONFIG_RTL8196_RTL8366)
					DEBUG_ERR("cp[%s][%d]-open[%d]\n", cp->dev->name, cp->opened, cp->id);
					DEBUG_ERR("cp[%s][%d]-open[%d]\n", cp_next->dev->name, cp_next->opened, cp_next->id);
					if (cp->opened && (cp->id==vid))
			    		{
			       	 		cp_this = cp;
			    		}
					else if (cp_next->opened && (cp_next->id==vid))
					{
						cp_this = cp_next;
					}
#else/*else of (CONFIG_RTK_VLAN_SUPPORT)*/
					/* for the issue: WAN port PC can not ping br0 (192.168.1.254) in Bridge mode */
					if (cp->opened && (cp->portmask & 1<<pid))
			    		{
			       	 		cp_this = cp;
			    		}
					else if (cp_next->opened && (cp_next->portmask & 1<<pid))
					{
						cp_this = cp_next;
					}
					#ifdef CONFIG_RTL865X_HW_PPTPL2TP
					else if (pid & BIT(7))
					{
						cp_this = cp;
					}
					#endif
#endif
				    	else
					{
						dev_kfree_skb_any(skb);
						continue;
			    		}
#else //CONFIG_RTL865X_HARDWARE_NAT
			    		if (cp->opened && cp->id==vid)				
			       	 		cp_this = cp;
					else if (cp_next->opened && cp_next->id==vid)
						cp_this = cp_next;
				    	else {    
							dev_kfree_skb_any(skb);
						continue;
			    		}
#endif //CONFIG_RTL865X_HARDWARE_NAT
}
#else/*else of (CONFIG_RTK_VLAN_SUPPORT)*/
#ifdef CONFIG_RTL865X_HARDWARE_NAT
#if	defined(CONFIG_RTL8196_RTL8366)
					DEBUG_ERR("cp[%s][%d]-open[%d]\n", cp->dev->name, cp->opened, cp->id);
					DEBUG_ERR("cp[%s][%d]-open[%d]\n", cp_next->dev->name, cp_next->opened, cp_next->id);
					if (cp->opened && (cp->id==vid))
			    		{
			       	 		cp_this = cp;
			    		}
					else if (cp_next->opened && (cp_next->id==vid))
					{
						cp_this = cp_next;
					}
#else
					/* for the issue: WAN port PC can not ping br0 (192.168.1.254) in Bridge mode */
					if (cp->opened && (cp->portmask & 1<<pid))
			    		{
			       	 		cp_this = cp;
			    		}
					else if (cp_next->opened && (cp_next->portmask & 1<<pid))
					{
						cp_this = cp_next;
					}
					#ifdef CONFIG_RTL865X_HW_PPTPL2TP
					else if (pid & BIT(7))
					{
						cp_this = cp;
					}
					#endif
#endif
				    	else
					{
						dev_kfree_skb_any(skb);
						continue;
			    		}
#else //CONFIG_RTL865X_HARDWARE_NAT
			    		if (cp->opened && cp->id==vid)				
			       	 		cp_this = cp;
					else if (cp_next->opened && cp_next->id==vid)
						cp_this = cp_next;
				    	else {    
							dev_kfree_skb_any(skb);
						continue;
			    		}
#endif //CONFIG_RTL865X_HARDWARE_NAT
				
#endif //defined(CONFIG_RTK_VLAN_SUPPORT)

DEBUG_ERR("open1 %d, portmask 0x%x.\n", cp->opened, cp->portmask);
DEBUG_ERR("open2 %d, portmask 0x%x.\n", cp_next->opened, cp_next->portmask);
	
				/*	multicast snooping process	*/
				/*	If a pkt was decided by hw to extension port, 
				  *	it can NOT be multicast pkt.
				  *	So we put the multicast process after
				  *	extention port process
				  *	20081020	-	ZhaoBo
				  */
#ifdef CONFIG_RTL865X_IGMP_SNOOPING
#ifndef CONFIG_RTL8196_RTL8366
				if(skb->data[0]&0x01)
				{

					int retVal;
					unsigned int  vlanRelayPortMask=0;
					struct iphdr *iph=NULL;
					struct rtl_multicastDataInfo multicastDataInfo;
					struct rtl_multicastFwdInfo brMCastFwdInfo;
					struct rtl_multicastFwdInfo nicMCastFwdInfo;
					//struct rtl_multicastDeviceInfo_s bridgeMCastDev;
					struct rtl_multicastDeviceInfo_s nicMCastDev;	

					if((skb->data[0]==0x01) && (skb->data[1]==0x00)&& (skb->data[2]==0x5e))
					{
						skb->dev = cp_this->dev;
						cp_this->net_stats.rx_packets++;	
						cp_this->net_stats.rx_bytes += skb->len;	
						cp_this->dev->last_rx = jiffies;
					
						strcpy(nicMCastDev.devName,cp_this->dev->name);
						if(cp_this->igmpModuleIndex!=0xFFFFFFFF)
						{
							iph=re865x_getIpv4Header(skb->data);
							if(iph!=NULL)
							{	
								/*udp or tcp packet*/
								if((iph->protocol==0x11) || (iph->protocol==0x06))
								{
									multicastDataInfo.ipVersion=4;
									multicastDataInfo.sourceIp[0]=  (uint32)(iph->saddr);
									multicastDataInfo.groupAddr[0]=  (uint32)(iph->daddr);
								
									retVal=rtl_getMulticastDataFwdInfo(brIgmpModuleIndex, &multicastDataInfo, &brMCastFwdInfo);
									//retVal=rtl_getIgmpSnoopingModuleDevInfo(brIgmpModuleIndex,&bridgeMCastDev);
	
									/*relay packets which are trapped by hardware multicast table*/
									#ifdef CONFIG_RTL865X_HARDWARE_MULTICAST
									if(rtl8651_getAsicOperationLayer()>2)
									{
										rtl_getMulticastDataFwdInfo(cp_this->igmpModuleIndex, &multicastDataInfo, &nicMCastFwdInfo);
										vlanRelayPortMask=rtl865x_getVlanPortMask(vid)& (~(1<<pid)) & nicMCastFwdInfo.fwdPortMask & ((1<<RTL8651_MAC_NUMBER)-1);
										if(vlanRelayPortMask!=0)
										{
											re865x_relayTrappedMCast( skb, vid, vlanRelayPortMask, TRUE);
										}
									}
									#if defined(CONFIG_RTK_VLAN_SUPPORT)
									if(bridgemode!=1)
									{
									if(rtk_vlan_support_enable == 0)
									{
										if((brMCastFwdInfo.unknownMCast==TRUE) && (vlanRelayPortMask==0))
										{
										
											rtl865x_blockMulticastFlow(vid, pid, multicastDataInfo.sourceIp[0], multicastDataInfo.groupAddr[0]);
											
											}
										}
									}
									else
									{
										if((brMCastFwdInfo.unknownMCast==TRUE) && (vlanRelayPortMask==0))
										{
								
											rtl865x_blockMulticastFlow(vid, pid, multicastDataInfo.sourceIp[0], multicastDataInfo.groupAddr[0]);
										
										}
									}
									#else
									if((brMCastFwdInfo.unknownMCast==TRUE) && (vlanRelayPortMask==0))
									{
								
										rtl865x_blockMulticastFlow(vid, pid, multicastDataInfo.sourceIp[0], multicastDataInfo.groupAddr[0]);
										
									}	
									#endif/*end of CONFIG_RTK_VLAN_SUPPORT*/
									#endif/*end of CONFIG_RTL865X_HARDWARE_MULTICAST*/
								}
								else if(iph->protocol==0x02) 
								{
									/*igmp packet*/
									rtl_igmpMldProcess(cp_this->igmpModuleIndex, skb->data, pid, &vlanRelayPortMask);
									vlanRelayPortMask=rtl865x_getVlanPortMask(vid) & vlanRelayPortMask & ((1<<RTL8651_MAC_NUMBER)-1);
									if(vlanRelayPortMask!=0)
									{
										re865x_relayTrappedMCast( skb, vid, vlanRelayPortMask, TRUE);
									}
								
								}
								else
								{
								
									#ifdef CONFIG_RTL865X_HARDWARE_MULTICAST
									if(rtl8651_getAsicOperationLayer()>2)
									{
										vlanRelayPortMask=rtl865x_getVlanPortMask(vid) & (~(1<<pid)) & ((1<<RTL8651_MAC_NUMBER)-1);
										if(vlanRelayPortMask!=0)
										{
											re865x_relayTrappedMCast( skb, vid, vlanRelayPortMask, TRUE);
										}
								
									}
									#endif
								
								}
							}
						}
						
					}
					else
					{
						/*hardware ip multicast table only trap 0x01-00-5e-XX-XX-XX type packets*/
						/*how about it's not trapped by hardware multicast table?----> other packet we assume it has been flooded by l2 table*/
					} 

				}
#endif
#endif
				{

					skb->dev = cp_this->dev;
					cp_this->net_stats.rx_packets++;	
					cp_this->net_stats.rx_bytes += skb->len;	
					cp_this->dev->last_rx = jiffies;
#if (defined(CONFIG_RTL865X_HARDWARE_NAT) && defined(CONFIG_RTL8192SE))
					if (vid == PKTHDR_EXTPORT_MAGIC)
					{
						if ((dev = get_shortcut_dev(skb->data)) != NULL)
						{
							/*skb->dev = dev;*/ /* for performance */
							dev->hard_start_xmit(skb, dev);
							continue;
						}
					}
					else
#endif
					if  (memcmp(&skb->data[6], cp_this->dev->dev_addr, 6)== 0)// check source mac
					{
			                        //printk("source mac is device self\n");
			                        dev_kfree_skb_any(skb);
			                       continue;
					}
					if(bridgemode!=1)
					{
#if defined(CONFIG_RTK_VLAN_SUPPORT)
					//if(cp_this->vlan_setting.is_lan &&
					if(vid != RTL_WANVLANID &&
#else
					if (vid==RTL_LANVLANID &&	/* lan side */
#endif
						!(skb->data[0] & 0x01) &&	/* unicast */
						memcmp(skb->data, skb->dev->dev_addr, ETH_ALEN) /* NOT to AP it self */)
					{
#ifdef BR_SHORTCUT
						if (enable_brsc && (eth_flag & BIT(2)) &&
							((dev = get_shortcut_dev(skb->data)) != NULL))
						{
							memcpy(cached_eth_addr, &skb->data[6], 6);
							cached_dev = cp_this->dev;	
							/*skb->dev = dev;*/ /* for performance */
							dev->hard_start_xmit(skb, dev);
							continue;
							}
					#endif
					#if defined (CONFIG_RTL_UNKOWN_UNICAST_CONTROL)
						rtl_unkownUnicastUpdate(skb->data);
					#endif
						}
					}
					else
					{
						if (vid==RTL_LANVLANID &&	/* lan side */
						!(skb->data[0] & 0x01) &&	/* unicast */
						memcmp(skb->data, skb->dev->dev_addr, ETH_ALEN) /* NOT to AP it self */)
						{
						#ifdef BR_SHORTCUT
							if (enable_brsc && (eth_flag & BIT(2)) &&
							((dev = get_shortcut_dev(skb->data)) != NULL))
							{
								memcpy(cached_eth_addr, &skb->data[6], 6);
								cached_dev = cp_this->dev;	
								/*skb->dev = dev;*/ /* for performance */
								dev->hard_start_xmit(skb, dev);
								continue;
							}
						#endif
					#if defined (CONFIG_RTL_UNKOWN_UNICAST_CONTROL)
						rtl_unkownUnicastUpdate(skb->data);
					#endif
						}
					}
				}
			}
#if defined(CONFIG_RTL_STP)
RxToPs:
#endif
			{
				DEBUG_ERR("skb->dev(%s).\n", skb->dev->name);
				skb->protocol = eth_type_trans(skb, skb->dev);
				/* for ipv6 ready logo */
				if (skb->protocol == ETH_P_IP)
					skb->ip_summed = CHECKSUM_UNNECESSARY;
				else
					skb->ip_summed = CHECKSUM_NONE;
				netif_rx(skb);		
			}
	    	}
	}

#ifdef RX_TASKLET
    	REG32(CPUIIMR) |= RX_DONE_IE_ALL;
#endif
}

#define	RTL_RUNOUT_PROCESS		0x7E0806	/*(PKTHDR_DESC_RUNOUT_IP_ALL|MBUF_DESC_RUNOUT_IP_ALL|TX_ALL_DONE_IP_ALL)*/

#if 1
//---------------------------------------------------------------------------
__IRAM_FWD
static void interrupt_dsr_tx(unsigned long task_priv)
{
 //   struct dev_priv *cp = (struct dev_priv*)task_priv;
//	int free_desc_num;
	unsigned long flags;
#if	defined(RTL_OUTPUTQUEUE_SUPPORT)
	int	idx;
//	int	wakeup;
#endif

#if 0
	refill_rx_skb();
#if	defined(RTL_OUTPUTQUEUE_SUPPORT)
	wakeup = 1;	
	for(idx=0;idx<RTL865X_SWNIC_TXRING_MAX_PKTDESC;idx++)
	{
		free_desc_num = swNic_txDone(idx); 
		if (free_desc_num<txRingSize[idx]/4)
			wakeup = 0;
	}

	if (wakeup==1)
#else
	free_desc_num = swNic_txDone(); 

    if (free_desc_num >= (txRingSize[0]/32))
#endif
    {
		if (cp->opened && netif_queue_stopped(cp->dev)) 			
			netif_wake_queue(cp->dev);
		
		if (NEXT_CP(cp)->opened && netif_queue_stopped(NEXT_DEV(cp)))
			netif_wake_queue(NEXT_DEV(cp));			
   	}
#else
	local_irq_save(flags);
	refill_rx_skb();

#if	defined(RTL_OUTPUTQUEUE_SUPPORT)
	for(idx=0;idx<RTL865X_SWNIC_TXRING_MAX_PKTDESC;idx++)
	{
		swNic_txDone(idx); 
	}
#else
	swNic_txDone();
#endif
	local_irq_restore(flags);
#endif
#ifdef TX_TASKLET		
	REG32(CPUIIMR) |= RTL_RUNOUT_PROCESS;		
#endif
}
#endif


unsigned int rtl865x_getLANLinkStatus(void)
{
	unsigned int port_num=0;
	int linkPortMask=0;
#if !defined(CONFIG_RTL8196_RTL8366)
	int i;
	int tmpPortMask;
	int32 totalVlans;
#endif
	
#ifdef CONFIG_RTL8196_RTL8366
	int link_status=0;
    
	for (port_num = 0; port_num < 4; port_num++){
		rtl8366rb_getPHYLinkStatus (port_num, &link_status);//PHY4
		if (link_status==1)
		{
			linkPortMask |= 1<<port_num;
		}
	}

#else
	tmpPortMask = 0;
	totalVlans=((sizeof(vlanconfig))/(sizeof(struct rtl865x_vlanConfig)))-1;
	for(i=0;i<totalVlans;i++)
	{
		if(vlanconfig[i].isWan==FALSE)
		{
			tmpPortMask = vlanconfig[i].memPort;
			if (tmpPortMask==0)
			{	
				continue;
			}

			for(port_num=0;port_num<=RTL8651_PHY_NUMBER;port_num++)
			{
				if( (1<<port_num)&tmpPortMask )
				{
					if((READ_MEM32(PSRP0+(port_num<<2))&PortStatusLinkUp)!=0)
					{
						linkPortMask |= 1<<port_num;
					}
				}
			}
		}
	}

	
#endif

	return linkPortMask;
}
#ifdef CONFIG_RTL865X_IGMP_SNOOPING
void rtl865x_igmpSyncLinkStatus(void)
{
	unsigned int newLinkPortMask=0;
	rtl_igmpPortInfo_t portInfo;
	int i;
	struct dev_priv *cp;
	int totalVlans=0;	
	newLinkPortMask=rtl865x_getLANLinkStatus();

	
	totalVlans=((sizeof(vlanconfig))/(sizeof(struct rtl865x_vlanConfig)))-1;
	
	for(i=0;i<totalVlans;i++)
	{
		if ((IF_ETHER==vlanconfig[i].if_type) && (TRUE!=vlanconfig[i].isWan))
		{
			if(_rtl86xx_dev.dev[i]!=NULL)
			{
				cp = ((struct dev_priv *)_rtl86xx_dev.dev[i]->priv);
				portInfo.linkPortMask=newLinkPortMask & cp->portmask;
				rtl865x_igmpLinkStatusChangeCallback(cp->igmpModuleIndex, &portInfo);
			}
		}	
	
	}
	
	if((newLinkPortMask & (~curLinkPortMask))!=0)
	{	
		/*there is some port linking up*/
		/*notice igmp proxy daemon to send general query to newly link up client*/
		#ifdef CONFIG_RTL8196BU_8186SDK_MP_SPI 
		#else
		br_signal_igmpProxy();
		#endif
	}
	
	curLinkPortMask=newLinkPortMask;
	return;
			
}
#endif
__MIPS16
__IRAM_FWD
int interrupt_isr(int irq, void *dev_instance)
{
	struct net_device *dev = dev_instance;
	struct dev_priv *cp = dev->priv;
	unsigned int status;

	status =*(volatile unsigned int*)(CPUIISR); 
	*(volatile unsigned int *)(CPUIISR) = status;

#if 0
	if (!status || (status == 0xFFFF))
	{
		/*spin_unlock(&cp->lock);*/	/* should be removed */
		return IRQ_HANDLED;
	}

	/*disable rx interrupt,avoid duplicate trigger*/
	/**(volatile unsigned int *)(CPUIIMR) &= ~RX_DONE_RUNOUT_MASK;*/
#endif

	if (status & (RX_DONE_IP_ALL))
	{
#ifdef RX_TASKLET
		*(volatile unsigned int *)(CPUIIMR) &= ~RX_DONE_IE_ALL;
		tasklet_hi_schedule(&cp->rx_dsr_tasklet);
#else
		interrupt_dsr_rx((unsigned long)cp);
#endif
	}		

#if 0
	if (status &TX_ALL_DONE_IP_ALL)
	{
  #ifdef TX_TASKLET		
		*(volatile unsigned int *)(CPUIIMR) &= ~TX_ALL_DONE_IP_ALL;
		tasklet_schedule(&cp->tx_dsr_tasklet);						
  #else
		interrupt_dsr_tx((unsigned long)cp);
  #endif			
	}
#else
	if (status & RTL_RUNOUT_PROCESS)
	{
  #ifdef TX_TASKLET		
		*(volatile unsigned int *)(CPUIIMR) &= ~RTL_RUNOUT_PROCESS;
		tasklet_hi_schedule(&cp->tx_dsr_tasklet);						
  #else
		interrupt_dsr_tx((unsigned long)cp);
  #endif
	}
#endif
#if	defined(CONFIG_RTL865X_IGMP_SNOOPING)||defined(CONFIG_RTL865X_LINKCHG_PROCESS)
	if (status & LINK_CHANGE_IP)
	{
#if defined(CONFIG_RTL865X_IGMP_SNOOPING)
	rtl865x_igmpSyncLinkStatus();
#endif

#if defined(CONFIG_RTL865X_LINKCHG_PROCESS) 
		rtl865x_LinkChange_Process();
#endif
	}
#endif

	return IRQ_HANDLED;
}

#if CONFIG_RTL_DYNAMIC_IRAM_MAPPING_FOR_WAPI
__IRAM_FWD_WAPI int interrupt_isr_wapi(int irq, void *dev_instance)
{
	struct net_device *dev = dev_instance;
	struct dev_priv *cp = dev->priv;
	unsigned int status;

	status =*(volatile unsigned int*)(CPUIISR); 
	*(volatile unsigned int *)(CPUIISR) = status;

#if 0
	if (!status || (status == 0xFFFF))
	{
		/*spin_unlock(&cp->lock);*/	/* should be removed */
		return IRQ_HANDLED;
	}

	/*disable rx interrupt,avoid duplicate trigger*/
	/**(volatile unsigned int *)(CPUIIMR) &= ~RX_DONE_RUNOUT_MASK;*/
#endif

	if (status & (RX_DONE_IP_ALL))
	{
#ifdef RX_TASKLET
		*(volatile unsigned int *)(CPUIIMR) &= ~RX_DONE_IE_ALL;
		tasklet_hi_schedule(&cp->rx_dsr_tasklet);
#else
		interrupt_dsr_rx((unsigned long)cp);
#endif
	}		

#if 0
	if (status &TX_ALL_DONE_IP_ALL)
	{
  #ifdef TX_TASKLET		
		*(volatile unsigned int *)(CPUIIMR) &= ~TX_ALL_DONE_IP_ALL;
		tasklet_schedule(&cp->tx_dsr_tasklet);						
  #else
		interrupt_dsr_tx((unsigned long)cp);
  #endif			
	}
#else
	if (status & RTL_RUNOUT_PROCESS)
	{
  #ifdef TX_TASKLET		
		*(volatile unsigned int *)(CPUIIMR) &= ~RTL_RUNOUT_PROCESS;
		tasklet_hi_schedule(&cp->tx_dsr_tasklet);						
  #else
		interrupt_dsr_tx((unsigned long)cp);
  #endif
	}
#endif

#if	defined(CONFIG_RTL865X_IGMP_SNOOPING)||defined(CONFIG_RTL865X_LINKCHG_PROCESS)
	if (status & LINK_CHANGE_IP)
	{
#if defined(CONFIG_RTL865X_IGMP_SNOOPING)
	rtl865x_igmpSyncLinkStatus();
#endif

#if defined(CONFIG_RTL865X_LINKCHG_PROCESS) 
		rtl865x_LinkChange_Process();
#endif
	}
#endif

	return IRQ_HANDLED;
}
#endif


static int rtl865x_init_hw(void)
{
	unsigned int mbufRingSize = rtl865x_rxSkbPktHdrDescNum;

	/* Initialize NIC module */
	if (swNic_init(rxRingSize, mbufRingSize, txRingSize, MBUF_LEN))
	{
		printk("865x-nic: swNic_init failed!\n");            
		return FAILED;
	}
			
	return SUCCESS;	
}

#ifdef CONFIG_RTL865X_HARDWARE_NAT		
static void reset_hw_mib_counter(struct net_device *dev)
{
	int i, port;
	int32 totalVlans=((sizeof(vlanconfig))/(sizeof(struct rtl865x_vlanConfig)))-1;

	for(i=0;i<totalVlans;i++)
	{
		if (IF_ETHER!=vlanconfig[i].if_type)
		{
			continue;
		}
		if (!memcmp(vlanconfig[i].mac.octet, dev->dev_addr, 6))
		{
			for (port=0; port<RTL8651_AGGREGATOR_NUMBER; port++)
			{
				if (vlanconfig[i].memPort & (1<<port))
				   WRITE_MEM32(MIB_CONTROL, (1<<port*2) | (1<<(port*2+1)));
				return;
			}
		}
	}
}
#endif

static int re865x_open (struct net_device *dev)
{
	struct dev_priv *cp = dev->priv;
	uint32 flags;
	int rc;

	if (cp->opened)
		return SUCCESS;
	
if(bridgemode!=1)
{	
#if defined(CONFIG_RTK_VLAN_SUPPORT)
	if (dev->name[3] < '2') 
#endif
	if (!NEXT_CP(cp)->opened)
	{
		/* this is the first open dev */
		rtk_queue_init(&rx_skb_queue);  
		spin_lock_irqsave (&cp->lock, flags);
#if	defined(LOCK_RX_TX)
		RTL_NIC_LOCK_INIT(rtl_nic_spinlock);
#endif
		rc = rtl865x_init_hw();
		spin_unlock_irqrestore(&cp->lock, flags);
		if (rc) {
			printk("rtl865x_init_hw() failed!\n");
			return FAILED;
		}

#if 1
#ifdef TX_TASKLET
		tasklet_init(&cp->tx_dsr_tasklet, interrupt_dsr_tx, (unsigned long)cp);
#endif
#endif

#if CONFIG_RTL_DYNAMIC_IRAM_MAPPING_FOR_WAPI
		#if defined(CONFIG_RTL_DYNAMIC_IRAM_MAPPING_FOR_WAPI_DEBUG)
		FUNCTION_CHECK(p_interrupt_isr);
		#endif
		rc = request_irq(dev->irq, p_interrupt_isr, SA_INTERRUPT, dev->name, dev);
#else
		rc = request_irq(dev->irq, interrupt_isr, SA_INTERRUPT, dev->name, dev);
#endif
		if (rc)
		{
			printk("request_irq() error!\n");
			goto err_out_hw;
		}
		cp->irq_owner =1;
		rtl865x_start();	
	}
}
else
{
	if (!NEXT_CP(cp)->opened)
	{
		/* this is the first open dev */
		rtk_queue_init(&rx_skb_queue);  
		spin_lock_irqsave (&cp->lock, flags);
#if	defined(LOCK_RX_TX)
		RTL_NIC_LOCK_INIT(rtl_nic_spinlock);
#endif
		rc = rtl865x_init_hw();
		spin_unlock_irqrestore(&cp->lock, flags);
		if (rc) {
			printk("rtl865x_init_hw() failed!\n");
			return FAILED;
		}

#if 1
#ifdef TX_TASKLET
		tasklet_init(&cp->tx_dsr_tasklet, interrupt_dsr_tx, (unsigned long)cp);
#endif
#endif

#if CONFIG_RTL_DYNAMIC_IRAM_MAPPING_FOR_WAPI
		#if defined(CONFIG_RTL_DYNAMIC_IRAM_MAPPING_FOR_WAPI_DEBUG)
		FUNCTION_CHECK(p_interrupt_isr);
		#endif
		rc = request_irq(dev->irq, p_interrupt_isr, SA_INTERRUPT, dev->name, dev);
#else
		rc = request_irq(dev->irq, interrupt_isr, SA_INTERRUPT, dev->name, dev);
#endif
		if (rc)
		{
			printk("request_irq() error!\n");
			goto err_out_hw;
		}
		cp->irq_owner =1;
		rtl865x_start();	
	}
	}

	cp->opened = 1;

#ifdef CONFIG_RTL865X_HARDWARE_NAT	
	reset_hw_mib_counter(dev);
#endif

	netif_start_queue(dev);
	
#if defined(DYNAMIC_ADJUST_TASKLET) || defined(BR_SHORTCUT)
#ifndef CONFIG_RTL8186_TR
	if (dev->name[3] == '0') 
#endif		
	{	
		init_timer(&cp->expire_timer);
		cp->expire_timer.expires = jiffies + 100;
		cp->expire_timer.data = (unsigned long)dev;
		cp->expire_timer.function = one_sec_timer;
		mod_timer(&cp->expire_timer, jiffies + 100);		
#ifdef DYNAMIC_ADJUST_TASKLET			
		rx_cnt = 0;
#endif
#ifdef BR_SHORTCUT
		pkt_cnt = 0;
#endif
	}
#endif


#ifdef CONFIG_RTL865X_LAYERED_DRIVER
#ifdef CONFIG_RTL865X_LAYERED_DRIVER_L3
	/*FIXME_hyking: should add default route to cpu....*/
	if(rtl865x_curOpMode == GATEWAY_MODE)
		rtl865x_addRoute(0,0,0,"eth1");
#else
	#if 0
	{
		rtl865x_tblAsicDrv_routingParam_t rth;
		memset(&rth, 0, sizeof(rtl865x_tblAsicDrv_routingParam_t));
		rth.process = 0x4; /* CPU */
		rth.ipAddr	= 0;
		rth.ipMask	= 0;
		rth.vidx			= _rtl865x_getNetifIdxByVid(8);
		rth.internal            = 0;
		rtl8651_setAsicRouting(7, &rth);
	}
	#endif
#endif

#endif

	rtl865x_enableDevPortForward( dev, cp);

	return SUCCESS;
	
err_out_hw:
    rtl865x_down();
    return rc;
}

static int re865x_close (struct net_device *dev)
{
	struct dev_priv *cp = dev->priv;

	if (!cp->opened)
		return SUCCESS;

	rtl865x_disableDevPortForward(dev, cp);
	netif_stop_queue(dev);

if(bridgemode!=1)
{
#if defined(CONFIG_RTK_VLAN_SUPPORT)
	if (dev->name[3] < '2') 
#endif
	if (!NEXT_CP(cp)->opened)
	{
		/*	warning:
			1.if we don't reboot,we shouldn't hold switch core from rx/tx, otherwise there will be some problem during change operation mode
			2.only when two devices go down,can we shut down nic interrupt
			3.the interrupt will be re_enable by rtl865x_start()
		*/
		rtl865x_disableInterrupt();
		
		free_irq(dev->irq, GET_IRQ_OWNER(cp));
		((struct dev_priv *)GET_IRQ_OWNER(cp)->priv)->irq_owner = 0;

#ifdef RX_TASKLET
		tasklet_kill(&cp->rx_dsr_tasklet);	
#endif

#ifdef TX_TASKLET
		tasklet_kill(&cp->tx_dsr_tasklet);
#endif
		free_rx_skb();
	}
}
else
{
	if (!NEXT_CP(cp)->opened)
	{
		/*	warning:
			1.if we don't reboot,we shouldn't hold switch core from rx/tx, otherwise there will be some problem during change operation mode
			2.only when two devices go down,can we shut down nic interrupt
			3.the interrupt will be re_enable by rtl865x_start()
		*/
		rtl865x_disableInterrupt();
		
		free_irq(dev->irq, GET_IRQ_OWNER(cp));
		((struct dev_priv *)GET_IRQ_OWNER(cp)->priv)->irq_owner = 0;

#ifdef RX_TASKLET
		tasklet_kill(&cp->rx_dsr_tasklet);	
#endif

#ifdef TX_TASKLET
		tasklet_kill(&cp->tx_dsr_tasklet);
#endif
		free_rx_skb();
	}
}
	memset(&cp->net_stats, '\0', sizeof(struct net_device_stats));
	cp->opened = 0;

#if defined(DYNAMIC_ADJUST_TASKLET) || defined(CONFIG_RTL8186_TR) || defined(BR_SHORTCUT)
    if (timer_pending(&cp->expire_timer))
        del_timer_sync(&cp->expire_timer);
#endif

#ifdef BR_SHORTCUT
	if (dev == cached_dev)
		cached_dev=NULL;
#endif

#ifdef CONFIG_RTL865X_HARDWARE_NAT	
	reset_hw_mib_counter(dev);
#endif

	return SUCCESS;
}

#ifdef CONFIG_RTL_STP
static int re865x_stp_open (struct net_device *dev)
{
	struct dev_priv *cp = dev->priv;

	if (cp->opened)
		return SUCCESS;
	
	cp->opened = 1;	
	netif_start_queue(dev);
	return SUCCESS;
}


static int re865x_stp_close (struct net_device *dev)
{
	struct dev_priv *cp = dev->priv;

	if (!cp->opened)
		return SUCCESS;
	netif_stop_queue(dev);

	memset(&cp->net_stats, '\0', sizeof(struct net_device_stats));
	cp->opened = 0;

#ifdef BR_SHORTCUT
	if (dev == cached_dev)
		cached_dev=NULL;
#endif
	return SUCCESS;
}

static int re865x_stp_mapping_init(void)
{
	int i, j, k, totalVlans;
	totalVlans=((sizeof(vlanconfig))/(sizeof(struct rtl865x_vlanConfig)))-1;
	
	for (i = 0; i < MAX_RE865X_STP_PORT; i++)
	{
		STP_PortDev_Mapping[i] = NO_MAPPING; 
	}
	
	#ifdef CONFIG_RTK_MESH
	STP_PortDev_Mapping[WLAN_PSEUDO_IF_INDEX] = WLAN_PSEUDO_IF_INDEX;
	STP_PortDev_Mapping[WLAN_MESH_PSEUDO_IF_INDEX] = WLAN_MESH_PSEUDO_IF_INDEX;
	#else
	STP_PortDev_Mapping[MAX_RE865X_STP_PORT -1] = WLAN_PSEUDO_IF_INDEX;
	#endif
	
	j = 0;
	for(k=0;k<totalVlans;k++)
	{
		if (vlanconfig[k].isWan == FALSE)
		{
			#ifdef CONFIG_RTK_MESH
			for(i=0; i< MAX_RE865X_ETH_STP_PORT ; i++)
			#else
			for(i=0; i< MAX_RE865X_STP_PORT-1 ; i++)
			#endif	
			{
				if ( (1<<i) & vlanconfig[k].memPort ) 
				{
					STP_PortDev_Mapping[j] = i;
					j++;
				}
			}
			
			break;
		}
	}

#if 0	
		printk("mapping table is ");
#ifdef CONFIG_RTK_MESH
	for(i=0; i< MAX_RE865X_ETH_STP_PORT ; i++)
#else
	for(i=0; i< MAX_RE865X_STP_PORT-1 ; i++)
#endif		
	{
		printk(" %d ", STP_PortDev_Mapping[i]);
	}
		printk("\n\n");
#endif
	return SUCCESS;
}

static int re865x_stp_mapping_reinit(void)
{
	int i, j, k, totalVlans;
	totalVlans=((sizeof(vlanconfig))/(sizeof(struct rtl865x_vlanConfig)))-1;
	
	for (i = 0; i < MAX_RE865X_STP_PORT; i++)
	{
		STP_PortDev_Mapping[i] = NO_MAPPING; 
	}

	#ifdef CONFIG_RTK_MESH
	STP_PortDev_Mapping[WLAN_PSEUDO_IF_INDEX] = WLAN_PSEUDO_IF_INDEX;
	STP_PortDev_Mapping[WLAN_MESH_PSEUDO_IF_INDEX] = WLAN_MESH_PSEUDO_IF_INDEX;
	#else
	STP_PortDev_Mapping[MAX_RE865X_STP_PORT -1] = WLAN_PSEUDO_IF_INDEX;
	#endif

	printk("=======stp port dev mapping reinit=======\n");
	j = 0;
	for(k=0;k<totalVlans;k++)
	{
		if (vlanconfig[k].isWan == FALSE)
		{
			#ifdef CONFIG_RTK_MESH
			for(i=0; i< MAX_RE865X_ETH_STP_PORT; i++)
			#else
			for(i=0; i< MAX_RE865X_STP_PORT-1 ; i++)
			#endif
			{
				if ( (1<<i) & vlanconfig[k].memPort ) 
				{
					STP_PortDev_Mapping[j] = i;
					printk("mapping: lan phycisal [port%d] <====>pseudo [port%d]\n", i, j);
					j++;
				}
			}
			
			break;
		}
	}
	
		printk("mapping table is ");
	#ifdef CONFIG_RTK_MESH
	for(i=0; i< MAX_RE865X_ETH_STP_PORT; i++)
	#else
	for(i=0; i< MAX_RE865X_STP_PORT-1 ; i++)
	#endif	
	{
		printk(" %d ", STP_PortDev_Mapping[i]);
	}
		printk("\n\n");
	return SUCCESS;
}

static int re865x_stp_get_pseudodevno(uint32 port_num)
{
	int i, dev_no;
	for(i=0; i< MAX_RE865X_STP_PORT-1 ; i++)
	{
		if( STP_PortDev_Mapping[i] == port_num)
		{
			dev_no = i;
			return dev_no;
		}		
	}
	return NO_MAPPING;

}

#endif
#ifdef CONFIG_RTL865X_HW_PPTPL2TP
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
int rtl865x_set_pptp(struct pptp_acc_info* info)
{
	int     i;

	rtl865x_addPpp(info->pppDev->name, (ether_addr_t*)info->peerMac, info->peerCallID, IF_PPTP);
	
	rtl865x_addVlan(RTL_PPTPL2TP_VLANID);
	rtl865x_addVlanPortMember(RTL_PPTPL2TP_VLANID, rtl865x_wanPortMask|(1<<PKTHDR_EXTPORT_P2));

	for(i=0; i<6; i++)
	{
		if ( (1<<i)&rtl865x_wanPortMask)
			rtl8651_setAsicPvid(i, RTL_PPTPL2TP_VLANID);
	}
	rtl8651_setAsicPvid(PKTHDR_EXTPORT_P2, RTL_PPTPL2TP_VLANID);

	rtl865x_setNetifVid("eth1", RTL_PPTPL2TP_VLANID);
	rtl865x_delVlan(RTL_WANVLANID);
	
	rtl865x_setNetifVid("ppp0", RTL_PPTPL2TP_VLANID);

	rtl865x_setNetifType("ppp0", IF_PPTP);

	rtl865x_addFilterDatabaseEntry(0, (ether_addr_t*)info->peerMac, 1<<PKTHDR_EXTPORT_P2, FDB_TYPE_FWD);
	return SUCCESS;
}

int rtl865x_clr_pptp(struct pptp_acc_info* info)
{
	int	i, j;
	int	pvid;

	rtl865x_addVlan(RTL_WANVLANID);
	rtl865x_addVlanPortMember(RTL_WANVLANID, rtl865x_wanPortMask);

	for(i=0; i<RTL8651_PORT_NUMBER + 3; i++) 
	{ 
		/* Set each port's PVID */
		for(j=0,pvid=0; vlanconfig[j].vid != 0; j++)
		{
			if ( (1<<i) & vlanconfig[j].memPort ) 
			{
				pvid = vlanconfig[j].vid;
				break;
			}
		}
		
		if (pvid!=0)	
		{
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
			CONFIG_CHECK(rtl8651_setAsicPvid(i,pvid));
if(bridgemode!=1)
{
#if defined(CONFIG_RTK_VLAN_SUPPORT)			
			rtl865x_setPortToNetif(vlanconfig[j].name,i);
#endif
}
#else
			LR_CONFIG_CHECK(port_attr.pvid_asic_set(i, pvid));
#endif
		}
	}

	rtl865x_setNetifVid("eth1", RTL_WANVLANID);
	rtl865x_setNetifVid("ppp0", RTL_WANVLANID);

	rtl865x_delVlan(RTL_PPTPL2TP_VLANID);
		
	rtl865x_setNetifType("ppp0", IF_PPPOE);
	rtl865x_delFilterDatabaseEntry(RTL865x_L2_TYPEII, 0, (ether_addr_t*)info->peerMac);

	rtl865x_delPpp(info->peerCallID);
	return SUCCESS;
}
#else
int rtl865x_pptp_set_hwaddr(unsigned char *addr)
{

#if CONFIG_RTK_VLAN_SUPPORT
	/*Hyking:[Fixme], mapping relationship....*/
	if(bridgemode!=1)
	{
	memcpy(vlanconfig[5].mac.octet,addr,ETHER_ADDR_LEN);
	}
	else
		memcpy(vlanconfig[2].mac.octet, addr, ETHER_ADDR_LEN);
#else
	memcpy(vlanconfig[2].mac.octet, addr, ETHER_ADDR_LEN);
#endif

if(bridgemode!=1)
{
#if CONFIG_RTK_VLAN_SUPPORT
	if(vlanconfig[5].vid!=0)
#else
	if(vlanconfig[2].vid != 0)
#endif
	{
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
	{
		rtl865x_netif_t netif;
#if CONFIG_RTK_VLAN_SUPPORT
		netif.macAddr = vlanconfig[5].mac;
		memcpy(netif.name,vlanconfig[5].ifname,MAX_IFNAMESIZE);
#else
		netif.macAddr = vlanconfig[2].mac;
		memcpy(netif.name,vlanconfig[2].ifname,MAX_IFNAMESIZE);
#endif
		rtl865x_setNetifMac(&netif);
	}		
#else
		INIT_CHECK(TBLFIELD(vlan_tbl, vlan_set_mac)(&vlanconfig[2]));
#endif
	}
}else
{
	if(vlanconfig[2].vid != 0)
	{
		
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
	{
		rtl865x_netif_t netif;
		netif.macAddr = vlanconfig[2].mac;
		memcpy(netif.name,vlanconfig[2].ifname,MAX_IFNAMESIZE);
		rtl865x_setNetifMac(&netif);
	}		
#else
		INIT_CHECK(TBLFIELD(vlan_tbl, vlan_set_mac)(&vlanconfig[2]));
#endif
	}
	}

	return SUCCESS;
}
#endif		/* CONFIG_RTL865X_LAYERED_DRIVER */

#endif		/* CONFIG_RTL865X_HW_PPTPL2TP */

#ifdef CONFIG_RTK_VLAN_SUPPORT
	#if defined(CONFIG_TRENDCHIP_ADSL_VLAN)
int rtk_ipoa_support_enable;
	#endif
#endif	

__MIPS16
#if defined(CONFIG_RTL8196_RTL8366)
__IRAM_FWD
#elif defined(CONFIG_RTL_DYNAMIC_IRAM_MAPPING_FOR_WAPI)
__IRAM_TX_WAPI //hyking:iram full...
#endif
static int re865x_start_xmit(struct sk_buff *skb, struct net_device *dev)
{
	//hyking:should lock??
	//unsigned long flags;
 	struct dev_priv *cp = dev->priv;
	rtl_nicTx_info	nicTx;
	
	if((cp->id==0) || (cp->portmask ==0))
	{
		dev_kfree_skb_any(skb);
		return 0;
	}

	nicTx.vid = cp->id;
	//spin_lock_irqsave (&cp->lock, flags);
    	dev->trans_start = jiffies;

#if	defined(RTL_OUTPUTQUEUE_SUPPORT)
	/*	egress qos check	*/
	{
		nicTx.priority= 0;
		nicTx.txIdx = 0;
	}
#endif

if(bridgemode!=1)
{
#ifdef CONFIG_RTK_VLAN_SUPPORT
	#if defined(CONFIG_TRENDCHIP_ADSL_VLAN)
	if(dev->ifindex==2)//wan port
	{
		cp->vlan_setting.global_vlan=1;
		cp->vlan_setting.is_lan=0;
		cp->vlan_setting.vlan=1;
		cp->vlan_setting.tag=1;
		cp->vlan_setting.id=8;
		if(skb->data[12]== 0xaa &&skb->data[13]== 0xaa)
		{
			#if defined(_CICMD_VLAN_TAG)
				rtk_vlan_support_enable=1;
			#else
				rtk_vlan_support_enable=0;
			#endif
		}
		else
		{
			if(rtk_ipoa_support_enable)
				memcpy(&skb->data[0],"\x00\xAA\xBB\x01\x23\x45",6);
			rtk_vlan_support_enable=1;
			
		}
		//rtlglue_printf("****TX to wan port*RTL_LANPORT_MASK=0x%x*****\n",RTL_LANPORT_MASK); //Jase add tmp
	}
	else //lan port
	{
		cp->vlan_setting.global_vlan=0;
		cp->vlan_setting.is_lan=1;
		cp->vlan_setting.vlan=1;
		cp->vlan_setting.tag=1;
		cp->vlan_setting.id=8;
		//rtk_vlan_support_enable=1;
		rtk_vlan_support_enable=0;
	}
	#endif
	if (rtk_vlan_support_enable && cp->vlan_setting.global_vlan) {
		if (tx_vlan_process(dev, &cp->vlan_setting, skb, 0)) {
			cp->net_stats.tx_dropped++;
			dev_kfree_skb_any(skb);
			return 0;				
		}
	}
#endif
}

#ifdef CONFIG_RTL8196_RTL8366
	if (*((unsigned short *)(skb->data+ETH_ALEN*2)) != __constant_htons(ETH_P_8021Q)) 
	{
#if 1
#if 0
		if (skb_headroom(skb) < VLAN_HLEN && skb_cow(skb, VLAN_HLEN) !=0 )
		{
			printk("%s-%d: error! (skb_headroom(skb) == %d < %d). Enlarge it!\n",
			__FUNCTION__, __LINE__, skb_headroom(skb), VLAN_HLEN);
			while (1) ;
		}
#else
		if (skb_headroom(skb) < VLAN_HLEN)
			skb_cow(skb, VLAN_HLEN);
#endif
		skb_push(skb, VLAN_HLEN);
		memmove(skb->data, skb->data + VLAN_HLEN, VLAN_ETH_ALEN<<1);
		*(uint16*)(&(skb->data[12])) = __constant_htons(ETH_P_8021Q);		/* TPID */
		*(uint16*)(&(skb->data[14])) = htons(cp->id);			/* VID */
		*(uint8*)(&(skb->data[14])) &= 0x0f;					/* clear most-significant nibble of byte 14 */
//		*(uint8*)(&(skb->data[14])) |= priorityField;
#else
		vlan_put_tag(skb, cp->id);
#endif
	}
#endif
#ifdef CONFIG_RTL865X_IGMP_SNOOPING
#ifdef CONFIG_RTL8196_RTL8366
	nicTx.portlist = 0x1;		/* must be set 0x7 */
	nicTx.srcExtPort = 0;
	nicTx.flags = (PKTHDR_USED|PKT_OUTGOING);
#else
	if((skb->data[0]&0x01)==0)
	{

if(bridgemode!=1)
{
#if defined(CONFIG_RTK_VLAN_SUPPORT)
                //if ( rtk_vlan_support_enable)
                {
                        /* the pkt must be tx to lan vlan */
                        nicTx.portlist = cp->portmask;
                        nicTx.srcExtPort = 0;		//PKTHDR_EXTPORT_LIST_CPU;
                        nicTx.flags = (PKTHDR_USED|PKT_OUTGOING);
                }
#else
		{	/*unicast process*/
			if (nicTx.vid != RTL_WANVLANID)
			{
				/* the pkt must be tx to lan vlan */
				nicTx.portlist = RTL8651_CPU_PORT;		/* must be set 0x7 */
				nicTx.srcExtPort = PKTHDR_EXTPORT_LIST_CPU;
				nicTx.flags = (PKTHDR_USED|PKTHDR_HWLOOKUP|PKTHDR_BRIDGING|PKT_OUTGOING);
			}
			else
			{
				nicTx.portlist = cp->portmask;
				nicTx.srcExtPort = 0;
				nicTx.flags = (PKTHDR_USED|PKT_OUTGOING);
			}
		}
#endif
}
else
{
	/*unicast process*/
			if (nicTx.vid != RTL_WANVLANID)
			{
				/* the pkt must be tx to lan vlan */
				nicTx.portlist = RTL8651_CPU_PORT;		/* must be set 0x7 */
				nicTx.srcExtPort = PKTHDR_EXTPORT_LIST_CPU;
				nicTx.flags = (PKTHDR_USED|PKTHDR_HWLOOKUP|PKTHDR_BRIDGING|PKT_OUTGOING);
			}
			else
			{
				nicTx.portlist = cp->portmask;
				nicTx.srcExtPort = 0;
				nicTx.flags = (PKTHDR_USED|PKT_OUTGOING);
			}
}
	}
	else
	{
		nicTx.srcExtPort = 0;
		nicTx.flags = (PKTHDR_USED|PKT_OUTGOING);
		//printk("XMIT to portlist 0x%x vid %d\n", cp->portmask, cp->id);
		nicTx.portlist=cp->portmask;
		/*multicast process*/
		if((skb->data[0]==0x01) && (skb->data[1]==0x00) && (skb->data[2]==0x5e))
		{
			int32 retVal;
			struct iphdr *iph;
			struct rtl_multicastDataInfo multicastDataInfo;
			struct rtl_multicastFwdInfo multicastFwdInfo;
			
			iph = skb->nh.iph;
			
			if((iph->protocol==0x11) || (iph->protocol==0x06) )
			{
				if(cp->portnum>1)
				{
				
					/*only process tcp/udp in igmp snooping data plane*/
					multicastDataInfo.ipVersion=4;
					multicastDataInfo.sourceIp[0]=  (uint32)(iph->saddr);
					multicastDataInfo.groupAddr[0]=  (uint32)(iph->daddr);
					//printk("multicastDataInfo.groupAddr[0]= 0x%x\n",multicastDataInfo.groupAddr[0]);//added for test
					retVal= rtl_getMulticastDataFwdInfo(cp->igmpModuleIndex, &multicastDataInfo, &multicastFwdInfo);
				
					/*record not found, broadcast within device vlan*/
					if(retVal==SUCCESS)
					{
						nicTx.portlist = multicastFwdInfo.fwdPortMask& cp->portmask & ((1<<RTL8651_MAC_NUMBER)-1);
					}
					else
					{
					/*unknown multicast*/
					}

				}
				
			}
			
					
		}
#ifdef CONFIG_RTL8186_GR
		else if (oldStatus)
		{
			if (((oldStatus&IP6_PASSTHRU_MASK)&&(*((uint16*)&skb->data[12])==0x86dd)) ||
				((oldStatus&PPPOE_PASSTHRU_MASK)&&((*((uint16*)&skb->data[12])==0x8864)||(*((uint16*)&skb->data[12])==0x8863))) )
			{
				nicTx.portlist = RTL8651_CPU_PORT;		/* must be set 0x7 */
				nicTx.srcExtPort = PKTHDR_EXTPORT_LIST_CPU;
				nicTx.flags = (PKTHDR_USED|PKTHDR_HWLOOKUP|PKT_OUTGOING);
			}
		}
#endif
#ifdef CONFIG_RTL_STP
		if(!dev->irq){
			//virtual interfaces have no IRQ assigned. We use this to identify STP port interfaces.
			uint8 stpPortNum= dev->name[strlen(dev->name)-1]-'0';
/*			printk("send bpdu packet in nic, port num is %d\n", STP_PortDev_Mapping[stpPortNum]);	*/
			if (STP_PortDev_Mapping[stpPortNum] != NO_MAPPING)
			{
				nicTx.portlist = STP_PortDev_Mapping[stpPortNum];
			}
			else
			{
				dev_kfree_skb_any(skb);
			}
		}
#endif
	}
	if(nicTx.portlist==0)
	{
		dev_kfree_skb_any(skb);
		return 0;
	}
#endif

#if CONFIG_RTL_DYNAMIC_IRAM_MAPPING_FOR_WAPI
		#if defined(CONFIG_RTL_DYNAMIC_IRAM_MAPPING_FOR_WAPI_DEBUG)
		FUNCTION_CHECK(p_swNic_send);		
		#endif
		if(p_swNic_send((void *)skb, skb->data, skb->len, &nicTx) < 0)
#else
		if (swNic_send((void *)skb, skb->data, skb->len, &nicTx) < 0) 
#endif
	
	 	{	// tx queue full
		DEBUG_ERR("%s: tx failed!\n", dev->name);
//		netif_stop_queue(dev);
#ifdef CONFIG_RTK_VOIP
		// avoid packet lost!
		struct Qdisc *q=dev->qdisc;
		q->ops->requeue(skb, q);
		netif_schedule(dev);		
		return 0;
#else
		cp->net_stats.tx_dropped++;
//		dev_kfree_skb_irq(skb);
		dev_kfree_skb_any(skb);
		//spin_unlock_irqrestore(&cp->lock, flags);		

		/* tx queue full and drop this packet, return 0 to indicate the caller that the packet is done.
		 * return 1 the caller will resend it again and cause core dump in __kfree_skb().
		 */
		return 0;
#endif		
    }
#else //CONFIG_RTL865X_IGMP_SNOOPING

	#ifdef CONFIG_RTL8196_RTL8366
	nicTx.portlist = 0x1;		/* must be set 0x7 */
	nicTx.srcExtPort = 0;
	nicTx.flags = (PKTHDR_USED|PKT_OUTGOING);
	#else
	if((skb->data[0]&0x01)==0)
	{
		/*unicast process*/
		if (nicTx.vid != RTL_WANVLANID)
		{
			/* the pkt must be tx to lan vlan */
			nicTx.portlist = RTL8651_CPU_PORT;		/* must be set 0x7 */
			nicTx.srcExtPort = PKTHDR_EXTPORT_LIST_CPU;
			nicTx.flags = (PKTHDR_USED|PKTHDR_HWLOOKUP|PKTHDR_BRIDGING|PKT_OUTGOING);
		}
		else
		{
			nicTx.portlist = cp->portmask;
			nicTx.srcExtPort = 0;
			nicTx.flags = (PKTHDR_USED|PKT_OUTGOING);
		}
	}
	else
	{
		nicTx.srcExtPort = 0;
		nicTx.flags = (PKTHDR_USED|PKT_OUTGOING);
		nicTx.portlist=cp->portmask;
	}
	if(nicTx.portlist==0)
	{
		dev_kfree_skb_any(skb);
		return 0;
	}
	#endif
{

#if CONFIG_RTL_DYNAMIC_IRAM_MAPPING_FOR_WAPI
	#if defined(CONFIG_RTL_DYNAMIC_IRAM_MAPPING_FOR_WAPI_DEBUG)
	FUNCTION_CHECK(p_swNic_send);		
	#endif

	if(p_swNic_send((void *)skb, skb->data, skb->len, &nicTx)< 0)
#else
	if ( swNic_send((void *)skb, skb->data, skb->len, &nicTx)< 0) 
#endif
	{	// tx queue full
		DEBUG_ERR("%s: tx failed!\n", dev->name);

//		netif_stop_queue(dev);
#ifdef CONFIG_RTK_VOIP
		// avoid packet lost!
		struct Qdisc *q=dev->qdisc;
		q->ops->requeue(skb, q);
		netif_schedule(dev);		
		return 0;
#else		
		cp->net_stats.tx_dropped++;
		dev_kfree_skb_any(skb);	

		/* tx queue full and drop this packet, return 0 to indicate the caller that the packet is done.
		 * return 1 the caller will resend it again and cause core dump in __kfree_skb().
		 */
		return 0;
#endif
}
    }
#endif	

	cp->net_stats.tx_packets++;		
	cp->net_stats.tx_bytes += skb->len;

#ifdef BR_SHORTCUT
	pkt_cnt++;
#endif

	//spin_unlock_irqrestore(&cp->lock, flags);   
    return 0;
}


static void re865x_tx_timeout (struct net_device *dev)
{	
	rtlglue_printf("Tx Timeout!!! Can't send packet\n");

	/*
	struct dev_priv *cp = (struct dev_priv*)dev->priv;
	int free_desc_num;
	unsigned long flags;

	
	
	local_irq_save(flags);

	//refill_rx_skb();

	free_desc_num = swNic_txDone(); 

	if (free_desc_num >= (txRingSize[0]/8))
    	{
		if (cp->opened && netif_queue_stopped(cp->dev)) 			
			netif_wake_queue(cp->dev);
		
		if (NEXT_CP(cp)->opened && netif_queue_stopped(NEXT_DEV(cp)))
			netif_wake_queue(NEXT_DEV(cp));			
   	}
		
	local_irq_restore(flags);
	*/
}


int re865x_ioctl (struct net_device *dev, struct ifreq *rq, int cmd)
{
	int32 rc = 0;
	unsigned long *data;
	int32 args[4];
	int32  * pRet;
#if defined(CONFIG_RTL8186_KB)||defined(CONFIG_RTL8186_GR)
	uint32	*pU32;
#endif

	if (cmd != SIOCDEVPRIVATE)
	{
		goto normal;
	}

	data = (unsigned long *)rq->ifr_data;

	if (copy_from_user(args, data, 4*sizeof(unsigned long)))
	{
		return -EFAULT;
	}
	

//    rtlglue_printf("____args[0]:%s args[0]:%s args[1]:%s args[2]:%s args[3]:%s args[4]:%s____\n",args[0],args[1],args[2],args[3],args[4]);

	bridgemode=0;
	switch (args[0])
	{
		case RTL8651_IOCTL_CHANGETOBRIDGE:
			bridgemode=1;
			rtl865x_changeOpMode(BRIDGE_MODE);	
		break;

        case RTL8651_IOCTL_CHANGETOGATEWAY:
        	rtl865x_changeOpMode(GATEWAY_MODE);
        break;

		case RTL8651_IOCTL_CHANGETOLANWANBRIDGE:
			bridgemode=1;
			rtl865x_changeOpMode(LANWAN_BRIDGE_MODE);	
		break;

        case RTL8651_IOCTL_CHANGETOVLANBRIDGE:	//2350
            rtl865x_changeOpMode(VLAN_BRIDGE_MODE);
        break;
        case 2351:
            rtl865x_changeOpMode(VLAN_BRIDGE_MODE_1);
            break;
        case 2352:
            rtl865x_changeOpMode(VLAN_BRIDGE_MODE_2);
            break;
        case 2353:
            rtl865x_changeOpMode(VLAN_BRIDGE_MODE_3);
			break;

#ifdef CONFIG_RTK_VLAN_SUPPORT
	#if defined(CONFIG_TRENDCHIP_ADSL_VLAN)
	        
        case RTL8651_IOCTL_CHANGETOIPOA:
			rtk_ipoa_support_enable=1;	
		break;
		
		case RTL8651_IOCTL_CHANGETONONIPOA:
			rtk_ipoa_support_enable=0;	
		break;
	#endif
#endif		

#ifdef CONFIG_RTL8196_RTL8366
		case RTL8651_IOCTL_GETWANLINKSTATUS:
			{
				uint32 phyNum;
				uint32 linkStatus;

				pRet = (int32 *)args[3];
				*pRet = FAILED;
				rc = SUCCESS;

				phyNum=4;//8366 WAN port
				rc=rtl8366rb_getPHYLinkStatus(phyNum, &linkStatus);

				if(rc==SUCCESS)
				{
					if(linkStatus==1)
					{
//						rtlglue_printf("8366 Wan port link up\n");//Added for test
						*pRet = SUCCESS;
					}
				}

//				rtlglue_printf("%s(%d): *pRet=%d\n",__FUNCTION__,__LINE__,*pRet);//Added for test
				break;
			}
#else
		case RTL8651_IOCTL_GETWANLINKSTATUS:
			{
				int i;
				int wanPortMask;
				int32 totalVlans;

				pRet = (int32 *)args[3];
				*pRet = FAILED;
				rc = SUCCESS;

				wanPortMask = 0;
				totalVlans=((sizeof(vlanconfig))/(sizeof(struct rtl865x_vlanConfig)))-1;
				for(i=0;i<totalVlans;i++)
				{
					if(vlanconfig[i].isWan==TRUE)
						wanPortMask = vlanconfig[i].memPort;
				}

//				rtlglue_printf("Wan port wanPortMask=%d,totalVlans=%d\n",wanPortMask,totalVlans);//Added for test
				if (wanPortMask==0)
				{
					/* no wan port exist */					
					break;
				}

				for(i=0;i<RTL8651_AGGREGATOR_NUMBER;i++)
				{
					if( (1<<i)&wanPortMask )
					{
						rtlglue_printf("PSRP0=0x%x,PortStatusLinkUp=0x%x",PSRP0,PortStatusLinkUp);//Added for test
						if((READ_MEM32(PSRP0+(i<<2))&PortStatusLinkUp)!=0)
						{
							rtlglue_printf("Wan port i=%d\n",i);//Added for test
							*pRet = SUCCESS;
						}
						break;
					}
				}
				
				break;
			}
#endif
		case RTL8651_IOCTL_GETWANLINKSPEED:
			{
				int i;
				int wanPortMask;
				int32 totalVlans;
					
				pRet = (int32 *)args[3];
				*pRet = FAILED;
				rc = FAILED;

				wanPortMask = 0;
				totalVlans=((sizeof(vlanconfig))/(sizeof(struct rtl865x_vlanConfig)))-1;
				for(i=0;i<totalVlans;i++)
				{
					if(vlanconfig[i].isWan==TRUE)
						wanPortMask = vlanconfig[i].memPort;
				}

				if (wanPortMask==0)
				{
					/* no wan port exist */
					//rtlglue_printf("%s(%d)Wan port: no wan port exist\n",__FUNCTION__,__LINE__);//Added for test
					break;
				}

				for(i=0;i<RTL8651_AGGREGATOR_NUMBER;i++)
				{
					if( (1<<i)&wanPortMask )
					{
						//if((READ_MEM32(PSRP0+(i<<2))&PortStatusLinkUp)!=0)
						//{
							//rtlglue_printf("%s(%d)Wan port i=%d\n",__FUNCTION__,__LINE__,i);//Added for test
							//*pRet = SUCCESS;
						//}
						break;
					}
				}
				
				switch(READ_MEM32(PSRP0 + (i<<2)) & PortStatusLinkSpeed_MASK)
				{
					case PortStatusLinkSpeed10M:
						*pRet = PortStatusLinkSpeed10M;
						rc = SUCCESS;
						//rtlglue_printf("%s(%d)Wan port: 10M\n",__FUNCTION__,__LINE__);//Added for test
						break;
					case PortStatusLinkSpeed100M:
						*pRet = PortStatusLinkSpeed100M;
						rc = SUCCESS;
						//rtlglue_printf("%s(%d)Wan port: 100M\n",__FUNCTION__,__LINE__);//Added for test
						break;
					case  PortStatusLinkSpeed1000M:
						*pRet = PortStatusLinkSpeed1000M;
						rc = SUCCESS;
						//rtlglue_printf("%s(%d)Wan port: 1000M\n",__FUNCTION__,__LINE__);//Added for test
						break;
					default:
						//rtlglue_printf("%s(%d)Wan port: no match\n",__FUNCTION__,__LINE__);//Added for test
						break;
				}
				break;
			}
#if 0		
#ifdef CONFIG_RTK_MESH
		case RTL8651_IOCTL_GETLANLINKSTATUSALL:
			{
				pRet = (int32 *)args[3];
				*pRet = FAILED;
				
				rc = SUCCESS;
				if (check_LAN_linkstatus_any()!=0)
					*pRet = SUCCESS;
				
				break;		
			}
#endif
#endif
#if defined(CONFIG_RTL8186_KB)|| defined(CONFIG_RTL8186_GR)
		case RTL8651_IOCTL_GETLANLINKSTATUS:
			{
				int i;
				int lanPortMask;
				int32 totalVlans;

				pRet = (int32 *)args[3];
				*pRet = FAILED;
				rc = SUCCESS;

				lanPortMask = 0;
				totalVlans=((sizeof(vlanconfig))/(sizeof(struct rtl865x_vlanConfig)))-1;
				for(i=0;i<totalVlans;i++)
				{
					if(vlanconfig[i].isWan==FALSE)
					{
						lanPortMask = vlanconfig[i].memPort;
						if (lanPortMask==0)
						{
							/* no wan port exist */					
							continue;
						}

						for(i=0;i<=RTL8651_PHY_NUMBER;i++)
						{
							if( (1<<i)&lanPortMask )
							{
								if((READ_MEM32(PSRP0+(i<<2))&PortStatusLinkUp)!=0)
								{
									//rtlglue_printf("Lan port i=%d\n",i);//Added for test
									*pRet = SUCCESS;
									return rc;
								}
							}
						}
					}
				}

				break;
			}
#if defined(CONFIG_RTL8186_KB)
		case RTL8651_IOCTL_GETWANTHROUGHPUT:
			{
				static unsigned long last_jiffies = 0;
				static unsigned long last_rxtx = 0;
				int i;
				int32 totalVlans;
				struct dev_priv *cp;
				int32 *throughputLevel;
				unsigned long	diff_jiffies;
				pRet = (int32 *)args[3];

				diff_jiffies = (jiffies-last_jiffies);
				if (diff_jiffies>HZ)
				{
					pU32 = (uint32*)args[1];
					throughputLevel = (uint32*)pU32[0];
					rc = SUCCESS;

					cp = NULL;
					totalVlans=((sizeof(vlanconfig))/(sizeof(struct rtl865x_vlanConfig)))-1;
					for(i=0;i<totalVlans;i++)
					{
						if(vlanconfig[i].isWan==TRUE)
						{
							cp = _rtl86xx_dev.dev[i]->priv;
							break;
						}
					}

					if (cp==NULL)
					{
						/* no wan port exist */					
						rc = FAILED;
					}

					for(i=1;i<20;i++)
					{
						if (diff_jiffies < (HZ<<i))
							break;
					}
					/* get the throughput level */
					*throughputLevel = (((cp->net_stats.rx_bytes + cp->net_stats.tx_bytes)-last_rxtx)>>(17+i));

					last_jiffies = jiffies;
					last_rxtx = (cp->net_stats.rx_bytes + cp->net_stats.tx_bytes);
					*pRet = SUCCESS;
				}
				else
				{
					*pRet = FAILED;
				}
			}
#endif
#endif
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
#if defined(CONFIG_RTL8186_GR)
		case RTL8651_IOCTL_SETWANLINKSTATUS:
			{
				int i;
				int wanPortMask;
				int32 totalVlans;
				int portStatusToSet;
				int forceMode;
				int forceLink;
				int forceLinkSpeed;
				int forceDuplex;
				uint32 regValue;
				uint32 advCapability;
				#define SPEED10M 	0
				#define SPEED100M 	1
				#define SPEED1000M 	2
				
				pRet = (int32 *)args[3];
				*pRet = FAILED;
				rc = SUCCESS;
				
				pU32 = (uint32*)args[1];
				portStatusToSet = *(uint32*)pU32[0];
				
				wanPortMask = 0;
				totalVlans=((sizeof(vlanconfig))/(sizeof(struct rtl865x_vlanConfig)))-1;
				for(i=0;i<totalVlans;i++)
				{
					if(vlanconfig[i].isWan==TRUE)
						wanPortMask = vlanconfig[i].memPort;
				}

				if (wanPortMask==0)
				{
					/* no wan port exist */					
					break;
				}

				for(i=0;i<RTL8651_AGGREGATOR_NUMBER;i++)
				{
					if( (1<<i)&wanPortMask )
					{
						/*write register*/
			
						if(HALF_DUPLEX_10M == portStatusToSet)
						{
							forceMode=TRUE;
							forceLink=TRUE;
							forceLinkSpeed=SPEED10M;
							forceDuplex=FALSE;
							advCapability=(1<<HALF_DUPLEX_10M);
						}else if(HALF_DUPLEX_100M == portStatusToSet)
						{
							forceMode=TRUE;
							forceLink=TRUE;
							forceLinkSpeed=SPEED100M;
							forceDuplex=FALSE;
							advCapability=(1<<HALF_DUPLEX_100M);
						}else if(HALF_DUPLEX_1000M == portStatusToSet)
						{
							forceMode=TRUE;
							forceLink=TRUE;
							forceLinkSpeed=SPEED1000M;
							forceDuplex=FALSE;
							advCapability=(1<<HALF_DUPLEX_1000M);
						}else if(DUPLEX_10M == portStatusToSet)
						{
							forceMode=TRUE;
							forceLink=TRUE;
							forceLinkSpeed=SPEED10M;
							forceDuplex=TRUE;
							advCapability=(1<<DUPLEX_10M);
						}else if(DUPLEX_100M == portStatusToSet)
						{
							forceMode=TRUE;
							forceLink=TRUE;
							forceLinkSpeed=SPEED100M;
							forceDuplex=TRUE;
							advCapability=(1<<DUPLEX_100M);
						}else if(DUPLEX_1000M == portStatusToSet)
						{
							forceMode=TRUE;
							forceLink=TRUE;
							forceLinkSpeed=SPEED1000M;
							forceDuplex=TRUE;
							advCapability=(1<<DUPLEX_1000M);
						}else if(PORT_AUTO == portStatusToSet)
						{
							forceMode=FALSE;
							forceLink=TRUE;
							/*all capality*/
							advCapability=(1<<PORT_AUTO);
							
						}else
						{
							forceMode=FALSE;
							forceLink=TRUE;
						}
						rtl865xC_setAsicEthernetForceModeRegs(i, forceMode, forceLink, forceLinkSpeed, forceDuplex);
						
						/*Set PHY Register*/
						rtl8651_setAsicEthernetPHYSpeed(i,forceLinkSpeed);
						rtl8651_setAsicEthernetPHYDuplex(i,forceDuplex);
						rtl8651_setAsicEthernetPHYAutoNeg(i,TRUE);
						rtl8651_setAsicEthernetPHYAdvCapality(i,advCapability);
						rtl8651_restartAsicEthernetPHYNway(i);
						break;
					}
				}
				
				break;
			}
		case RTL8651_IOCTL_GETLANPORTLINKSTATUS:
			{
				int i;
                            int lanPortMask;
                            int32 totalVlans;
				int32 *lanportnum;
				int32 lanPortTypeMask;
				uint32 regVal;
				uint32 portLinkSpeed;
 
                            pRet = (int32 *)args[3];
                            *pRet = FAILED;
                            rc = SUCCESS;

				pU32 = (uint32*)args[1];
				lanportnum = (uint32*)pU32[0];
 
                                lanPortMask = 0;
                                totalVlans=((sizeof(vlanconfig))/(sizeof(struct rtl865x_vlanConfig)))-1;
                                for(i=0;i<totalVlans;i++)
                                {
                                        if(vlanconfig[i].isWan==FALSE)
                                        {
                                                lanPortMask = vlanconfig[i].memPort;
                                                if (lanPortMask==0)
                                                {
                                                        /* no wan port exist */
                                                        continue;
                                                }
 
                                                for(i=0;i<=RTL8651_PHY_NUMBER;i++)
                                                {
                                                        if( (1<<i)&lanPortMask )
                                                        {
                                                        	regVal=READ_MEM32(PSRP0+(i<<2));
                                                                if((regVal&PortStatusLinkUp)!=0)
                                                                {
										if(i==(*lanportnum))
										{
											//rtlglue_printf("Lan port i=%d\n",i);//Added for test
                                                                        	*pRet = SUCCESS;
																			
											if((regVal&PortStatusDuplex)!=0)
											{
												lanPortTypeMask=1;
												*pRet |= lanPortTypeMask;
												//rtlglue_printf("Duplex port!\n");//Added for test
											}

											portLinkSpeed=regVal&PortStatusLinkSpeed_MASK;
											if(PortStatusLinkSpeed100M==portLinkSpeed)
											{
												lanPortTypeMask=4;
												*pRet |= lanPortTypeMask;
												//rtlglue_printf("100M port!\n");//Added for test
											}
											else if(PortStatusLinkSpeed1000M==portLinkSpeed)
											{
												lanPortTypeMask=8;
												*pRet |= lanPortTypeMask;
												//rtlglue_printf("1000M port!\n");//Added for test
											}
											else
											{
												lanPortTypeMask=2;
												*pRet |= lanPortTypeMask;
												//rtlglue_printf("10M port!\n");//Added for test
											}
											
                                                                        	return rc;
										}
                                                                }
                                                        }
                                                }
                                        }
                                }
 
                                break;
			}

		case RTL8651_IOCTL_GETWANPORTLINKSTATUS:
			{
				int i;
				int wanPortMask;
				int32 totalVlans;
				int32 wanPortTypeMask;
				uint32 regVal;
				uint32 portLinkSpeed;

				pRet = (int32 *)args[3];
				*pRet = FAILED;
				rc = SUCCESS;

				wanPortMask = 0;
				totalVlans=((sizeof(vlanconfig))/(sizeof(struct rtl865x_vlanConfig)))-1;
				for(i=0;i<totalVlans;i++)
				{
					if(vlanconfig[i].isWan==TRUE)
						wanPortMask = vlanconfig[i].memPort;
				}

				if (wanPortMask==0)
				{
					/* no wan port exist */					
					break;
				}

				for(i=0;i<RTL8651_AGGREGATOR_NUMBER;i++)
				{
					if( (1<<i)&wanPortMask )
					{
						/*check phy status link up or down*/
						rtl8651_getAsicEthernetPHYStatus(i,&regVal);
						if(regVal & (1<<2))
						{
							regVal=READ_MEM32(PSRP0+(i<<2));
							if((regVal&PortStatusLinkUp)!=0)
							{
								//rtlglue_printf("Wan port i=%d\n",i);//Added for test
								*pRet = SUCCESS;

								if((regVal&PortStatusDuplex)!=0)
								{
										wanPortTypeMask=1;
										*pRet |= wanPortTypeMask;
										//rtlglue_printf("WAN: Duplex port!\n");//Added for test
								}

								portLinkSpeed=regVal&PortStatusLinkSpeed_MASK;
								if(PortStatusLinkSpeed100M==portLinkSpeed)
								{
										wanPortTypeMask=4;
										*pRet |= wanPortTypeMask;
										//rtlglue_printf("WAN: 100M port!\n");//Added for test
								}
								else if(PortStatusLinkSpeed1000M==portLinkSpeed)
								{
										wanPortTypeMask=8;
										*pRet |= wanPortTypeMask;
										//rtlglue_printf("WAN: 1000M port!\n");//Added for test
								}
								else
								{
										wanPortTypeMask=2;
										*pRet |= wanPortTypeMask;
										//rtlglue_printf("WAN: 10M port!\n");//Added for test
								}
							}
						}
						break;
					}
				}
				
				break;
			}
#endif
#endif
		default:
			rc = SUCCESS;
			break;
	}
	
	return rc;
			
normal:	
	if (!netif_running(dev))
		return -EINVAL;
	switch (cmd)
        {
	    default:
		rc = -EOPNOTSUPP;
		break;
	}
	return rc;
}

#if defined(DYNAMIC_ADJUST_TASKLET) || defined(CONFIG_RTL8186_TR) || defined(BR_SHORTCUT) 
static void one_sec_timer(unsigned long task_priv)
{
	uint32 flags;
	
    struct dev_priv *cp = ((struct net_device *)task_priv)->priv;

	spin_lock_irqsave (&cp->lock, flags);

#ifdef DYNAMIC_ADJUST_TASKLET
    if (((struct net_device *)task_priv)->name[3] == '0' && rx_pkt_thres > 0 && 
									((eth_flag&TASKLET_MASK) == 0))  { 
        if (rx_cnt > rx_pkt_thres) {
			if (!rx_tasklet_enabled) {
				rx_tasklet_enabled = 1;
            }							
        }               
        else {
			if (rx_tasklet_enabled) { // tasklet enabled
				rx_tasklet_enabled = 0;
			}						
        }       
        rx_cnt = 0;     
    }   
#endif

#ifdef CONFIG_RTL8186_TR
    cp->tx_avarage = (cp->tx_avarage/10)*7 + (cp->tx_byte_cnt/10)*3;
    if (cp->tx_avarage > cp->tx_peak)
        cp->tx_peak = cp->tx_avarage;
    cp->tx_byte_cnt = 0;
    
    cp->rx_avarage = (cp->rx_avarage/10)*7 + (cp->rx_byte_cnt/10)*3;
    if (cp->rx_avarage > cp->rx_peak)
        cp->rx_peak = cp->rx_avarage;
    cp->rx_byte_cnt = 0;    
#endif

#ifdef BR_SHORTCUT
	if (pkt_cnt > 100)
		enable_brsc=1;
	else
		enable_brsc=0;
#endif

#if 0
#ifdef CONFIG_RTK_MESH
	static int link_state = 0;
	int current_ls;

	current_ls = check_LAN_linkstatus_any();
	if (current_ls!=link_state){
		br_signal_pathsel();
		link_state = current_ls;
	}
		
#endif
#endif

	mod_timer(&cp->expire_timer, jiffies + 100);

	spin_unlock_irqrestore(&cp->lock, flags);   
}
#endif


static int rtl865x_set_hwaddr(struct net_device *dev, void *addr)
{
	unsigned long flags;
	int i;
	unsigned char *p;

	p = ((struct sockaddr *)addr)->sa_data;
 	local_irq_save(flags);
	for (i = 0; i<ETHER_ADDR_LEN; ++i) {
		dev->dev_addr[i] = p[i];
	}

if(bridgemode!=1)
{
#if defined(CONFIG_RTK_VLAN_SUPPORT)
	i = dev->name[3] - '0';
	if ((i >=0) && (i <5))
		memcpy(vlanconfig[i].mac.octet, dev->dev_addr, ETHER_ADDR_LEN);

#else
	if (!strcmp(dev->name, "eth0"))
		i = 0;
	else
		i = 1;	
	memcpy(vlanconfig[i].mac.octet, dev->dev_addr, ETHER_ADDR_LEN);
#endif
}
else
{
	if (!strcmp(dev->name, "eth0"))
		i = 0;
	else
		i = 1;	
	memcpy(vlanconfig[i].mac.octet, dev->dev_addr, ETHER_ADDR_LEN);
}

#ifdef CONFIG_HARDWARE_NAT_DEBUG
/*2007-12-19*/
	rtlglue_printf("%s:%d:dev->name is %s,__lrconfig[%d].mac is 0x%x:%x:%x:%x:%x:%x\n",__FUNCTION__,__LINE__,dev->name, i, 
	vlanconfig[i].mac.octet[0],vlanconfig[i].mac.octet[1],vlanconfig[i].mac.octet[2],vlanconfig[i].mac.octet[3],vlanconfig[i].mac.octet[4],vlanconfig[i].mac.octet[5]);

	rtlglue_printf("%s:%d:__lrconfig[%d].vid is %d\n",__FUNCTION__,__LINE__,i,vlanconfig[i].vid);
#endif


#ifdef CONFIG_RTL865X_LAYERED_DRIVER
	if(vlanconfig[i].vid != 0)
	{
		rtl865x_netif_t netif;
		netif.macAddr = vlanconfig[i].mac;
		memcpy(netif.name,vlanconfig[i].ifname,MAX_IFNAMESIZE);
		rtl865x_setNetifMac(&netif);
	}
	
#else

	if(vlanconfig[i].vid!=0)
	{	
		INIT_CHECK(TBLFIELD(vlan_tbl, vlan_set_mac)(&vlanconfig[i]));
	}	
#endif

	local_irq_restore(flags);
	return SUCCESS;
}

#if defined(CONFIG_RTL8186_LINK_CHANGE)
static int rtl865x_set_link(struct net_device *dev, int enable)
{
	int32 i;
	struct dev_priv *cp;
	int32 portmask;
	int32 totalVlans;
	
	cp = dev->priv;
	portmask=0;
	totalVlans=((sizeof(vlanconfig))/(sizeof(struct rtl865x_vlanConfig)))-1;

	for(i=0;i<totalVlans;i++)
	{
		if(vlanconfig[i].vid==cp->id)
		{
			portmask = vlanconfig[i].memPort;
			break;
		}
	}
	
	if (portmask)
	{
		if (enable)
		{
			for(i=0;i<RTL8651_PHY_NUMBER;i++)
			{
				if (portmask & (1<<i))
				{
					rtl865xC_setAsicEthernetForceModeRegs(i, FALSE, TRUE, 1, TRUE);
					rtl8651_restartAsicEthernetPHYNway(i);
				}
			}
		}
		else
		{
			for(i=0;i<RTL8651_PHY_NUMBER;i++)
			{
				if (portmask & (1<<i))
					rtl865xC_setAsicEthernetForceModeRegs(i, TRUE, FALSE, 1, TRUE);
			}
		}
	}

	return SUCCESS;
}
#endif

static int rtl865x_set_mtu(struct net_device *dev, int new_mtu)
{
	unsigned long flags;
	int i;

 	local_irq_save(flags);
if(bridgemode!=1)
{
#if defined(CONFIG_RTK_VLAN_SUPPORT)
	i = dev->name[3] - '0';
	if (i>5)
		return FAILED;		
#else
	if (!strcmp(dev->name, "eth0"))
		i = 0;
	else
		i = 1;
#endif
}
else
{
	if (!strcmp(dev->name, "eth0"))
		i = 0;
	else
		i = 1;
}

	dev->mtu = new_mtu;	
	vlanconfig[i].mtu=(uint32)new_mtu;
	if(vlanconfig[i].vid!=0)
	{
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
	{
		rtl865x_netif_t netif;
		netif.mtu = vlanconfig[i].mtu;
		memcpy(netif.name,vlanconfig[i].ifname,MAX_IFNAMESIZE);

		rtl865x_setNetifMtu(&netif);
	}
#else
		INIT_CHECK(TBLFIELD(vlan_tbl, vlan_set_mtu)(&vlanconfig[i]));
#endif
	}
	#ifdef CONFIG_HARDWARE_NAT_DEBUG
	/*2007-12-19*/
		rtlglue_printf("%s:%d:new_mtu is %d\n",__FUNCTION__,__LINE__,new_mtu);
	#endif

	local_irq_restore(flags);

	return SUCCESS;
}

int  __init re865x_probe (void)
{
/*2007-12-19*/
	extern	uint32* rxMbufRing;
	int32 i, j;
	int32 totalVlans=((sizeof(vlanconfig))/(sizeof(struct rtl865x_vlanConfig)))-1;
#ifdef CONFIG_RTL865X_IGMP_SNOOPING
	int32 retVal;
	int32 igmpInitFlag=FAILED;
	struct rtl_mCastSnoopingGlobalConfig mCastSnoopingGlobalConfig;
	#ifdef CONFIG_RTL865X_HARDWARE_MULTICAST
	rtl865x_mCastConfig_t mCastConfig;
	#endif
#endif
#if defined(CONFIG_RTK_VLAN_SUPPORT)
	struct proc_dir_entry *res_stats_root;
	struct proc_dir_entry *res_stats;
	struct proc_dir_entry *rtk_vlan_support_entry;

	struct proc_dir_entry *rtk_vlan_netmask_entry;
	struct proc_dir_entry *vlan_num_entry;
#endif

	rtlglue_printf("Probing RTL8186 10/100 NIC...\n");
    	REG32(CPUIIMR) = 0x00;
    	REG32(CPUICR) &= ~(TXCMD | RXCMD);
	rxMbufRing=NULL;
	/*Initial ASIC table*/

	{
		rtl8651_tblAsic_InitPara_t para;

		memset(&para, 0, sizeof(rtl8651_tblAsic_InitPara_t));

		/*
			For DEMO board layout, RTL865x platform define corresponding PHY setting and PHYID.
		*/
#ifdef CONFIG_RTL865X_LAYERED_DRIVER

#ifdef CONFIG_RTL865X_LAYERED_ASIC_DRIVER
		INIT_CHECK(rtl865x_initAsicL2(&para));
#ifdef CONFIG_RTL865X_LAYERED_ASIC_DRIVER_L3
		INIT_CHECK(rtl865x_initAsicL3());
#endif
#ifdef CONFIG_RTL865X_LAYERED_ASIC_DRIVER_L4
		INIT_CHECK(rtl865x_initAsicL4());
#endif
#else
		INIT_CHECK(rtl8651_initAsic(&para));
#endif

#else
		DRV_INIT_CHECK(rtl8651_initAsic(&para));
#endif
		/*
			Re-define the wan port according the wan port detection result.
			NOTE:
				There are a very strong assumption that if port5 was giga port, 
				then wan port was port 5.
		*/
		if (RTL865X_PORTMASK_UNASIGNED==rtl865x_wanPortMask)
		{
			/* keep the original mask */
			assert(RTL865X_PORTMASK_UNASIGNED==rtl865x_lanPortMask);
			rtl865x_wanPortMask = RTL_WANPORT_MASK;
			rtl865x_lanPortMask = RTL_LANPORT_MASK;
		}
		else
		{
			/* redefine wan port mask */
			assert(RTL865X_PORTMASK_UNASIGNED!=rtl865x_lanPortMask);
			for(i=0;i<totalVlans;i++)
			{
				if (TRUE==vlanconfig[i].isWan)
				{
					vlanconfig[i].memPort = vlanconfig[i].untagSet = rtl865x_wanPortMask;
				}
				else
				{
					vlanconfig[i].memPort = vlanconfig[i].untagSet = rtl865x_lanPortMask;
				}
			}
		}
#if 1		/*	10/100 & giga use the same pre-allocated skb number */
		/*
			Re-define the pre-allocated skb number according the wan 
			port detection result.
			NOTE:
				There are a very strong assumption that if port1~port4 were
				all giga port, then the sdram was 32M.
		*/
		{
			if (RTL865X_PREALLOC_SKB_UNASIGNED==rtl865x_maxPreAllocRxSkb)
			{
				assert(rtl865x_rxSkbPktHdrDescNum==
					rtl865x_txSkbPktHdrDescNum==
					RTL865X_PREALLOC_SKB_UNASIGNED);

				rtl865x_maxPreAllocRxSkb = MAX_PRE_ALLOC_RX_SKB;
				rtl865x_rxSkbPktHdrDescNum = NUM_RX_PKTHDR_DESC;
				rtl865x_txSkbPktHdrDescNum = NUM_TX_PKTHDR_DESC;
			}
			else
			{
				assert(rtl865x_rxSkbPktHdrDescNum!=RTL865X_PREALLOC_SKB_UNASIGNED);
				assert(rtl865x_txSkbPktHdrDescNum!=RTL865X_PREALLOC_SKB_UNASIGNED);
				/* Assigned value in function of rtl8651_initAsic() */
				rxRingSize[0] = rtl865x_rxSkbPktHdrDescNum;
				txRingSize[0] = rtl865x_txSkbPktHdrDescNum;
			}
		}
#else
		{
			rtl865x_maxPreAllocRxSkb = MAX_PRE_ALLOC_RX_SKB;
			rtl865x_rxSkbPktHdrDescNum = NUM_RX_PKTHDR_DESC;
			rtl865x_txSkbPktHdrDescNum = NUM_TX_PKTHDR_DESC;
		}
#endif

	}

#ifdef BR_SHORTCUT
	cached_dev=NULL;
#endif
	/*init PHY LED style*/
#if defined(CONFIG_RTL865X_BICOLOR_LED)
		#ifdef BICOLOR_LED_VENDOR_BXXX
			REG32(LEDCR) |= (1 << 19); // 5 ledmode set to 1 for bi-color LED
			REG32(PABCNR) &= ~0x001f0000; /* set port port b-4/3/2/1/0 to gpio */
			REG32(PABDIR) |=  0x001f0000; /* set port port b-4/3/2/1/0 gpio direction-output */
		#else
			//8650B demo board default: Bi-color 5 LED
			WRITE_MEM32(LEDCR, READ_MEM32(LEDCR) | 0x01180000 ); // bi-color LED
		#endif
		/* config LED mode */
		WRITE_MEM32(SWTAA, PORT5_PHY_CONTROL);
		WRITE_MEM32(TCR0, 0x000002C2); //8651 demo board default: 15 LED boards
		WRITE_MEM32(SWTACR, CMD_FORCE | ACTION_START); // force add
#else /* CONFIG_RTL865X_BICOLOR_LED */

		/* config LED mode */
		WRITE_MEM32(LEDCR, 0x00000000 ); // 15 LED
		WRITE_MEM32(SWTAA, PORT5_PHY_CONTROL);
		WRITE_MEM32(TCR0, 0x000002C7); //8651 demo board default: 15 LED boards
		WRITE_MEM32(SWTACR, CMD_FORCE | ACTION_START); // force add
#endif /* CONFIG_RTL865X_BICOLOR_LED */

/*2007-12-19*/
if(bridgemode!=1)
{

#if defined(CONFIG_RTK_VLAN_SUPPORT)
		//port based decision
	rtl865xC_setNetDecisionPolicy(NETIF_PORT_BASED);
		WRITE_MEM32(PLITIMR,0);

#endif
}
	INIT_CHECK(rtl865x_init());
	INIT_CHECK(rtl865x_config(vlanconfig));

	/* create all default VLANs */
//	rtlglue_printf("	creating eth0~eth%d...\n",totalVlans-1 );
#ifdef CONFIG_RTL865X_IGMP_SNOOPING
	memset(&mCastSnoopingGlobalConfig, 0, sizeof(struct rtl_mCastSnoopingGlobalConfig));
	mCastSnoopingGlobalConfig.maxGroupNum=256;
	mCastSnoopingGlobalConfig.maxSourceNum=300;
	mCastSnoopingGlobalConfig.hashTableSize=64;
	
	mCastSnoopingGlobalConfig.groupMemberAgingTime=260;
	mCastSnoopingGlobalConfig.lastMemberAgingTime=1;
	mCastSnoopingGlobalConfig.querierPresentInterval=260;  
	
	mCastSnoopingGlobalConfig.dvmrpRouterAgingTime=120;
	mCastSnoopingGlobalConfig.mospfRouterAgingTime=120;
	mCastSnoopingGlobalConfig.pimRouterAgingTime=120;
	
	igmpInitFlag=rtl_initMulticastSnooping(mCastSnoopingGlobalConfig);
#endif	

	for(i=0;i<totalVlans;i++)
	{
		struct net_device *dev;
		struct dev_priv	  *dp;
		int rc;

		if (IF_ETHER!=vlanconfig[i].if_type)
		{
			continue;
		}	
		dev = alloc_etherdev(sizeof(struct dev_priv));
		if (!dev){
			printk("failed to allocate dev %d", i);
			return -1;
		}
		SET_MODULE_OWNER(dev);
		dp = dev->priv;
		memset(dp,0,sizeof(*dp));
		dp->dev = dev;
		dp->id = vlanconfig[i].vid;
		dp->portmask =  vlanconfig[i].memPort;
		dp->portnum  = 0;
		for(j=0;j<RTL8651_AGGREGATOR_NUMBER;j++){
			if(dp->portmask & (1<<j))
				dp->portnum++;
		}
		
		memcpy((char*)dev->dev_addr,(char*)(&(vlanconfig[i].mac)),ETHER_ADDR_LEN);
		dev->open = re865x_open;
		dev->stop = re865x_close;
		dev->set_multicast_list = re865x_set_rx_mode;
		dev->hard_start_xmit = re865x_start_xmit;
		dev->get_stats = re865x_get_stats;
		dev->do_ioctl = re865x_ioctl;
		dev->tx_timeout = re865x_tx_timeout;
		dev->set_mac_address = rtl865x_set_hwaddr;
		dev->change_mtu = rtl865x_set_mtu;
#if defined(CONFIG_RTL8186_LINK_CHANGE)
		dev->change_link = rtl865x_set_link;
#endif
		dev->watchdog_timeo = TX_TIMEOUT;
#if 0 
		dev->features |= NETIF_F_SG | NETIF_F_IP_CSUM;
#endif
#ifdef CP_VLAN_TAG_USED
		dev->features |= NETIF_F_HW_VLAN_TX | NETIF_F_HW_VLAN_RX;
		dev->vlan_rx_register = cp_vlan_rx_register;
		dev->vlan_rx_kill_vid = cp_vlan_rx_kill_vid;
#endif
		dev->irq = ICU_NIC;
		rc = register_netdev(dev);
		if(!rc){
			_rtl86xx_dev.dev[i]=dev;
/*2007-12-19*/
			rtlglue_printf("eth%d added. vid=%d Member port 0x%x...\n", i,vlanconfig[i].vid ,vlanconfig[i].memPort );
		}else
			rtlglue_printf("Failed to allocate eth%d\n", i);

if(bridgemode!=1)
{	
#if defined(CONFIG_RTK_VLAN_SUPPORT)

		res_stats_root = proc_mkdir(dev->name, NULL);
		if (res_stats_root == NULL) 
		{
			printk("proc_mkdir failed!\n");		
		}
	
		if ((res_stats = create_proc_read_entry("mib_vlan", 0644, res_stats_root,
			read_proc_vlan, (void *)dev)) == NULL) 
		{
		printk("create_proc_read_entry failed!\n");		
		}
		res_stats->write_proc = write_proc_vlan;	

#endif
}


#ifdef CONFIG_RTL865X_IGMP_SNOOPING
		if(igmpInitFlag==SUCCESS)
		{
			if((vlanconfig[i].isWan!=TRUE) && (vlanconfig[i].if_type==IF_ETHER))
			{
				retVal=rtl_registerIgmpSnoopingModule(&dp->igmpModuleIndex);
				#ifdef CONFIG_RTL865X_HARDWARE_MULTICAST
				if(retVal==SUCCESS)
				{
					rtl_multicastDeviceInfo_t devInfo;
					strcpy(devInfo.devName, dp->dev->name);
					devInfo.portMask=dp->portmask;
					devInfo.swPortMask=devInfo.portMask & (~ ((1<<RTL8651_MAC_NUMBER)-1));
					rtl_setIgmpSnoopingModuleDevInfo(dp->igmpModuleIndex, &devInfo);
				}
				#endif
			}
		}
#endif

	}
	
#ifdef CONFIG_RTL865X_IGMP_SNOOPING
	curLinkPortMask=rtl865x_getLANLinkStatus();
	
	#ifdef CONFIG_RTL865X_HARDWARE_MULTICAST
	memset(&mCastConfig, 0, sizeof(rtl865x_mCastConfig_t));
	for(i=0;i<totalVlans;i++)
	{
		if (TRUE==vlanconfig[i].isWan)
		{
			mCastConfig.externalPortMask |=vlanconfig[i].memPort;
		}
	}
	rtl865x_initMulticast(&mCastConfig);
	#endif

#endif


#ifdef CONFIG_RTL_STP
	#ifdef CONFIG_RTK_MESH
	printk("Configuration LINUX to process port 0 ~ port %d for Spanning tree process\n", MAX_RE865X_ETH_STP_PORT-1);
	#else
	printk("Configuration LINUX to process port 0 ~ port %d for Spanning tree process\n", MAX_RE865X_STP_PORT-2);
	#endif
	rtl865x_setSpanningEnable(TRUE);
	#ifdef CONFIG_RTK_MESH
	for ( i = 0 ; i < MAX_RE865X_ETH_STP_PORT; i ++ )
	#else
	for ( i = 0 ; i < MAX_RE865X_STP_PORT-1 ; i ++ )
	#endif
	{
		struct net_device *dev;
		struct dev_priv *dp;
		int rc;
		struct re865x_priv *rp;
		
		rp = &_rtl86xx_dev;
		dev = alloc_etherdev(sizeof(struct dev_priv));
		if (!dev){
			rtlglue_printf("failed to allocate dev %d", i);
			return -1;
		}
		strcpy(dev->name, "port%d");	
		memcpy((char*)dev->dev_addr,(char*)(&(vlanconfig[0].mac)),ETHER_ADDR_LEN);
		dp = dev->priv;
		memset(dp,0,sizeof(*dp));
		dp->dev = dev;
		dev->open = re865x_stp_open;
		dev->stop = re865x_stp_close;
		dev->set_multicast_list = NULL;
		dev->hard_start_xmit = re865x_start_xmit;
		dev->get_stats = re865x_get_stats;
		dev->do_ioctl = re865x_ioctl;
		dev->tx_timeout = NULL;
		dev->watchdog_timeo = TX_TIMEOUT;
		dev->irq = 0;				/* virtual interfaces has no IRQ allocated */
		rc = register_netdev(dev);
		if (rc == 0)
		{
			_rtl86xx_dev.stp_port[i] = dev;
			printk("=> [stp pseudo port%d] done\n", i);
		} else
		{
			printk("=> Failed to register [stp pseudo port%d]", i);
			return -1;
		}
	}

	re865x_stp_mapping_init();

#endif
	((struct dev_priv*)(_rtl86xx_dev.dev[0]->priv))->dev_next = _rtl86xx_dev.dev[1];
	((struct dev_priv*)(_rtl86xx_dev.dev[1]->priv))->dev_prev = _rtl86xx_dev.dev[0];

#ifdef CONFIG_RTL865X_ETH_PRIV_SKB
	init_priv_eth_skb_buf();



#endif	
#if (defined(CONFIG_RTL8186_GR) && !defined(CONFIG_RTL8196_RTL8366)) 
	//cary
	rtl8651_customPassthru_init();
	rtl8651_initStormCtrl();
#endif
#ifdef CONFIG_RTL865X_LAYERED_ASIC_DRIVER_L3
#ifdef CONFIG_RTL865X_HARDWARE_MULTICAST
	rtl8651_setAsicMulticastEnable(TRUE);
#else
	rtl8651_setAsicMulticastEnable(FALSE);
#endif
#endif
if(bridgemode!=1)
{
#if defined(CONFIG_RTK_VLAN_SUPPORT)

	rtk_vlan_support_enable= 0;
	rtk_vlan_support_entry=create_proc_entry("rtk_vlan_support",0,NULL);
	if (rtk_vlan_support_entry) 
	{
	    rtk_vlan_support_entry->read_proc=rtk_vlan_support_read;
	    rtk_vlan_support_entry->write_proc=rtk_vlan_support_write;
	}

  /*
    rtk_vlan_netmask_entry=create_proc_entry("rtk_vlan_netmask",0,NULL);
    if (rtk_vlan_netmask_entry)
    {
        rtk_vlan_netmask_entry->read_proc=rtk_vlan_netmask_read;
        rtk_vlan_netmask_entry->write_proc=rtk_vlan_netmask_write;
    }
 */
	vlan_num= 0;
	vlan_num_entry=create_proc_entry("amit_vlan_num",0,NULL);
	if (vlan_num_entry) 
	{
	    vlan_num_entry->read_proc=read_vlan_num;
	    vlan_num_entry->write_proc=write_vlan_num;
	}
#endif
}
	return 0;
}

#ifdef CONFIG_RTL865X_ETH_PRIV_SKB

//---------------------------------------------------------------------------
static void init_priv_eth_skb_buf(void)
{
	int i;

	DEBUG_ERR("Init priv skb.\n");
	memset(eth_skb_buf, '\0', sizeof(struct priv_skb_buf2)*(MAX_ETH_SKB_NUM));
	INIT_LIST_HEAD(&eth_skbbuf_list);

	for (i=0; i<MAX_ETH_SKB_NUM; i++)  {
		memcpy(eth_skb_buf[i].magic, ETH_MAGIC_CODE, 4);	
		INIT_LIST_HEAD(&eth_skb_buf[i].list);
		list_add_tail(&eth_skb_buf[i].list, &eth_skbbuf_list);	
        
	}
}

//---------------------------------------------------------------------------
static __inline__ unsigned char *get_buf_from_poll(struct list_head *phead, unsigned int *count)
{
	unsigned long flags;
	unsigned char *buf;
	struct list_head *plist;

	local_irq_save(flags);

	if (list_empty(phead)) {
		local_irq_restore(flags);
		DEBUG_ERR("eth_drv: phead=%X buf is empty now!\n", (unsigned int)phead);
		DEBUG_ERR("free count %d\n", *count);
		return NULL;
	}

	if (*count == 1) {
		local_irq_restore(flags);
		DEBUG_ERR("eth_drv: phead=%X under-run!\n", (unsigned int)phead);
		return NULL;
	}

	*count = *count - 1;
	plist = phead->next;
	list_del_init(plist);
	buf = (unsigned char *)((unsigned int)plist + sizeof (struct list_head));
	local_irq_restore(flags);
	return buf;
}

//---------------------------------------------------------------------------
static __inline__ void release_buf_to_poll(unsigned char *pbuf, struct list_head	*phead, unsigned int *count)
{
	unsigned long flags;
	struct list_head *plist;

	local_irq_save(flags);

	*count = *count + 1;
	plist = (struct list_head *)((unsigned int)pbuf - sizeof(struct list_head));
	list_add_tail(plist, phead);
	local_irq_restore(flags);
}

//---------------------------------------------------------------------------
__IRAM_GEN void free_rtl865x_eth_priv_buf(unsigned char *head)
{
	#ifdef DELAY_REFILL_ETH_RX_BUF
	extern int return_to_rx_pkthdr_ring(unsigned char *head);
	if (!return_to_rx_pkthdr_ring(head)) 
	#endif
		release_buf_to_poll(head, &eth_skbbuf_list, (unsigned int *)&eth_skb_free_num);	
}

#if CONFIG_RTL_DYNAMIC_IRAM_MAPPING_FOR_WAPI
__IRAM_GEN_WAPI void free_rtl865x_eth_priv_buf_wapi(unsigned char *head)
{
	#ifdef DELAY_REFILL_ETH_RX_BUF
	extern int return_to_rx_pkthdr_ring(unsigned char *head);
	if (!return_to_rx_pkthdr_ring(head)) 
	#endif
		release_buf_to_poll(head, &eth_skbbuf_list, (unsigned int *)&eth_skb_free_num);	
}

extern void (*p_free_rtl865x_eth_priv_buf)(unsigned char *head);
#endif

//---------------------------------------------------------------------------
static struct sk_buff *dev_alloc_skb_priv_eth(unsigned int size)
{
	struct sk_buff *skb;
	unsigned char *data; 
#if	defined(LOCK_RX_TX)
	RTL_NIC_LOCK(rtl_nic_spinlock);
#endif
	/* first argument is not used */
	data = get_buf_from_poll(&eth_skbbuf_list, (unsigned int *)&eth_skb_free_num);
	if (data == NULL) {
		DEBUG_ERR("eth_drv: priv skb buffer empty!\n");
#if	defined(LOCK_RX_TX)
		RTL_NIC_UNLOCK(rtl_nic_spinlock);
#endif
		return NULL;
	}
	skb = dev_alloc_8190_skb(data, size);
	if (skb == NULL) {
#if CONFIG_RTL_DYNAMIC_IRAM_MAPPING_FOR_WAPI

	#if defined(CONFIG_RTL_DYNAMIC_IRAM_MAPPING_FOR_WAPI_DEBUG)
		FUNCTION_CHECK(p_free_rtl865x_eth_priv_buf);
	#endif
		p_free_rtl865x_eth_priv_buf(data);
#else
		free_rtl865x_eth_priv_buf(data);
#endif
#if	defined(LOCK_RX_TX)
		RTL_NIC_UNLOCK(rtl_nic_spinlock);
#endif
		return NULL;
	}

#if	defined(LOCK_RX_TX)
	RTL_NIC_UNLOCK(rtl_nic_spinlock);
#endif
	return skb;
}

//---------------------------------------------------------------------------
int is_rtl865x_eth_priv_buf(unsigned char *head)
{
	unsigned long offset = (unsigned long)(&((struct priv_skb_buf2 *)0)->buf);
	struct priv_skb_buf2 *priv_buf = (struct priv_skb_buf2 *)(((unsigned long)head) - offset);

	if (!memcmp(priv_buf->magic, ETH_MAGIC_CODE, 4)) {
		return 1;	
	}
	else {
		return 0;
	}
}
//---------------------------------------------------------------------------
#if defined(CONFIG_NET_WIRELESS_AGN) || defined(CONFIG_NET_WIRELESS_AG)
#include <net/dst.h>

 //---------------------------------------------------------------------------
 struct sk_buff *priv_skb_copy(struct sk_buff *skb)
 {
         struct sk_buff *n;
 
         if (rx_skb_queue.qlen == 0) {
 #ifdef CONFIG_RTL865X_ETH_PRIV_SKB
                 n = dev_alloc_skb_priv_eth(CROSS_LAN_MBUF_LEN);
 #else
                 n  = dev_alloc_skb(CROSS_LAN_MBUF_LEN);
 #endif
         }
         else {
 #ifdef RTK_QUE
                 n = rtk_dequeue(&rx_skb_queue);
 #else
                 n = __skb_dequeue(&rx_skb_queue);
 #endif
         }
 
         if (n == NULL)
                 return NULL;
 
         /* Set the tail pointer and length */
         skb_put(n, skb->len);
         n->csum = skb->csum;
         n->ip_summed = skb->ip_summed;
         memcpy(n->data, skb->data, skb->len);
 
         copy_skb_header(n, skb);
         return n;
 }
 #endif // defined(CONFIG_NET_WIRELESS_AGN) || defined(CONFIG_NET_WIRELESS_AG)
#endif // CONFIG_RTL865X_ETH_PRIV_SKB

static void __exit re865x_exit (void)
{

#ifdef RTL865X_DRIVER_DEBUG_FLAG
	rtl865x_proc_debug_cleanup();
#endif

#if defined(CONFIG_PROC_FS) && defined(CONFIG_NET_QOS) && defined(CONFIG_RTL865X_LAYERED_DRIVER)
#ifdef CONFIG_RTL865X_HW_QOS_SUPPORT
	rtl865x_exitOutputQueue();
#endif
#endif
	
#ifdef CONFIG_RTL865X_IGMP_SNOOPING
	rtl_exitMulticastSnooping();
#endif

	return;
}

module_init(re865x_probe);
module_exit(re865x_exit);

/*
@func enum RTL_RESULT | rtl865x_init | Initialize light rome driver and RTL865x ASIC.
@rvalue RTL_SUCCESS | Initial success. 
@comm
	Its important to call this API before using the driver. Note taht you can not call this API twice !
*/
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
int32 rtl865x_init(void)
#else
enum RTL_RESULT rtl865x_init(void)
#endif

{
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
	int32 retval = 0;
#endif

	__865X_Config = 0;
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
#else
	memset(&lr_stats, 0, sizeof(struct lr_cpu_stats));
#endif

#ifdef CONFIG_RTL8196_RTL8366
	/*	configure 8366 */
	{
		int ret;
		int i;
		rtl8366rb_phyAbility_t phy;

		REG32(PEFGHCNR_REG) = REG32(PEFGHCNR_REG)& (~(1<<11) ); //set byte F GPIO3 = gpio
		REG32(PEFGHDIR_REG) = REG32(PEFGHDIR_REG) | (1<<11);  //0 input, 1 output, set F bit 3 output
		REG32(PEFGHDAT_REG) = REG32(PEFGHDAT_REG) |( (1<<11) ); //F3 GPIO
		mdelay(150);

		ret = smi_init(GPIO_PORT_F, 2, 1);
//		printk("[%s][%d]:ret %d\n", __FUNCTION__, __LINE__, ret);
		ret = rtl8366rb_initChip();
//		ret = rtl8366rb_initChip();
		ret = rtl8366rb_initVlan();
//		ret = rtl8366rb_initAcl();
		ret = smi_write(0x0f09, 0x0020);
		ret = smi_write(0x0012, 0xe0ff);
#if 1
		memset(&phy, 0, sizeof(rtl8366rb_phyAbility_t));
		phy.Full_1000 = 1;
		phy.Full_100 = 1;
		phy.Full_10 = 1;
		phy.Half_100 = 1;
		phy.Half_10 = 1;
		phy.FC = 1;
		phy.AsyFC = 1;
		phy.AutoNegotiation = 1;
		for(i=0;i<5;i++)
		{
			ret = rtl8366rb_setEthernetPHY(i,&phy);
		}
#endif
	}

#if 1
 	 REG32(0xb8010000)=REG32(0xb8010000)&(0x20000000);
        REG32(0xbb80414c)=0x00037d16;
        REG32(0xbb804100)=1;
        REG32(0xbb804104)=0x00E80367;
#else		
	/*	configure 8196	*/
	{
		rtl865xC_setAsicEthernetRGMIITiming(0, RGMII_TCOMP_4NS, RGMII_RCOMP_2NS);
		WRITE_MEM32(PITCR, Port0_TypeCfg_GMII_MII_RGMII);
		WRITE_MEM32(PCRP0, 0xe80367);		/* force mode 1G */
	}
#endif
#endif

/*common*/
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
	retval = rtl865x_initNetifTable();
	retval = rtl865x_initVlanTable();
#ifdef CONFIG_RTL865X_LAYERED_DRIVER_ACL
	retval = rtl865x_init_acl();
#endif
	retval = rtl865x_initEventMgr(NULL);	
	//printk("%s(%d),retval(%d)\n",__FUNCTION__,__LINE__,retval);
	
#else
	LR_INIT_CHECK( TBLFIELD(vlan_tbl, vlan_init)() );
	LR_INIT_CHECK( TBLFIELD(acl_tbl, acl_init)() );	
#endif

/*l2*/
 #ifdef CONFIG_RTL865X_LAYERED_DRIVER
 #ifdef CONFIG_RTL865X_LAYERED_DRIVER_L2
	retval = rtl865x_layer2_init();
 #endif
 #else
 	LR_INIT_CHECK( TBLFIELD(fdb_tbl, fdb_init)() );
 #endif
 
/*layer3*/
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
#ifdef CONFIG_RTL865X_LAYERED_DRIVER_L3
	retval = rtl865x_initIpTable();
	retval = rtl865x_initPppTable();
	retval = rtl865x_initRouteTable();
	retval = rtl865x_initNxtHopTable();
	retval = rtl865x_arp_init();
#endif
#else
	LR_INIT_CHECK( TBLFIELD(if_tbl, if_init)() );	
	LR_INIT_CHECK( TBLFIELD(natip_tbl, natip_init)() );	
	LR_INIT_CHECK( TBLFIELD(rt_tbl, route_init)() );	
	LR_INIT_CHECK( TBLFIELD(arpt_tbl, arp_init)() );	
#endif

/*layer4*/
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
#ifdef CONFIG_RTL865X_LAYERED_DRIVER_L4
	//nat_tbl.nat_init();
	rtl865x_nat_init();
#endif
#else
	LR_INIT_CHECK( TBLFIELD(nat_tbl, nat_init)() );	
#endif

 
	/*LR_INIT_CHECK( TBLFIELD(pppoe_tbl, pppoe_init)() );*/
 #ifdef CONFIG_RTL865X_LAYERED_DRIVER
 	//INIT_CHECK( rtl8651_regLinkChangeCallBackFun(port_attr.link_change) );
 #else
	LR_INIT_CHECK( rtl8651_regLinkChangeCallBackFun(port_attr.link_change) );
 #endif

 #ifdef RTL865X_MULTIPLE_RX_RING
 	/*queue id & rx ring descriptor mapping*/
 	REG32(0xb8010030)=0x01000;
 	REG32(0xb8010034)=0x20003000;
	REG32(0xb8010038)=0x40005000;
 #endif

#ifdef RTL865X_DRIVER_DEBUG_FLAG
	rtl865x_proc_debug_init();
#endif

#if CONFIG_RTL_DYNAMIC_IRAM_MAPPING_FOR_WAPI
	switch_iram_init();
#endif

#ifdef CONFIG_RTL865X_LAYERED_DRIVER
	return SUCCESS;
#else
	return RTL_SUCCESS;
#endif
}



/*
@func enum RTL_RESULT | rtl865x_config | Configure light rome driver. Create VLAN and Network interface.
@parm struct rtl865x_vlanConfig * | vlanconfig | 
@rvlaue RTL_SUCCESS | Sucessful configuration.
@rvalue RTL_INVVID | Invalid VID.
@comm
	struct rtl865x_vlanConfig is defined as follows:
	
			ifname:		Layer 3 Network Interface name, eg: eth0, eth1, ppp0...etc,. If it is specified, both layer 2 vlan and layer 3
						netwrok interface are created and bound together. It also can be a NULL value. In this case, only a layer 2 VLAN 
						is created.
			isWan:		1 for WAN interface and 0 for LAN interface in a layer 4 mode.
			if_type:		IF_ETHER sets a network interface to be ETHER type. Instead, IF_PPPOE sets a netwrok to be PPPoE type.
						This field is meaningful only when the ifname is specified.
			vid:			VLAN ID to create a vlan.
			memPort:		VLAN member port.
			untagSet:	VLAN untag Set.
			mtu:			MTU.
			mac:		MAC address of the VLAN or network interface.	
	eg1:

	struct rtl865x_vlanConfig vlanconfig[] = {
		{ 	"eth0",	 1,   IF_ETHER, 	8, 	   1, 	0x01, 		0x01,		1500, 	{ { 0x00, 0x00, 0xda, 0xcc, 0xcc, 0x08 } }	},
		{	"eth1",	 0,   IF_ETHER,	9,	   1,		0x1e,		0x1e,		1500,	{ { 0x00, 0x00, 0xda, 0xcc, 0xcc, 0x09 } }	},

		LRCONFIG_END,
	}
*/
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
int32 rtl865x_config(struct rtl865x_vlanConfig vlanconfig[])
#else
enum RTL_RESULT rtl865x_config(struct rtl865x_vlanConfig vlanconfig[])
#endif

{

#ifdef CONFIG_RTL865X_LAYERED_DRIVER	
#else
	enum RTL_RESULT rc;
#endif

	uint16 pvid;
	int32 i, j;
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
	int32 retval = 0;
#endif

	if (!vlanconfig[0].vid)
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
		return RTL_EINVALIDVLANID;
#else
		return RTL_INVVID;
#endif

#ifdef CONFIG_RTL865X_LAYERED_DRIVER
	INIT_CHECK(rtl8651_setAsicOperationLayer(2));
#else
	LR_INIT_CHECK(rtl8651_setAsicOperationLayer(2));
#endif
	
	for(i=0; vlanconfig[i].vid != 0; i++)
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
	{
		rtl865x_netif_t netif;

		/*add vlan*/
		retval = rtl865x_addVlan(vlanconfig[i].vid);

		if(retval == SUCCESS)
		{
			rtl865x_addVlanPortMember(vlanconfig[i].vid,vlanconfig[i].memPort);
#ifdef CONFIG_RTL8196_RTL8366
			rtl865x_setVlanPortTag(vlanconfig[i].vid,vlanconfig[i].memPort, TRUE);
#endif
			rtl865x_setVlanFilterDatabase(vlanconfig[i].vid,vlanconfig[i].fid);			
		
		}
		/*add network interface*/
		memset(&netif, 0, sizeof(rtl865x_netif_t));
		memcpy(netif.name,vlanconfig[i].ifname,MAX_IFNAMESIZE);
		netif.macAddr = vlanconfig[i].mac;
		netif.mtu = vlanconfig[i].mtu;
		netif.if_type = vlanconfig[i].if_type;
		netif.vid = vlanconfig[i].vid;
		netif.is_wan = vlanconfig[i].isWan;
		netif.is_slave = vlanconfig[i].is_slave;
		#ifdef CONFIG_RTL865X_HARDWARE_MULTICAST
		netif.enableRoute=1;
		#endif
		retval = rtl865x_addNetif(&netif);

		if(netif.is_slave == 1)
			rtl865x_attachMasterNetif(netif.name, "eth1");
#if defined(CONFIG_PROC_FS) && defined(CONFIG_NET_QOS) && defined(CONFIG_RTL865X_LAYERED_DRIVER)
		memcpy(&netIfName[i][0], vlanconfig[i].ifname, sizeof(vlanconfig[i].ifname));
#endif

#if defined (CONFIG_RTL_UNKOWN_UNICAST_CONTROL)
		if (vlanconfig[i].isWan==0)
			memcpy(lanIfName, vlanconfig[i].ifname, sizeof(vlanconfig[i].ifname));
#endif
		if(retval != SUCCESS && retval != RTL_EVLANALREADYEXISTS)
			return retval;
		
	}
#else
	{
		if (vlanconfig[i].ifname[0] != '\0')
		{
			rc = (TBLFIELD(if_tbl, if_attach)(&vlanconfig[i]));
			if (rc != RTL_SUCCESS)
				return rc;
		}
		
		rc = TBLFIELD(vlan_tbl, vlan_create)(&vlanconfig[i]);
		if (rc != RTL_SUCCESS && rc != RTL_DUPENTRY)
			return rc;
	}
#endif
 
#ifdef CONFIG_RTL8196_RTL8366
	{
		/* for lan */
		int iport;
		rtl8366rb_vlanConfig_t vlancfg_8366;

		memset(&vlancfg_8366, 0, sizeof(rtl8366rb_vlanConfig_t));
		vlancfg_8366.fid = 0;
		vlancfg_8366.mbrmsk = (RTL8366RB_LANPORT&RTL8366RB_PORTMASK)|RTL8366RB_GMIIPORT;
		vlancfg_8366.untagmsk = vlancfg_8366.mbrmsk&(~RTL8366RB_GMIIPORT);
		vlancfg_8366.vid = RTL_LANVLANID;
		retval = rtl8366rb_setVlan(&vlancfg_8366);
		/*need set pvid??*/
		for(iport=0;iport<8;iport++)
			if  ((1<<iport)&vlancfg_8366.mbrmsk)
			{
				retval = rtl8366rb_setVlanPVID(iport, vlancfg_8366.vid, 0);
			}
	}
	{
		/* for wan */
		int iport;
		rtl8366rb_vlanConfig_t vlancfg_8366;

		vlancfg_8366.fid = 1;
		vlancfg_8366.mbrmsk = (RTL8366RB_WANPORT&RTL8366RB_PORTMASK)|RTL8366RB_GMIIPORT;
		vlancfg_8366.untagmsk = vlancfg_8366.mbrmsk&(~RTL8366RB_GMIIPORT);
		vlancfg_8366.vid = RTL_WANVLANID;
		retval = rtl8366rb_setVlan(&vlancfg_8366);
		/*need set pvid??*/
		for(iport=0;iport<8;iport++)
			if  ((1<<iport)&vlancfg_8366.mbrmsk)
			{
				retval = rtl8366rb_setVlanPVID(iport, vlancfg_8366.vid, 0);
			}
	}
#endif		
	/*this is a one-shot config*/
	if ((++__865X_Config) == 1)
	{
		for(i=0; i<RTL8651_PORT_NUMBER + 3; i++) 
		{ 
			/* Set each port's PVID */
			for(j=0,pvid=0; vlanconfig[j].vid != 0; j++)
			{
				if ( (1<<i) & vlanconfig[j].memPort ) 
				{
					pvid = vlanconfig[j].vid;
					break;
				}
			}
			
			if (pvid!=0)	
			{
	#ifdef CONFIG_HARDWARE_NAT_DEBUG
	/*2007-12-19*/
			rtlglue_printf("%s:%d:lrconfig[j].vid is %d,pvid is %d, j is %d,i is %d\n",__FUNCTION__,__LINE__,vlanconfig[j].vid,pvid,j, i);
	#endif
	
	#ifdef CONFIG_RTL865X_LAYERED_DRIVER	
			CONFIG_CHECK(rtl8651_setAsicPvid(i, pvid));
	if(bridgemode!=1)
	{
	#if defined(CONFIG_RTK_VLAN_SUPPORT)
			rtl865x_setPortToNetif(vlanconfig[j].ifname,i);
	#endif
	}
	#else
			LR_CONFIG_CHECK(port_attr.pvid_asic_set(i, pvid));
	#endif
			}
		}
	}

//#if defined(CONFIG_PROC_FS) && defined(CONFIG_NET_QOS) && defined(CONFIG_RTL865X_LAYERED_DRIVER)
#if defined (CONFIG_RTL865X_HW_QOS_SUPPORT)
	rtl865x_initOutputQueue((uint8 **)netIfName);
#endif
//#endif

#if defined (CONFIG_RTL_UNKOWN_UNICAST_CONTROL)
	{
		rtl865x_tblAsicDrv_rateLimitParam_t	asic_rl;
		/* 
	  	* Designer said: The time unit used to achieve rate limit is 1.67s (5/3), hence here we change 
	  	* the time unit to 1 sec.
	  	*/
		bzero(&asic_rl, sizeof(rtl865x_tblAsicDrv_rateLimitParam_t));
		asic_rl.maxToken			= RTL_MAC_REFILL_TOKEN;
		asic_rl.refill_number		= RTL_MAC_REFILL_TOKEN;
		asic_rl.t_intervalUnit		= 1; 
		asic_rl.t_remainUnit		= 1;
		asic_rl.token				= RTL_MAC_REFILL_TOKEN;
		rtl8651_setAsicRateLimitTable(0, &asic_rl);

		macRecordIdx = 0;
		bzero(macRecord, RTL_MAC_RECORD_NUM*sizeof(rtlMacRecord));

		for(i=0;i<RTL_MAC_RECORD_NUM;i++)
		{
			init_timer(&macRecord[i].timer);
			macRecord[i].timer.function = rtl_unkownUnicastTimer;
		}

		WRITE_MEM32(TEACR, (READ_MEM32(TEACR)|EnRateLimitTbAging));
	}
#endif

#ifdef CONFIG_RTL865X_LAYERED_DRIVER
	return SUCCESS;
#else
	return RTL_SUCCESS;	
#endif
}

#if defined (CONFIG_RTL_UNKOWN_UNICAST_CONTROL)
static void rtl_unkownUnicastTimer(unsigned long data)
{
	rtlMacRecord	*record;
	rtl865x_AclRule_t	rule;

	record = (rtlMacRecord*)data;
	bzero((void*)&rule,sizeof(rtl865x_AclRule_t));
	rule.ruleType_ = RTL865X_ACL_MAC;
	rule.pktOpApp_ = RTL865X_ACL_ONLY_L2;
	rule.actionType_ = RTL865X_ACL_DROP_RATE_EXCEED_PPS;
	memset(rule.dstMacMask_.octet, 0xff, ETHER_ADDR_LEN);
	memcpy(rule.dstMac_.octet, record->mac, ETHER_ADDR_LEN);
	rtl865x_del_acl(&rule, lanIfName, RTL865X_ACL_SYSTEM_USED);

	bzero(record, sizeof(rtlMacRecord));
	init_timer(&record->timer);
	record->timer.function = rtl_unkownUnicastTimer;
}

static void	rtl_unkownUnicastUpdate(uint8 *mac)
{
	int	idx;
	rtl865x_AclRule_t	rule;

	for(idx=0;idx<RTL_MAC_RECORD_NUM;idx++)
	{
		if (macRecord[idx].enable==0||memcmp(mac, macRecord[idx].mac, ETHER_ADDR_LEN))
			continue;

		/*	The mac has already recorded	*/
		if (macRecord[idx].cnt==RTL_MAC_THRESHOLD||++macRecord[idx].cnt<RTL_MAC_THRESHOLD)
			return;

		break;
	}

	/*	add/del the rules at lan side */
	bzero((void*)&rule,sizeof(rtl865x_AclRule_t));
	rule.ruleType_ = RTL865X_ACL_MAC;
	rule.pktOpApp_ = RTL865X_ACL_ONLY_L2;
	rule.actionType_ = RTL865X_ACL_DROP_RATE_EXCEED_PPS;
	memset(rule.dstMacMask_.octet, 0xff, ETHER_ADDR_LEN);

	if (idx==RTL_MAC_RECORD_NUM)
	{
		if (macRecord[macRecordIdx].enable!=0&&macRecord[macRecordIdx].cnt>RTL_MAC_THRESHOLD)
		{
			memcpy(rule.dstMac_.octet, macRecord[macRecordIdx].mac, ETHER_ADDR_LEN);
			rtl865x_del_acl(&rule, lanIfName, RTL865X_ACL_SYSTEM_USED);
			init_timer(&macRecord[macRecordIdx].timer);
			macRecord[macRecordIdx].timer.function = rtl_unkownUnicastTimer;
		}
		else
		{
			macRecord[macRecordIdx].enable = 1;
		}
		macRecord[macRecordIdx].cnt = 0;
		memcpy(macRecord[macRecordIdx].mac, mac, ETHER_ADDR_LEN);
		macRecordIdx = (macRecordIdx+1)&(RTL_MAC_RECORD_NUM-1);
	}
	else
	{
		memcpy(rule.dstMac_.octet, mac, ETHER_ADDR_LEN);
		rtl865x_add_acl(&rule, lanIfName, RTL865X_ACL_SYSTEM_USED);
		macRecord[idx].timer.data = (unsigned long)&(macRecord[idx]);
		mod_timer(&macRecord[idx].timer, jiffies+RTL_MAC_TIMEOUT);
	}
}
#endif

#ifdef CONFIG_RTL865X_IGMP_SNOOPING

static int re865x_reInitIgmpSetting(int mode)
{
	struct dev_priv	  *dp;
	#ifdef CONFIG_RTL865X_HARDWARE_MULTICAST
	rtl_multicastDeviceInfo_t devInfo;
	uint32 externalPortMask=0;
	#endif	
	int32 i;
	int32 totalVlans=((sizeof(vlanconfig))/(sizeof(struct rtl865x_vlanConfig)))-1;
	
	for(i=0; i<totalVlans; i++)
	{
		#ifdef CONFIG_RTL865X_HARDWARE_MULTICAST
		if (TRUE==vlanconfig[i].isWan)
		{
			externalPortMask |=vlanconfig[i].memPort;
		}
		else
		{
			devInfo.portMask|=vlanconfig[i].memPort ;
		}
		#endif
		if ((IF_ETHER==vlanconfig[i].if_type) && (TRUE != vlanconfig[i].isWan))
		{
			dp =(struct dev_priv	  *)_rtl86xx_dev.dev[i]->priv;
			#ifdef CONFIG_RTL865X_HARDWARE_MULTICAST
			memset(&devInfo,0,sizeof(rtl_multicastDeviceInfo_t));
			strcpy(devInfo.devName, dp->dev->name);
			devInfo.portMask=dp->portmask;
			devInfo.swPortMask=dp->portmask & (~ ((1<<RTL8651_MAC_NUMBER)-1));
			rtl_setIgmpSnoopingModuleDevInfo(dp->igmpModuleIndex, &devInfo);
			#endif
			
			if(mode==GATEWAY_MODE)
			{
				rtl_setIgmpSnoopingModuleStaticRouterPortMask(dp->igmpModuleIndex, 0);
			}
			else
			{
				rtl_setIgmpSnoopingModuleStaticRouterPortMask(dp->igmpModuleIndex, 0x01);
			}
	
			rtl_setIgmpSnoopingModuleUnknownMCastFloodMap(dp->igmpModuleIndex, 0x0);
			
			
		}
		
	}
	
	#ifdef CONFIG_RTL865X_HARDWARE_MULTICAST
	rtl865x_reinitMulticast();
	//rtl865x_setMulticastExternalPortMask(externalPortMask);
	rtl865x_setMulticastExternalPortMask(0);
	#endif
	rtl865x_igmpSyncLinkStatus();
	return SUCCESS;
}
#endif

#ifdef CONFIG_RTL865X_LAYERED_DRIVER
int32 rtl865x_changeOpMode(int mode)
#else
enum RTL_RESULT rtl865x_changeOpMode(int mode)
#endif
{
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
	int32 retval = 0;
#else
	enum RTL_RESULT rc;
#endif

	uint16 pvid;
	int32 i, j;
	struct net_device *dev;
	struct dev_priv	  *dp;

//#if !defined(CONFIG_RTK_VLAN_SUPPORT)
#if 1
	int32 totalVlans;
	totalVlans = ((sizeof(vlanconfig))/(sizeof(struct rtl865x_vlanConfig)))-1;
#endif

#if 0 //defined(CONFIG_RTK_VLAN_SUPPORT)
	{
		if(rtk_vlan_support_enable == 0)
		{
			vlanconfig[0].memPort = RTL_LANPORT_MASK;
			((struct dev_priv *)_rtl86xx_dev.dev[0]->priv)->portmask = RTL_LANPORT_MASK; //eth0
			vlanconfig[2] .vid = 0;
			vlanconfig[3] .vid = 0;
			vlanconfig[4] .vid = 0;			
		}
		else
		{
			vlanconfig[0].vid = RTL_LANVLANID;
			vlanconfig[0].memPort = RTL_LANPORT_MASK_4;
			vlanconfig[0].untagSet= RTL_LANPORT_MASK_4;			
			((struct dev_priv *)_rtl86xx_dev.dev[0]->priv)->portmask = RTL_LANPORT_MASK_4; //eth0
			((struct dev_priv *)_rtl86xx_dev.dev[0]->priv)->id = RTL_LANVLANID; //eth0
			vlanconfig[2] .vid = RTL_LANVLANID_2;
			vlanconfig[2].memPort = RTL_LANPORT_MASK_3;
			vlanconfig[2].untagSet = RTL_LANPORT_MASK_3;
			((struct dev_priv *)_rtl86xx_dev.dev[2]->priv)->portmask = RTL_LANPORT_MASK_3; //eth2
			((struct dev_priv *)_rtl86xx_dev.dev[3]->priv)->id = RTL_LANVLANID_2; //eth2			
			vlanconfig[3] .vid = RTL_LANVLANID_3;
			vlanconfig[3].memPort = RTL_LANPORT_MASK_2;
			vlanconfig[3].untagSet = RTL_LANPORT_MASK_2;
			((struct dev_priv *)_rtl86xx_dev.dev[3]->priv)->portmask = RTL_LANPORT_MASK_2; //eth3
			((struct dev_priv *)_rtl86xx_dev.dev[3]->priv)->id = RTL_LANVLANID_3; //eth3
			
			vlanconfig[4] .vid = RTL_LANVLANID_4;
			vlanconfig[4].memPort = RTL_LANPORT_MASK_1;
			vlanconfig[4].untagSet = RTL_LANPORT_MASK_1;
			((struct dev_priv *)_rtl86xx_dev.dev[3]->priv)->portmask = RTL_LANPORT_MASK_1; //eth4
			((struct dev_priv *)_rtl86xx_dev.dev[3]->priv)->id = RTL_LANVLANID_4; //eth4			
		}
	}
#endif
	printk("rtl865x_curOpMode = %d mode:%d\n",rtl865x_curOpMode,mode);

	if(mode==rtl865x_curOpMode)
	{
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
		return SUCCESS;
#else
		return RTL_SUCCESS;
#endif
	}

if(bridgemode!=1)
{
#if defined(CONFIG_RTK_VLAN_SUPPORT)
	//hyking:keep vlanconfig as before if vlan support is disable...2009_08_07
	if(!rtk_vlan_support_enable)
	{
		/*lan config*/
		{

			vlanconfig[2].vid = 0; //eth2
			vlanconfig[3].vid = 0; //eth3
			vlanconfig[4].vid = 0; //eth4

		}
		
		{                                 
			dev=_rtl86xx_dev.dev[0];
			dp = dev->priv;
			vlanconfig[0].vid=RTL_LANVLANID;
			dp->id = RTL_LANVLANID;
			if(mode==GATEWAY_MODE)
			{
				vlanconfig[0].memPort=rtl865x_lanPortMask;
				vlanconfig[0].untagSet=rtl865x_lanPortMask;
				dp->portmask = rtl865x_lanPortMask;				
			}
			else if(mode==BRIDGE_MODE)
			{
				vlanconfig[0].memPort=0x1ff;
				vlanconfig[0].untagSet=0x1ff;
				dp->portmask = 0x1ff;
			}
			else if(mode==WISP_MODE)
			{
				vlanconfig[0].memPort=0x1ff;
				vlanconfig[0].untagSet=0x1ff;
				dp->portmask = 0x1ff;
			}
			#ifdef CONFIG_RTL865X_LAYERED_DRIVER
			else if(mode==MULTIPLE_VLAN_BRIDGE_MODE)
			{
				vlanconfig[0].memPort=rtl865x_lanPortMask;
				vlanconfig[0].untagSet=rtl865x_lanPortMask;
				dp->portmask = rtl865x_lanPortMask;
			}
			else if(mode==MULTIPLE_VLAN_WISP_MODE)
			{
				vlanconfig[0].memPort=rtl865x_lanPortMask;
				vlanconfig[0].untagSet=rtl865x_lanPortMask;
				dp->portmask = rtl865x_lanPortMask;
			}
			#endif
#if 0		
            else if(mode==VLAN_BRIDGE_MODE)
            {

	            vlanconfig[2].vid = 10; //eth2
    	        vlanconfig[3].vid = 11; //eth3
				      vlanconfig[4].vid = 12; //eth4

                printk("mode==VLAN_BRIDGE_MODE\n");
                printk("netmask %x %x %x %x\n",net_mask[0],net_mask[1],net_mask[2],net_mask[3]);

            	vlanconfig[0].memPort = net_mask[0];
            	vlanconfig[0].untagSet= net_mask[0];
            	((struct dev_priv *)_rtl86xx_dev.dev[0]->priv)->portmask = net_mask[0]; //eth0
//          	((struct dev_priv *)_rtl86xx_dev.dev[0]->priv)->id = RTL_LANVLANID; //eth0
/*
            	vlanconfig[1].vid = RTL_WANVLANID;
            	vlanconfig[1].memPort = RTL_WANPORT_MASK;
            	vlanconfig[1].untagSet= RTL_WANPORT_MASK;
            	((struct dev_priv *)_rtl86xx_dev.dev[1]->priv)->portmask = RTL_WANPORT_MASK; //eth1
            	((struct dev_priv *)_rtl86xx_dev.dev[1]->priv)->id = RTL_WANVLANID; //eth1
*/
            	vlanconfig[2].memPort = net_mask[1];
            	vlanconfig[2].untagSet = net_mask[1];
            	((struct dev_priv *)_rtl86xx_dev.dev[2]->priv)->portmask = net_mask[1]; //eth2
//          	((struct dev_priv *)_rtl86xx_dev.dev[2]->priv)->id = RTL_LANVLANID_2; //eth2


            	vlanconfig[3].memPort = net_mask[2];
            	vlanconfig[3].untagSet = net_mask[2];
            	((struct dev_priv *)_rtl86xx_dev.dev[3]->priv)->portmask = net_mask[2]; //eth3
//          	((struct dev_priv *)_rtl86xx_dev.dev[3]->priv)->id = RTL_LANVLANID_3; //eth3

            	vlanconfig[4].memPort = net_mask[3];
            	vlanconfig[4].untagSet = net_mask[3];
            	((struct dev_priv *)_rtl86xx_dev.dev[4]->priv)->portmask = net_mask[3]; //eth4
//          	((struct dev_priv *)_rtl86xx_dev.dev[4]->priv)->id = RTL_LANVLANID_4; //eth4

			       	mode=GATEWAY_MODE;
            }
#endif
            else if(mode==VLAN_BRIDGE_MODE || VLAN_BRIDGE_MODE_1 || VLAN_BRIDGE_MODE_2 || VLAN_BRIDGE_MODE_3)
            {
				unsigned int  vlan_vid[4]= {0};
              	unsigned int  net_mask[4]= {0};
              	switch(mode)
              	{
                	case VLAN_BRIDGE_MODE:
              			printk("mode==VLAN_BRIDGE_MODE\n");
              			net_mask[0] = 0x11e ; 
              			net_mask[1] = 0x11e ;
              			net_mask[2] = 0x11e ;
              			net_mask[3] = 0x11e;
              			
						vlan_vid[0] = 9;
              			vlan_vid[1] = 0;
              			vlan_vid[2] = 0; 
              			vlan_vid[3] = 0;
             			 break;
              
              		case VLAN_BRIDGE_MODE_1:
                  		printk("mode==VLAN_BRIDGE_MODE_1\n");
              			net_mask[0] = 0x11c ; 
              			net_mask[1] = 0x02 ;
              			net_mask[2] = 0x11c ;
              			net_mask[3] = 0x11c;
              			vlan_vid[0] = 9;
              			vlan_vid[1] = 10;
              			vlan_vid[2] = 0; 
              			vlan_vid[3] = 0;
              			break; 


                    case VLAN_BRIDGE_MODE_2:
                        printk("mode==VLAN_BRIDGE_MODE_2\n");
                        net_mask[0] = 0x118 ;
                        net_mask[1] = 0x02 ;
                        net_mask[2] = 0x04 ;
                        net_mask[3] = 0x118;
                        vlan_vid[0] = 9;
                        vlan_vid[1] = 10;
                        vlan_vid[2] = 11;
                        vlan_vid[3] = 0;
                        break;

                    case VLAN_BRIDGE_MODE_3:
                        printk("mode==VLAN_BRIDGE_MODE_3\n");
                        net_mask[0] = 0x110 ;
                        net_mask[1] = 0x02 ;
                        net_mask[2] = 0x04 ;
                        net_mask[3] = 0x08;
                        vlan_vid[0] = 9;
                        vlan_vid[1] = 10;
                        vlan_vid[2] = 11;
                        vlan_vid[3] = 12;
                        break;
              		default:
              			break;
				}            
                
           		vlanconfig[0].vid = vlan_vid[0];
				vlanconfig[0].memPort = net_mask[0];
            	vlanconfig[0].untagSet= net_mask[0];
            	((struct dev_priv *)_rtl86xx_dev.dev[0]->priv)->portmask = net_mask[0]; //eth0
          		((struct dev_priv *)_rtl86xx_dev.dev[0]->priv)->id = vlan_vid[0]; //eth0
				
				vlanconfig[2].vid = vlan_vid[1];
            	vlanconfig[2].memPort = net_mask[1];
            	vlanconfig[2].untagSet = net_mask[1];
            	((struct dev_priv *)_rtl86xx_dev.dev[2]->priv)->portmask = net_mask[1]; //eth2
          		((struct dev_priv *)_rtl86xx_dev.dev[2]->priv)->id = vlan_vid[1]; //eth2

				vlanconfig[3].vid = vlan_vid[2];
            	vlanconfig[3].memPort = net_mask[2];
            	vlanconfig[3].untagSet = net_mask[2];
            	((struct dev_priv *)_rtl86xx_dev.dev[3]->priv)->portmask = net_mask[2]; //eth3
          		((struct dev_priv *)_rtl86xx_dev.dev[3]->priv)->id = vlan_vid[2]; //eth3

				vlanconfig[4].vid = vlan_vid[3];
            	vlanconfig[4].memPort = net_mask[3];
            	vlanconfig[4].untagSet = net_mask[3];
            	((struct dev_priv *)_rtl86xx_dev.dev[4]->priv)->portmask = net_mask[3]; //eth4
          		((struct dev_priv *)_rtl86xx_dev.dev[4]->priv)->id = vlan_vid[3]; //eth4

			    mode=GATEWAY_MODE;
            }

		}

		/*wan config*/
		{	
			dev=_rtl86xx_dev.dev[1];
			dp = dev->priv;			
			if(mode==GATEWAY_MODE)
			{
				vlanconfig[1].vid=RTL_WANVLANID;
				vlanconfig[1].memPort=rtl865x_wanPortMask;
				vlanconfig[1].untagSet=rtl865x_wanPortMask;

				dp->portmask = rtl865x_wanPortMask;
				dp->id = RTL_WANVLANID;
		
			}
			else if(mode==BRIDGE_MODE)
			{
				vlanconfig[1].vid=0;
				vlanconfig[1].memPort=0x0;
				vlanconfig[1].untagSet=0x0;
				dp->portmask =0;
				dp->id = 0;

				/*only one vlan in bridge mode*/
			
			}
			else if(mode==WISP_MODE)
			{
				vlanconfig[1].vid=0;
				vlanconfig[1].memPort=0x0;
				vlanconfig[1].untagSet=0x0;
				dp->portmask =0;
				dp->id = 0;

				/*only one vlan in bridge mode*/
				
			}
			#ifdef CONFIG_RTL865X_LAYERED_DRIVER
			else if(mode==MULTIPLE_VLAN_BRIDGE_MODE)
			{
				vlanconfig[1].vid=RTL_WANVLANID;
				vlanconfig[1].memPort=rtl865x_wanPortMask;
				vlanconfig[1].untagSet=rtl865x_wanPortMask;
				dp->portmask =rtl865x_wanPortMask;
				dp->id=RTL_WANVLANID;
			}
			else if(mode==MULTIPLE_VLAN_WISP_MODE)
			{
				vlanconfig[1].vid=RTL_WANVLANID;
				vlanconfig[1].memPort=rtl865x_wanPortMask;
				vlanconfig[1].untagSet=rtl865x_wanPortMask;
				dp->portmask =rtl865x_wanPortMask;
				dp->id=RTL_WANVLANID;
			}
			#endif
		}
	}
#else
	for(i=0; i<totalVlans; i++)
	{
		if (IF_ETHER!=vlanconfig[i].if_type)
		{
			continue;
		}
		
		dev=_rtl86xx_dev.dev[i];
		dp = dev->priv;
		
		if(vlanconfig[i].isWan==0)/*lan config*/
		{
			vlanconfig[i].vid=RTL_LANVLANID;
			if(mode==GATEWAY_MODE)
			{
				vlanconfig[i].memPort=rtl865x_lanPortMask;
				vlanconfig[i].untagSet=rtl865x_lanPortMask;
			}
			else if(mode==BRIDGE_MODE)
			{
				vlanconfig[i].memPort=0x1ff;
				vlanconfig[i].untagSet=0x1ff;
			}
			else if(mode==WISP_MODE)
			{
				vlanconfig[i].memPort=0x1ff;
				vlanconfig[i].untagSet=0x1ff;
			}
			#ifdef CONFIG_RTL865X_LAYERED_DRIVER
			else if(mode==MULTIPLE_VLAN_BRIDGE_MODE)
			{
				vlanconfig[i].memPort=rtl865x_lanPortMask;
				vlanconfig[i].untagSet=rtl865x_lanPortMask;
			}
			else if(mode==MULTIPLE_VLAN_WISP_MODE)
			{
				vlanconfig[i].memPort=rtl865x_lanPortMask;
				vlanconfig[i].untagSet=rtl865x_lanPortMask;
			}
			#endif
			
		}
		else/*wan config*/
		{	
			if(mode==GATEWAY_MODE)
			{
				vlanconfig[i].vid=RTL_WANVLANID;
				vlanconfig[i].memPort=rtl865x_wanPortMask;
				vlanconfig[i].untagSet=rtl865x_wanPortMask;

				/*there are two vlan in gateway mode*/
		
			}
			else if(mode==BRIDGE_MODE)
			{
				vlanconfig[i].vid=0;
				vlanconfig[i].memPort=0x0;
				vlanconfig[i].untagSet=0x0;

				/*only one vlan in bridge mode*/
			
			}
			else if(mode==WISP_MODE)
			{
				vlanconfig[i].vid=0;
				vlanconfig[i].memPort=0x0;
				vlanconfig[i].untagSet=0x0;

				/*only one vlan in bridge mode*/
				
			}
			#ifdef CONFIG_RTL865X_LAYERED_DRIVER
			else if(mode==MULTIPLE_VLAN_BRIDGE_MODE)
			{
				vlanconfig[i].vid=RTL_WANVLANID;
				vlanconfig[i].memPort=rtl865x_wanPortMask;
				vlanconfig[i].untagSet=rtl865x_wanPortMask;
				dp->id=8;
			}
			else if(mode==MULTIPLE_VLAN_WISP_MODE)
			{
				vlanconfig[i].vid=RTL_WANVLANID;
				vlanconfig[i].memPort=rtl865x_wanPortMask;
				vlanconfig[i].untagSet=rtl865x_wanPortMask;
				dp->id=8;
			}
			#endif
		}

		/*update vlan id,port mask and port number*/
		dp->id = vlanconfig[i].vid;
		dp->portmask =  vlanconfig[i].memPort;
		dp->portnum  = 0;
		for(j=0;j<RTL8651_AGGREGATOR_NUMBER;j++)
		{
			if(dp->portmask & (1<<j))
				dp->portnum++;
		}	
		
	}
#endif //defined CONFIG_RTK_VLAN_SUPPORT
}
else
{
	for(i=0; i<totalVlans; i++)
	{
		if (IF_ETHER!=vlanconfig[i].if_type)
		{
			continue;
		}
		
		dev=_rtl86xx_dev.dev[i];
		dp = dev->priv;
		
		if(vlanconfig[i].isWan==0)/*lan config*/
		{
			vlanconfig[i].vid=RTL_LANVLANID;
			if(mode==GATEWAY_MODE)
			{
				vlanconfig[i].memPort=rtl865x_lanPortMask;
				vlanconfig[i].untagSet=rtl865x_lanPortMask;
			}
			else if(mode==LANWAN_BRIDGE_MODE)
			{
				vlanconfig[i].memPort=0x11f;
				vlanconfig[i].untagSet=0x11f;
			}
			else if(mode==BRIDGE_MODE)
			{
				vlanconfig[i].memPort=0x1ff;
				vlanconfig[i].untagSet=0x1ff;
			}
			else if(mode==WISP_MODE)
			{
				vlanconfig[i].memPort=0x1ff;
				vlanconfig[i].untagSet=0x1ff;
			}
			#ifdef CONFIG_RTL865X_LAYERED_DRIVER
			else if(mode==MULTIPLE_VLAN_BRIDGE_MODE)
			{
				vlanconfig[i].memPort=rtl865x_lanPortMask;
				vlanconfig[i].untagSet=rtl865x_lanPortMask;
			}
			else if(mode==MULTIPLE_VLAN_WISP_MODE)
			{
				vlanconfig[i].memPort=rtl865x_lanPortMask;
				vlanconfig[i].untagSet=rtl865x_lanPortMask;
			}
			#endif
			
		}
		else/*wan config*/
		{	
			if(mode==GATEWAY_MODE)
			{
				vlanconfig[i].vid=RTL_WANVLANID;
				vlanconfig[i].memPort=rtl865x_wanPortMask;
				vlanconfig[i].untagSet=rtl865x_wanPortMask;

				/*there are two vlan in gateway mode*/
		
			}
			else if(mode==LANWAN_BRIDGE_MODE)
			{
				vlanconfig[i].vid=RTL_WANVLANID;
				vlanconfig[i].memPort=RTL_WANPORT_MASK;
				vlanconfig[i].untagSet=RTL_WANPORT_MASK;
			}
			else if(mode==BRIDGE_MODE)
			{
				vlanconfig[i].vid=0;
				vlanconfig[i].memPort=0x0;
				vlanconfig[i].untagSet=0x0;

				/*only one vlan in bridge mode*/
			
			}
			else if(mode==WISP_MODE)
			{
				vlanconfig[i].vid=0;
				vlanconfig[i].memPort=0x0;
				vlanconfig[i].untagSet=0x0;

				/*only one vlan in bridge mode*/
				
			}
			#ifdef CONFIG_RTL865X_LAYERED_DRIVER
			else if(mode==MULTIPLE_VLAN_BRIDGE_MODE)
			{
				vlanconfig[i].vid=RTL_WANVLANID;
				vlanconfig[i].memPort=rtl865x_wanPortMask;
				vlanconfig[i].untagSet=rtl865x_wanPortMask;
				dp->id=8;
			}
			else if(mode==MULTIPLE_VLAN_WISP_MODE)
			{
				vlanconfig[i].vid=RTL_WANVLANID;
				vlanconfig[i].memPort=rtl865x_wanPortMask;
				vlanconfig[i].untagSet=rtl865x_wanPortMask;
				dp->id=8;
			}
			#endif
		}

		/*update vlan id,port mask and port number*/
		dp->id = vlanconfig[i].vid;
		dp->portmask =  vlanconfig[i].memPort;
		dp->portnum  = 0;
		for(j=0;j<RTL8651_AGGREGATOR_NUMBER;j++)
		{
			if(dp->portmask & (1<<j))
				dp->portnum++;
		}	
		
	}
}

	if (!vlanconfig[0].vid && !vlanconfig[1].vid )
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
		return RTL_EINVALIDVLANID;
#else
		return RTL_INVVID;
#endif	


#ifdef CONFIG_RTL865X_LAYERED_DRIVER
		
	/*re-init sequence eventmgr->l4->l3->l2->common is to make sure delete asic entry,
	if not following this sequence, 
	some asic entry can't be deleted due to reference count is not zero*/

	/*to-do:in each layer reinit function, memset all software entry to zero, 
	and force to clear all asic entry of own module, 
	then the re-init sequence can be common->l2->l3->l4 */

	/*event management */
	//rtl865x_reInitEventMgr();

	/*l4*/
	#ifdef CONFIG_RTL865X_LAYERED_DRIVER_L4
	rtl865x_nat_reinit();
	#endif

	/*l3*/
	#ifdef CONFIG_RTL865X_LAYERED_DRIVER_L3
	rtl865x_reinitRouteTable();
	rtl865x_reinitNxtHopTable();
	rtl865x_reinitIpTable();	
	rtl865x_reinitPppTable();	
	rtl865x_arp_reinit();
	#endif

	/*l2*/
	#ifdef CONFIG_RTL865X_LAYERED_DRIVER_L2
	rtl865x_layer2_reinit();
	#endif

	/*common*/	
	rtl865x_reinitNetifTable();	
	rtl865x_reinitVlantable();
	#ifdef CONFIG_RTL865X_LAYERED_DRIVER_ACL
	rtl865x_reinit_acl();	
	#endif

#else
	/*l4*/
	LR_INIT_CHECK( TBLFIELD(nat_tbl, nat_init)() );

	/*l3*/
	LR_INIT_CHECK( TBLFIELD(if_tbl, if_init)() );	
	LR_INIT_CHECK( TBLFIELD(natip_tbl, natip_init)() );	
	LR_INIT_CHECK( TBLFIELD(rt_tbl, route_init)() );	
	LR_INIT_CHECK( TBLFIELD(arpt_tbl, arp_init)() );

	/*l2*/
	LR_INIT_CHECK( TBLFIELD(fdb_tbl, fdb_init)() );

	/*common*/
	LR_INIT_CHECK( TBLFIELD(vlan_tbl, vlan_init)() );
	LR_INIT_CHECK( TBLFIELD(acl_tbl, acl_init)() );	

	rtl8651_clearAsicAllTable();
#endif	


	
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
	INIT_CHECK(rtl8651_setAsicOperationLayer(2));
#else
	LR_INIT_CHECK(rtl8651_setAsicOperationLayer(2));
#endif

	for(i=0; vlanconfig[i].vid != 0; i++)
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
	{
		rtl865x_netif_t netif;
		
		/*add vlan*/
		retval = rtl865x_addVlan(vlanconfig[i].vid);

		if(retval == SUCCESS)
		{
//			printk("__LINE__:%d, vlanconfig[%d].vid:%d .memPort %x fid %x\n",__LINE__, i, vlanconfig[i].vid, vlanconfig[i].memPort,vlanconfig[i].fid);
			rtl865x_addVlanPortMember(vlanconfig[i].vid,vlanconfig[i].memPort);
			rtl865x_setVlanFilterDatabase(vlanconfig[i].vid,vlanconfig[i].fid);
			
		}

		/*add network interface*/
		memset(&netif, 0, sizeof(rtl865x_netif_t));
		memcpy(netif.name,vlanconfig[i].ifname,MAX_IFNAMESIZE);
		netif.macAddr = vlanconfig[i].mac;
		netif.mtu = vlanconfig[i].mtu;
		netif.if_type = vlanconfig[i].if_type;
		netif.vid = vlanconfig[i].vid;
		netif.is_wan = vlanconfig[i].isWan;
		netif.is_slave = vlanconfig[i].is_slave;
		#ifdef CONFIG_RTL865X_HARDWARE_MULTICAST
		//netif.enableRoute=1;
		#endif
		retval = rtl865x_addNetif(&netif);
		
		if(retval != SUCCESS && retval != RTL_EVLANALREADYEXISTS)
			return retval;
		
		
	}
#else
	{
		if (vlanconfig[i].ifname[0] != '\0')
		{
			rc = (TBLFIELD(if_tbl, if_attach)(&vlanconfig[i]));
			if (rc != RTL_SUCCESS)
				return rc;
		}
		
		rc = TBLFIELD(vlan_tbl, vlan_create)(&vlanconfig[i]);
		if (rc != RTL_SUCCESS && rc != RTL_DUPENTRY)
			return rc;
	}
#endif

#ifdef CONFIG_RTL865X_LAYERED_DRIVER
	if((mode==MULTIPLE_VLAN_BRIDGE_MODE) || (mode==MULTIPLE_VLAN_WISP_MODE))
	{
		for(i=0; vlanconfig[i].vid != 0; i++)
		{
			if(vlanconfig[i].isWan==0)
			{
				rtl865x_setVlanFilterDatabase(vlanconfig[i].vid, 0);
			}
			else
			{
				rtl865x_setVlanFilterDatabase(vlanconfig[i].vid,1);
			}
		}
	}
#endif

	for(i=0; i<RTL8651_PORT_NUMBER + 3; i++) 
	{ 
		/* Set each port's PVID */
		for(j=0,pvid=0; vlanconfig[j].vid != 0; j++)
		{
			if ( (1<<i) & vlanconfig[j].memPort ) 
			{
				pvid = vlanconfig[j].vid;
				break;
			}
		}
		
		if (pvid!=0)	
		{
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
			CONFIG_CHECK(rtl8651_setAsicPvid(i,pvid));
			if(bridgemode!=1)
			{
#if defined(CONFIG_RTK_VLAN_SUPPORT)
			rtl865x_setPortToNetif(vlanconfig[j].ifname,i);
#endif
			}
#else
			LR_CONFIG_CHECK(port_attr.pvid_asic_set(i, pvid));
#endif
		}
	}
	#if defined(CONFIG_RTK_VLAN_SUPPORT)
	if(bridgemode==1)
	{
		CONFIG_CHECK(rtl8651_setAsicPvid(0,9));
		rtl865x_setPortToNetif("br0",0);
		CONFIG_CHECK(rtl8651_setAsicPvid(1,9));
		rtl865x_setPortToNetif("br0",1);
		CONFIG_CHECK(rtl8651_setAsicPvid(2,9));
		rtl865x_setPortToNetif("br0",2);
		CONFIG_CHECK(rtl8651_setAsicPvid(3,9));
		rtl865x_setPortToNetif("br0",3);
		CONFIG_CHECK(rtl8651_setAsicPvid(4,9));
		rtl865x_setPortToNetif("br0",4);
		CONFIG_CHECK(rtl8651_setAsicPvid(8,9));
		rtl865x_setPortToNetif("br0",8);
	}
	#endif
#ifdef CONFIG_RTL_STP
	re865x_stp_mapping_reinit();
#endif

#ifdef CONFIG_RTL8196_RTL8366
  #ifdef CONFIG_RTL865X_LAYERED_DRIVER
	if (mode==GATEWAY_MODE||mode==MULTIPLE_VLAN_BRIDGE_MODE||mode==MULTIPLE_VLAN_WISP_MODE)
  #else
	if (mode==GATEWAY_MODE)
  #endif
	{
		retval = rtl8366rb_initVlan();
		{
			/* for lan */
			int iport;
			rtl8366rb_vlanConfig_t vlancfg_8366;

			memset(&vlancfg_8366, 0, sizeof(rtl8366rb_vlanConfig_t));
			vlancfg_8366.fid = 0;
			vlancfg_8366.mbrmsk = (RTL8366RB_LANPORT&RTL8366RB_PORTMASK)|RTL8366RB_GMIIPORT;
			vlancfg_8366.untagmsk = vlancfg_8366.mbrmsk&(~RTL8366RB_GMIIPORT);
			vlancfg_8366.vid = RTL_LANVLANID;
			retval = rtl8366rb_setVlan(&vlancfg_8366);
			/*need set pvid??*/
			for(iport=0;iport<8;iport++)
				if  ((1<<iport)&vlancfg_8366.mbrmsk)
				{
					retval = rtl8366rb_setVlanPVID(iport, vlancfg_8366.vid, 0);
				}
		}
		{
			/* for wan */
			int iport;
			rtl8366rb_vlanConfig_t vlancfg_8366;

			vlancfg_8366.fid = 0;
			vlancfg_8366.mbrmsk = (RTL8366RB_WANPORT&RTL8366RB_PORTMASK)|RTL8366RB_GMIIPORT;
			vlancfg_8366.untagmsk = vlancfg_8366.mbrmsk&(~RTL8366RB_GMIIPORT);
			vlancfg_8366.vid = RTL_WANVLANID;
			retval = rtl8366rb_setVlan(&vlancfg_8366);
			/*need set pvid??*/
			for(iport=0;iport<8;iport++)
				if  ((1<<iport)&vlancfg_8366.mbrmsk)
				{
					retval = rtl8366rb_setVlanPVID(iport, vlancfg_8366.vid, 0);
				}
		}
	} else if (mode==BRIDGE_MODE||mode==WISP_MODE)
	{
		retval = rtl8366rb_initVlan();
		{
			/* for lan */
			int iport;
			rtl8366rb_vlanConfig_t vlancfg_8366;

			memset(&vlancfg_8366, 0, sizeof(rtl8366rb_vlanConfig_t));
			vlancfg_8366.fid = 0;
			vlancfg_8366.mbrmsk = ((RTL8366RB_WANPORT|RTL8366RB_LANPORT)&RTL8366RB_PORTMASK)|RTL8366RB_GMIIPORT;
			vlancfg_8366.untagmsk = vlancfg_8366.mbrmsk&(~RTL8366RB_GMIIPORT);
			vlancfg_8366.vid = RTL_LANVLANID;
			retval = rtl8366rb_setVlan(&vlancfg_8366);
			/*need set pvid??*/
			for(iport=0;iport<8;iport++)
				if  ((1<<iport)&vlancfg_8366.mbrmsk)
				{
					retval = rtl8366rb_setVlanPVID(iport, vlancfg_8366.vid, 0);
				}
		}
	}
#endif		/* CONFIG_RTL8196_RTL8366	*/

#ifdef CONFIG_RTL865X_IGMP_SNOOPING
	re865x_reInitIgmpSetting(mode);

#if 0 //def CONFIG_RTL865X_HARDWARE_MULTICAST
	/*add default route*/
	if(mode==GATEWAY_MODE)
	{
		rtl865x_addRoute(0,0,0,"eth1");
	}
	else
	{
		rtl865x_addRoute(0,0,0,"br0");
	}
	
	#endif
	
#endif

	/*update current operation mode*/
	rtl865x_curOpMode=mode;

#ifdef CONFIG_RTL865X_LAYERED_DRIVER
	return SUCCESS;
#else
	return RTL_SUCCESS;	
#endif
}

#ifdef CONFIG_RTK_VLAN_SUPPORT

static int32 reinit_vlan_configure(int rtk_vlan_enable)
{
	uint16 pvid;
	int32 i, j;

	int32 totalVlans=((sizeof(vlanconfig))/(sizeof(struct rtl865x_vlanConfig)))-1;	

	/*re-init sequence eventmgr->l4->l3->l2->common is to make sure delete asic entry,
	if not following this sequence, 
	some asic entry can't be deleted due to reference count is not zero*/

	/*to-do:in each layer reinit function, memset all software entry to zero, 
	and force to clear all asic entry of own module, 
	then the re-init sequence can be common->l2->l3->l4 */

	/*event management */
	//rtl865x_reInitEventMgr();

	/*l4*/
	#ifdef CONFIG_RTL865X_LAYERED_DRIVER_L4
	rtl865x_nat_reinit();
	#endif

	/*l3*/
	#ifdef CONFIG_RTL865X_LAYERED_DRIVER_L3
	rtl865x_reinitRouteTable();
	rtl865x_reinitNxtHopTable();
	rtl865x_reinitIpTable();	
	rtl865x_reinitPppTable();	
	rtl865x_arp_reinit();
	#endif

	/*l2*/
	#ifdef CONFIG_RTL865X_LAYERED_DRIVER_L2
	rtl865x_layer2_reinit();
	#endif

	/*common*/	
	rtl865x_reinitNetifTable();	
	rtl865x_reinitVlantable();
	#ifdef CONFIG_RTL865X_LAYERED_DRIVER_ACL
	rtl865x_reinit_acl();	
	#endif

	WRITE_MEM32(PLITIMR,0);

#if defined (CONFIG_RTL865X_IGMP_SNOOPING) && defined (CONFIG_RTL865X_HARDWARE_MULTICAST)	
	INIT_CHECK(rtl8651_setAsicOperationLayer(3));
#endif

	for(i=0; i<totalVlans; i++)
	{
		rtl865x_netif_t netif;

		if(vlanconfig[i].vid == 0)
			continue;
		
		/*add vlan*/
		rtl865x_addVlan(vlanconfig[i].vid);
		rtl865x_addVlanPortMember(vlanconfig[i].vid,vlanconfig[i].memPort);
		

		/*add network interface*/
		memset(&netif, 0, sizeof(rtl865x_netif_t));
		memcpy(netif.name,vlanconfig[i].ifname,MAX_IFNAMESIZE);
		netif.macAddr = vlanconfig[i].mac;
		netif.mtu = vlanconfig[i].mtu;
		netif.if_type = vlanconfig[i].if_type;
		netif.vid = vlanconfig[i].vid;
		netif.is_wan = vlanconfig[i].isWan;
		netif.is_slave = vlanconfig[i].is_slave;
		#ifdef CONFIG_RTL865X_HARDWARE_MULTICAST
		netif.enableRoute=1;
		#endif
		rtl865x_addNetif(&netif);
		if(netif.is_slave == 1)
			rtl865x_attachMasterNetif(netif.name, "eth1");		

		printk("\n");

	}


	for(i=0; i<RTL8651_PORT_NUMBER + 3; i++) 
	{ 
		/* Set each port's PVID */
		for(j=0,pvid=0; vlanconfig[j].vid != 0; j++)
		{
			if ( (1<<i) & vlanconfig[j].memPort ) 
			{
				pvid = vlanconfig[j].vid;
				break;
			}
		}
		
		if (pvid!=0)	
		{
			CONFIG_CHECK(rtl8651_setAsicPvid(i,pvid));
			rtl865x_setPortToNetif(vlanconfig[j].ifname, i);
		}
	}
#ifdef CONFIG_RTL_STP
	re865x_stp_mapping_reinit();
#endif

	return SUCCESS;
}

static int32 rtk_vlan_support_read( char *page, char **start, off_t off, int count, int *eof, void *data )
{
	int len;
	len = sprintf(page, "%s %d\n", "rtk_vlan_support_enable:",rtk_vlan_support_enable);


#if 0
	printk("================\n");
	{
		int j ;
		for(j = 0; j < ETH_INTF_NUM; j++)
		{
			printk("%s, type(%d),iswan(%d),vid(%d),memberport(0x%x),untagSet(0x%x)\n",vlanconfig[j].ifname,vlanconfig[j].if_type,vlanconfig[j].isWan,vlanconfig[j].vid,vlanconfig[j].memPort,vlanconfig[j].untagSet);
		}

		printk("--------------------------\n");
		for(j = 0; j < ETH_INTF_NUM; j++)
		{
			printk("dev(%s) vid(%d), portmask(0x%x)\n",((struct dev_priv *)_rtl86xx_dev.dev[j]->priv)->dev->name, ((struct dev_priv *)_rtl86xx_dev.dev[j]->priv)->id, ((struct dev_priv *)_rtl86xx_dev.dev[j]->priv)->portmask);
		}
		
	}
#endif

	if (len <= off+count) *eof = 1;
	*start = page + off;
	len -= off;
	if (len>count)
		len = count;
	if (len<0)
	  	len = 0;

	return len;
}



static int32 rtk_vlan_support_write( struct file *filp, const char *buff,unsigned long len, void *data )
{
	char 		tmpbuf[32];	
	if (buff && !copy_from_user(tmpbuf, buff, len))
	{
		tmpbuf[len] = '\0';
		if(tmpbuf[0] == '0')
		{
			rtk_vlan_support_enable = 0;
			if(rtl865x_curOpMode == BRIDGE_MODE || rtl865x_curOpMode== WISP_MODE)
			{
				vlanconfig[0].memPort = 0x1ff;
				vlanconfig[0].untagSet = 0x1ff;
				vlanconfig[0].vid=RTL_LANVLANID;
				vlanconfig[1].vid = 0;
				((struct dev_priv *)_rtl86xx_dev.dev[0]->priv)->portmask = 0x1ff; //eth0
				((struct dev_priv *)_rtl86xx_dev.dev[0]->priv)->id = RTL_LANVLANID;
			}
			else
			{				
				vlanconfig[0].memPort = RTL_LANPORT_MASK;
				vlanconfig[0].vid=RTL_LANVLANID;
				vlanconfig[0].untagSet = RTL_LANPORT_MASK;
				((struct dev_priv *)_rtl86xx_dev.dev[0]->priv)->portmask = RTL_LANPORT_MASK; //eth0
				((struct dev_priv *)_rtl86xx_dev.dev[0]->priv)->id = RTL_LANVLANID;
				
				vlanconfig[1].vid = RTL_WANVLANID;
				vlanconfig[1].memPort = RTL_WANPORT_MASK;
				vlanconfig[1].untagSet = RTL_WANPORT_MASK;
				((struct dev_priv *)_rtl86xx_dev.dev[1]->priv)->portmask = RTL_WANPORT_MASK; //eth1
				((struct dev_priv *)_rtl86xx_dev.dev[1]->priv)->id = RTL_WANVLANID; //eth1
				
			}
			
			
			vlanconfig[2] .vid = 0;
			vlanconfig[3] .vid = 0;
			vlanconfig[4] .vid = 0;			
			reinit_vlan_configure(0);
#if defined(CONFIG_RTL865X_LAYERED_DRIVER_ACL)
			rtl865x_enable_acl(1); //enable acl feature
#endif
		}
		else
		{
			rtk_vlan_support_enable = 1;	
			vlanconfig[0].vid = RTL_LANVLANID;
			vlanconfig[0].memPort = RTL_LANPORT_MASK_4;
			vlanconfig[0].untagSet= RTL_LANPORT_MASK_4;			
			((struct dev_priv *)_rtl86xx_dev.dev[0]->priv)->portmask = RTL_LANPORT_MASK_4; //eth0
			((struct dev_priv *)_rtl86xx_dev.dev[0]->priv)->id = RTL_LANVLANID; //eth0

			vlanconfig[1].vid = RTL_WANVLANID;
			vlanconfig[1].memPort = RTL_WANPORT_MASK;
			vlanconfig[1].untagSet= RTL_WANPORT_MASK;			
			((struct dev_priv *)_rtl86xx_dev.dev[1]->priv)->portmask = RTL_WANPORT_MASK; //eth1
			((struct dev_priv *)_rtl86xx_dev.dev[1]->priv)->id = RTL_WANVLANID; //eth1
			
			vlanconfig[2] .vid = RTL_LANVLANID_2;
			vlanconfig[2].memPort = RTL_LANPORT_MASK_3;
			vlanconfig[2].untagSet = RTL_LANPORT_MASK_3;
			((struct dev_priv *)_rtl86xx_dev.dev[2]->priv)->portmask = RTL_LANPORT_MASK_3; //eth2
			((struct dev_priv *)_rtl86xx_dev.dev[2]->priv)->id = RTL_LANVLANID_2; //eth2			
			vlanconfig[3] .vid = RTL_LANVLANID_3;
			vlanconfig[3].memPort = RTL_LANPORT_MASK_2;
			vlanconfig[3].untagSet = RTL_LANPORT_MASK_2;
			((struct dev_priv *)_rtl86xx_dev.dev[3]->priv)->portmask = RTL_LANPORT_MASK_2; //eth3
			((struct dev_priv *)_rtl86xx_dev.dev[3]->priv)->id = RTL_LANVLANID_3; //eth3
			
			vlanconfig[4] .vid = RTL_LANVLANID_4;
			vlanconfig[4].memPort = RTL_LANPORT_MASK_1;
			vlanconfig[4].untagSet = RTL_LANPORT_MASK_1;
			((struct dev_priv *)_rtl86xx_dev.dev[4]->priv)->portmask = RTL_LANPORT_MASK_1; //eth4
			((struct dev_priv *)_rtl86xx_dev.dev[4]->priv)->id = RTL_LANVLANID_4; //eth4

			reinit_vlan_configure(1);
#if defined(CONFIG_RTL865X_LAYERED_DRIVER_ACL)
			rtl865x_enable_acl(0); //disable acl feature
#endif
		}

		#ifdef CONFIG_RTL865X_IGMP_SNOOPING
		re865x_reInitIgmpSetting(rtl865x_curOpMode);
		#endif

	}
	return len;
}


//*********************************************//
#if 0
static int rtk_vlan_netmask_read(char *page, char **start, off_t off, int count, int *eof, void *data)
{

    struct net_device *dev = (struct net_device *)data;
    struct dev_priv *cp = dev->priv;
    int len = 0;

	int j ;
	for(j = 0; j < ETH_INTF_NUM; j++)
	{
		len += sprintf(page+len, "%s, type(%d),iswan(%d),vid(%d),memberport(0x%x),untagSet(0x%x)\n",vlanconfig[j].ifname,vlanconfig[j].if_type,vlanconfig[j].isWan,vlanconfig[j].vid,vlanconfig[j].memPort,vlanconfig[j].untagSet);
	}

	
	for(j = 0; j < ETH_INTF_NUM; j++)
	{
		len += sprintf(page+len, "dev(%s) vid(%d), portmask(0x%x)\n",((struct dev_priv *)_rtl86xx_dev.dev[j]->priv)->dev->name, ((struct dev_priv *)_rtl86xx_dev.dev[j]->priv)->id, ((struct dev_priv *)_rtl86xx_dev.dev[j]->priv)->portmask);
	}
			
    if (len <= off+count) 
        *eof = 1;      
    *start = page + off;      
    len -= off;      
    if (len > count) 
        len = count;      
    if (len < 0) 
        len = 0;      
    return len;
     
}

static int rtk_vlan_netmask_write(struct file *file, const char *buffer, unsigned long count, void *data)
{
//    struct net_device *dev = (struct net_device *)data;
//    struct dev_priv *cp = dev->priv;
	//int old_vlan_id = cp->vlan_setting.id;
	
	char tmp[30];
//	char ifname[5];
//  unsigned int  net_mask[4];

	net_mask[0]=net_mask[1]=net_mask[2]=net_mask[3]=0;
	if (count < 2)
		return -EFAULT;

    	memset(tmp, 0, sizeof(tmp));

	if (buffer && !copy_from_user(tmp, buffer, count)) {

		tmp[count] = '\0';
//		printk("tmp:%s\n",tmp);

		int num = sscanf(tmp, "%x %x %x %x", &net_mask[0],&net_mask[1], &net_mask[2],&net_mask[3]);

       
		if (num !=  4) {
			printk("invalid vlan parameter! %d\n", num);
		}
		printk("test: 0x%x 0x%x 0x%x 0x%x\n",net_mask[0],net_mask[1],net_mask[2],net_mask[3]);
	}

	return count;	
}
#endif

static int read_vlan_num( char *page, char **start, off_t off, int count, int *eof, void *data )
{
	int len;
	len = sprintf(page, "%s %d\n", "amit_vlan_num:", vlan_num);

	int j ;
	for(j = 0; j < ETH_INTF_NUM; j++)
	{
		len += sprintf(page+len, "%s, type(%d),iswan(%d),vid(%d),memberport(0x%x),untagSet(0x%x)\n",vlanconfig[j].ifname,vlanconfig[j].if_type,vlanconfig[j].isWan,vlanconfig[j].vid,vlanconfig[j].memPort,vlanconfig[j].untagSet);
	}

	
	for(j = 0; j < ETH_INTF_NUM; j++)
	{
		len += sprintf(page+len, "dev(%s) vid(%d), portmask(0x%x)\n",((struct dev_priv *)_rtl86xx_dev.dev[j]->priv)->dev->name, ((struct dev_priv *)_rtl86xx_dev.dev[j]->priv)->id, ((struct dev_priv *)_rtl86xx_dev.dev[j]->priv)->portmask);
	}
	if (len <= off+count) *eof = 1;
	*start = page + off;
	len -= off;
	if (len>count)
		len = count;
	if (len<0)
	  	len = 0;

	return len;
}

static int write_vlan_num( struct file *filp, const char *buff,unsigned long len, void *data )
{
	char tmpbuf[32];	
	if (buff && !copy_from_user(tmpbuf, buff, len))
	{
		tmpbuf[len] = '\0';
		if(tmpbuf[0] == '0')
		{
			vlan_num = 0;
			vlanconfig[0].memPort = RTL_LANPORT_MASK;
			vlanconfig[0].vid=RTL_LANVLANID;
			vlanconfig[0].untagSet = RTL_LANPORT_MASK;
			((struct dev_priv *)_rtl86xx_dev.dev[0]->priv)->portmask = RTL_LANPORT_MASK; //eth0
			((struct dev_priv *)_rtl86xx_dev.dev[0]->priv)->id = RTL_LANVLANID;
			
			vlanconfig[1].vid = RTL_WANVLANID;
			vlanconfig[1].memPort = RTL_WANPORT_MASK;
			vlanconfig[1].untagSet = RTL_WANPORT_MASK;
			((struct dev_priv *)_rtl86xx_dev.dev[1]->priv)->portmask = RTL_WANPORT_MASK; //eth1
			((struct dev_priv *)_rtl86xx_dev.dev[1]->priv)->id = RTL_WANVLANID; //eth1
			
			vlanconfig[2] .vid = 0;
			vlanconfig[3] .vid = 0;
			vlanconfig[4] .vid = 0;

      vlanconfig[2].memPort = RTL_LANPORT_MASK;
      vlanconfig[2].untagSet = RTL_LANPORT_MASK;
      ((struct dev_priv *)_rtl86xx_dev.dev[2]->priv)->portmask = RTL_LANPORT_MASK; //eth0
      ((struct dev_priv *)_rtl86xx_dev.dev[2]->priv)->id = 0;
            
			vlanconfig[3].memPort = RTL_LANPORT_MASK;
      vlanconfig[3].untagSet = RTL_LANPORT_MASK;
      ((struct dev_priv *)_rtl86xx_dev.dev[3]->priv)->portmask = RTL_LANPORT_MASK; //eth0
      ((struct dev_priv *)_rtl86xx_dev.dev[3]->priv)->id = 0;
            
			vlanconfig[4].memPort = RTL_LANPORT_MASK;
      vlanconfig[4].untagSet = RTL_LANPORT_MASK;
      ((struct dev_priv *)_rtl86xx_dev.dev[4]->priv)->portmask = RTL_LANPORT_MASK; //eth0
      ((struct dev_priv *)_rtl86xx_dev.dev[4]->priv)->id = 0;



			
//			reinit_vlan_configure(0);
#if defined(CONFIG_RTL865X_LAYERED_DRIVER_ACL)
			rtl865x_enable_acl(1); //enable acl feature
#endif
		}
		else if(tmpbuf[0] == '1')
		{
			vlan_num = 1;	
			vlanconfig[0].vid = RTL_LANVLANID;
			vlanconfig[0].memPort = AMIT_VLAN_MASK_1;
			vlanconfig[0].untagSet= AMIT_VLAN_MASK_1;			
			((struct dev_priv *)_rtl86xx_dev.dev[0]->priv)->portmask = AMIT_VLAN_MASK_1; //eth0
			((struct dev_priv *)_rtl86xx_dev.dev[0]->priv)->id = RTL_LANVLANID; //eth0

			vlanconfig[1].vid = RTL_WANVLANID;
			vlanconfig[1].memPort = RTL_WANPORT_MASK;
			vlanconfig[1].untagSet= RTL_WANPORT_MASK;			
			((struct dev_priv *)_rtl86xx_dev.dev[1]->priv)->portmask = RTL_WANPORT_MASK; //eth1
			((struct dev_priv *)_rtl86xx_dev.dev[1]->priv)->id = RTL_WANVLANID; //eth1
			
			vlanconfig[2] .vid = RTL_LANVLANID_2;
			vlanconfig[2].memPort = RTL_LANPORT_MASK_1;
			vlanconfig[2].untagSet = RTL_LANPORT_MASK_1;
			((struct dev_priv *)_rtl86xx_dev.dev[2]->priv)->portmask = RTL_LANPORT_MASK_1; //eth2
			((struct dev_priv *)_rtl86xx_dev.dev[2]->priv)->id = RTL_LANVLANID_2; //eth2			

			vlanconfig[3] .vid = 0;
			vlanconfig[4] .vid = 0;			
            
			vlanconfig[3].memPort = AMIT_VLAN_MASK_1;
      vlanconfig[3].untagSet = AMIT_VLAN_MASK_1;
      ((struct dev_priv *)_rtl86xx_dev.dev[3]->priv)->portmask = AMIT_VLAN_MASK_1; //eth0
      ((struct dev_priv *)_rtl86xx_dev.dev[3]->priv)->id = 0;
            
			vlanconfig[4].memPort = AMIT_VLAN_MASK_1;
      vlanconfig[4].untagSet = AMIT_VLAN_MASK_1;
      ((struct dev_priv *)_rtl86xx_dev.dev[4]->priv)->portmask = AMIT_VLAN_MASK_1; //eth0
      ((struct dev_priv *)_rtl86xx_dev.dev[4]->priv)->id = 0;


      rtl865x_curOpMode=GATEWAY_MODE;
//			reinit_vlan_configure(1);
#if defined(CONFIG_RTL865X_LAYERED_DRIVER_ACL)
			rtl865x_enable_acl(1); //disable acl feature
#endif
		}
		else if(tmpbuf[0] == '2')
		{
			vlan_num = 2;	
			vlanconfig[0].vid = RTL_LANVLANID;
			vlanconfig[0].memPort = AMIT_VLAN_MASK_1;
			vlanconfig[0].untagSet= AMIT_VLAN_MASK_1;			
			((struct dev_priv *)_rtl86xx_dev.dev[0]->priv)->portmask = AMIT_VLAN_MASK_1; //eth0
			((struct dev_priv *)_rtl86xx_dev.dev[0]->priv)->id = RTL_LANVLANID; //eth0

			vlanconfig[1].vid = RTL_WANVLANID;
			vlanconfig[1].memPort = RTL_WANPORT_MASK;
			vlanconfig[1].untagSet= RTL_WANPORT_MASK;			
			((struct dev_priv *)_rtl86xx_dev.dev[1]->priv)->portmask = RTL_WANPORT_MASK; //eth1
			((struct dev_priv *)_rtl86xx_dev.dev[1]->priv)->id = RTL_WANVLANID; //eth1
			
			vlanconfig[2] .vid = RTL_LANVLANID_2;
			vlanconfig[2].memPort = RTL_LANPORT_MASK_1;
			vlanconfig[2].untagSet = RTL_LANPORT_MASK_1;
			((struct dev_priv *)_rtl86xx_dev.dev[2]->priv)->portmask = RTL_LANPORT_MASK_1; //eth2
			((struct dev_priv *)_rtl86xx_dev.dev[2]->priv)->id = RTL_LANVLANID_2; //eth2			

			vlanconfig[3] .vid = RTL_LANVLANID_3;
			vlanconfig[3].memPort = RTL_LANPORT_MASK_2;
			vlanconfig[3].untagSet = RTL_LANPORT_MASK_2;
			((struct dev_priv *)_rtl86xx_dev.dev[3]->priv)->portmask = RTL_LANPORT_MASK_2; //eth3
			((struct dev_priv *)_rtl86xx_dev.dev[3]->priv)->id = RTL_LANVLANID_3; //eth3

			vlanconfig[4] .vid = 0;
      
      vlanconfig[4].memPort = AMIT_VLAN_MASK_1;
      vlanconfig[4].untagSet = AMIT_VLAN_MASK_1;
      ((struct dev_priv *)_rtl86xx_dev.dev[4]->priv)->portmask = AMIT_VLAN_MASK_1; //eth0
      ((struct dev_priv *)_rtl86xx_dev.dev[4]->priv)->id = 0;
      			
			reinit_vlan_configure(1);
#if defined(CONFIG_RTL865X_LAYERED_DRIVER_ACL)
			rtl865x_enable_acl(1); //disable acl feature
#endif
		}
		else if(tmpbuf[0] == '3')
		{
			vlan_num = 3;	
			vlanconfig[0].vid = RTL_LANVLANID;
			vlanconfig[0].memPort = RTL_LANPORT_MASK_4;
			vlanconfig[0].untagSet= RTL_LANPORT_MASK_4;			
			((struct dev_priv *)_rtl86xx_dev.dev[0]->priv)->portmask = RTL_LANPORT_MASK_4; //eth0
			((struct dev_priv *)_rtl86xx_dev.dev[0]->priv)->id = RTL_LANVLANID; //eth0

			vlanconfig[1].vid = RTL_WANVLANID;
			vlanconfig[1].memPort = RTL_WANPORT_MASK;
			vlanconfig[1].untagSet= RTL_WANPORT_MASK;			
			((struct dev_priv *)_rtl86xx_dev.dev[1]->priv)->portmask = RTL_WANPORT_MASK; //eth1
			((struct dev_priv *)_rtl86xx_dev.dev[1]->priv)->id = RTL_WANVLANID; //eth1
			
			vlanconfig[2] .vid = RTL_LANVLANID_2;
			vlanconfig[2].memPort = RTL_LANPORT_MASK_2;
			vlanconfig[2].untagSet = RTL_LANPORT_MASK_2;
			((struct dev_priv *)_rtl86xx_dev.dev[2]->priv)->portmask = RTL_LANPORT_MASK_2; //eth2
			((struct dev_priv *)_rtl86xx_dev.dev[2]->priv)->id = RTL_LANVLANID_2; //eth2			

			vlanconfig[3] .vid = RTL_LANVLANID_3;
			vlanconfig[3].memPort = RTL_LANPORT_MASK_3;
			vlanconfig[3].untagSet = RTL_LANPORT_MASK_3;
			((struct dev_priv *)_rtl86xx_dev.dev[3]->priv)->portmask = RTL_LANPORT_MASK_3; //eth3
			((struct dev_priv *)_rtl86xx_dev.dev[3]->priv)->id = RTL_LANVLANID_3; //eth3
			
			vlanconfig[4] .vid = RTL_LANVLANID_4;
			vlanconfig[4].memPort = RTL_LANPORT_MASK_1;
			vlanconfig[4].untagSet = RTL_LANPORT_MASK_1;
			((struct dev_priv *)_rtl86xx_dev.dev[4]->priv)->portmask = RTL_LANPORT_MASK_1; //eth4
			((struct dev_priv *)_rtl86xx_dev.dev[4]->priv)->id = RTL_LANVLANID_4; //eth4

			reinit_vlan_configure(1);
#if defined(CONFIG_RTL865X_LAYERED_DRIVER_ACL)
			rtl865x_enable_acl(0); //disable acl feature
#endif
		}

		#ifdef CONFIG_RTL865X_IGMP_SNOOPING
		re865x_reInitIgmpSetting(rtl865x_curOpMode);
		#endif

	}
	return len;
}



//************************************//

static int read_proc_vlan(char *page, char **start, off_t off,
        int count, int *eof, void *data)
{

    struct net_device *dev = (struct net_device *)data;
    struct dev_priv *cp = dev->priv;
    int len;

    len = sprintf(page, "gvlan=%d, lan=%d, vlan=%d, tag=%d, vid=%d, priority=%d, cfi=%d\n", 
		cp->vlan_setting.global_vlan, cp->vlan_setting.is_lan, cp->vlan_setting.vlan, cp->vlan_setting.tag, 
		cp->vlan_setting.id, cp->vlan_setting.pri, cp->vlan_setting.cfi);
			
    if (len <= off+count) 
        *eof = 1;      
    *start = page + off;      
    len -= off;      
    if (len > count) 
        len = count;      
    if (len < 0) 
        len = 0;      
    return len; 
}


static int write_proc_vlan(struct file *file, const char *buffer,
              unsigned long count, void *data)
{
    struct net_device *dev = (struct net_device *)data;
    struct dev_priv *cp = dev->priv;
	//int old_vlan_id = cp->vlan_setting.id;
	
	char tmp[128];
	char ifname[5];

	if (count < 2)
		return -EFAULT;

	if(rtk_vlan_support_enable == 0)
		return count;
	
	if (buffer && !copy_from_user(tmp, buffer, 128)) {
		int num = sscanf(tmp, "%d %d %d %d %d %d %d", 
			&cp->vlan_setting.global_vlan, &cp->vlan_setting.is_lan, 
			&cp->vlan_setting.vlan, &cp->vlan_setting.tag, 
			&cp->vlan_setting.id, &cp->vlan_setting.pri, 
			&cp->vlan_setting.cfi);

		if (num !=  7) {
			printk("invalid vlan parameter!\n");
		}

		{
			int i;	
			//according to user configuration, configure ASIC
			if(cp->id != cp->vlan_setting.id && cp->vlan_setting.global_vlan)
			{
				//remove original vlan's member port
				rtl865x_delVlanPortMember(cp->id, cp->portmask);
				rtl865x_addVlan(cp->vlan_setting.id);
				rtl865x_addVlanPortMember(cp->vlan_setting.id,cp->portmask);
			}
				//set pvid
				for(i = 0; i < RTL8651_AGGREGATOR_NUMBER; i++)
				{
					if(cp->portmask & (1<<i))
						break;
				}
				if(i >= RTL8651_AGGREGATOR_NUMBER)
				{
					printk("BUG!!!!!\n");
					return count;
				}
			

			rtl8651_setAsicPvid(i,cp->vlan_setting.id);				

			//netif
			if(memcmp(dev->name, "eth0",4) == 0)
			{
				rtl865x_setNetifVid("br0", cp->vlan_setting.id);
				rtl865x_setPortToNetif("br0",i);
				memcpy(ifname,"br0",4);
			}
			else
			{
				rtl865x_setNetifVid(dev->name,cp->vlan_setting.id);
				rtl865x_setPortToNetif(dev->name,i);
				memcpy(ifname,dev->name,strlen(dev->name)+1);
			}

			for(i = 0; i < ETH_INTF_NUM; i++)
				if(vlanconfig[i].vid == cp->id && memcmp(vlanconfig[i].ifname,ifname,strlen(ifname)) == 0)
					break;
				
			if(i >= ETH_INTF_NUM)
				printk("--------%s(%d),BUG!!!\n",__FUNCTION__,__LINE__);

			vlanconfig[i].vid = cp->vlan_setting.id;
			cp->id = cp->vlan_setting.id;
				
		}
	}

#if 0
	printk("================\n");

	printk("--%s %s(%d), dev(%s),global_vlan(%d),is_lan(%d),vlan(%d),tag(%d),id(%d),pri(%d),cfi(%d)\n",__FILE__,__FUNCTION__,__LINE__,dev->name,
		cp->vlan_setting.global_vlan, cp->vlan_setting.is_lan, cp->vlan_setting.vlan, cp->vlan_setting.tag, 
		cp->vlan_setting.id, cp->vlan_setting.pri, cp->vlan_setting.cfi);
	printk("cp->id(%d),cp->portmask(%d)\n",cp->id,cp->portmask);

	{
		int j ;
		for(j = 0; j < ETH_INTF_NUM; j++)
		{
			printk("%s, type(%d),iswan(%d),vid(%d),memberport(0x%x)\n",vlanconfig[j].ifname,vlanconfig[j].if_type,vlanconfig[j].isWan,vlanconfig[j].vid,vlanconfig[j].memPort);
			printk(" vid(%d), portmask(0x%x)\n",((struct dev_priv *)_rtl86xx_dev.dev[j]->priv)->id, ((struct dev_priv *)_rtl86xx_dev.dev[j]->priv)->portmask);
		}
	}
#endif
	return count;	
}
#endif // CONFIG_RTK_VLAN_SUPPORT


#if (defined(CONFIG_RTL8186_GR) && !defined(CONFIG_RTL8196_RTL8366)) 

static int isCreated;
static struct proc_dir_entry *res=NULL;
static char passThru_flag[1];

static unsigned long atoi_dec(char *s)
{
	unsigned long k = 0;

	k = 0;
	while (*s != '\0' && *s >= '0' && *s <= '9') {
		k = 10 * k + (*s - '0');
		s++;
	}
	return k;
}

static int custom_Passthru_read_proc(char *page, char **start, off_t off,
		     int count, int *eof, void *data)
{
	int len;	
	len = sprintf(page, "%s\n", passThru_flag);
	if (len <= off+count) 
		*eof = 1;

	*start = page + off;
	len -= off;

	if (len>count) 
		len = count;

	if (len<0) len = 0;

	return len;
}

static int custom_Passthru_write_proc(struct file *file, const char *buffer,
		      unsigned long count, void *data)
{
	int flag,i;

#if	!defined(CONFIG_RTL865X_LAYERED_DRIVER)
	struct rtl865x_vlanConfig  passThruVlanConf = {"passThru",0,IF_ETHER,PASSTHRU_VLAN_ID,0,
		rtl865x_lanPortMask|rtl865x_wanPortMask,
		rtl865x_lanPortMask|rtl865x_wanPortMask,
		1500,{ { 0x00, 0x12, 0x34, 0x56, 0x78, 0x90 } } };
#endif
//	printk("old status=%d\n",oldStatus);	
	if (buffer && !copy_from_user(&passThru_flag, buffer, count))
	{			
		flag=(int)atoi_dec(passThru_flag);		
		//printk("rtl865x_wanPortMask=%d	",rtl865x_wanPortMask);
		if(flag ^ oldStatus)
		{
			
			//IPv6 PassThru
			if((flag & IP6_PASSTHRU_MASK) ^ (oldStatus & IP6_PASSTHRU_MASK)) 
			{			
				if(flag & IP6_PASSTHRU_MASK)
				{//add				
					for(i=0; i<RTL865XC_PORT_NUMBER; i++)
					{
						rtl8651_setProtocolBasedVLAN(IP6_PASSTHRU_RULEID, i, TRUE, PASSTHRU_VLAN_ID);
					}				
				}
				else
				{//delete
					for(i=0; i<RTL865XC_PORT_NUMBER; i++)
					{
						rtl8651_setProtocolBasedVLAN(IP6_PASSTHRU_RULEID, i, FALSE, PASSTHRU_VLAN_ID);
					}
				}
			}

			//PPPoE PassThru
			if((flag & PPPOE_PASSTHRU_MASK) ^ (oldStatus & PPPOE_PASSTHRU_MASK)) 
			{			
				if(flag & PPPOE_PASSTHRU_MASK)
				{//add				
					for(i=0; i<RTL865XC_PORT_NUMBER; i++)
					{
						rtl8651_setProtocolBasedVLAN(RTL8651_PBV_RULE_PPPOE_CONTROL, i, TRUE, PASSTHRU_VLAN_ID);
						rtl8651_setProtocolBasedVLAN(RTL8651_PBV_RULE_PPPOE_SESSION, i, TRUE, PASSTHRU_VLAN_ID);
					}				
				}
				else
				{//delete
					for(i=0; i<RTL865XC_PORT_NUMBER; i++)
					{
						rtl8651_setProtocolBasedVLAN(RTL8651_PBV_RULE_PPPOE_CONTROL, i, FALSE, PASSTHRU_VLAN_ID);
						rtl8651_setProtocolBasedVLAN(RTL8651_PBV_RULE_PPPOE_SESSION, i, FALSE, PASSTHRU_VLAN_ID);
					}
				}
			}
			
			//vlan
			#if 0
			if(!oldStatus)
			{
				TBLFIELD(vlan_tbl, vlan_create)(&passThruVlanConf);
			}
			
			if(!flag)
			{
				TBLFIELD(vlan_tbl, vlan_remove)(PASSTHRU_VLAN_ID);
			}
			#else
			if(!isCreated)
                        {
#ifdef CONFIG_RTL865X_LAYERED_DRIVER
				rtl865x_addVlan(PASSTHRU_VLAN_ID);
				rtl865x_addVlanPortMember(PASSTHRU_VLAN_ID, rtl865x_lanPortMask|rtl865x_wanPortMask);
#else
                                TBLFIELD(vlan_tbl, vlan_create)(&passThruVlanConf);
#endif
				isCreated=1;
                        }
			#endif			
		}
		oldStatus=flag;
		return count;
	}
	return -EFAULT;
}

int32 rtl8651_customPassthru_init(void)
{
	oldStatus=0;
	isCreated=0;
	res = create_proc_entry("custom_Passthru", 0, NULL);	
	if(res)
	{
		res->read_proc = custom_Passthru_read_proc;
		res->write_proc = custom_Passthru_write_proc;
	}
	rtl8651_defineProtocolBasedVLAN( IP6_PASSTHRU_RULEID, 0x0, 0x86DD );
	return 0;
}

void __exit rtl8651_customPassthru_exit(void)
{
	if (res) {
		remove_proc_entry("custom_Passthru", res);				
		res = NULL;
	}
}
#define	MULTICAST_STORM_CONTROL	1
#define	BROADCAST_STORM_CONTROL	2

static struct proc_dir_entry *stormCtrlProc=NULL;
static int rtl865x_stormCtrlReadProc(char *page, char **start, off_t off,
		     int count, int *eof, void *data)
{
	int len;	
	if (len <= off+count) 
		*eof = 1;

	*start = page + off;
	len -= off;

	if (len>count) 
		len = count;

	if (len<0) len = 0;

	return len;
}

static int rtl865x_stormCtrlWriteProc(struct file *file, const char *buffer,
		      unsigned long count, void *data)
{
	int flag,i;

	uint32 tmpBuf[32];
	uint32 stormCtrlType=MULTICAST_STORM_CONTROL;
	uint32 enableStormCtrl=FALSE;
	uint32 percentage=0;
	uint32 uintVal;

	if (buffer && !copy_from_user(tmpBuf, buffer, count))
	{
		tmpBuf[count-1]=0;
		uintVal=simple_strtol(tmpBuf, NULL, 0);
		//printk("%s(%d),tmpBuf=%s,count=%d,uintVal=%d\n",__FUNCTION__,__LINE__,
		//	tmpBuf,count,uintVal);//Added for test
		if(uintVal>100)
		{
			enableStormCtrl=FALSE;
			percentage=0;
		}
		else
		{
			enableStormCtrl=TRUE;
			percentage=uintVal;
		}
		//printk("%s(%d),enableStormCtrl=%d,percentage=%d\n",__FUNCTION__,__LINE__,
		//	enableStormCtrl,percentage);//Added for test
		rtl865x_setStormControl(stormCtrlType,enableStormCtrl,percentage);
		return count;
	}
	return -EFAULT;
}

int32 rtl8651_initStormCtrl(void)
{
	stormCtrlProc = create_proc_entry("StormCtrl", 0, NULL);	
	if(stormCtrlProc)
	{
		stormCtrlProc->read_proc = rtl865x_stormCtrlReadProc;
		stormCtrlProc->write_proc = rtl865x_stormCtrlWriteProc;
	}
	
	return 0;
}

void __exit rtl8651_exitStormCtrl(void)
{
	if (stormCtrlProc) {
		remove_proc_entry("StormCtrl", stormCtrlProc);				
		stormCtrlProc = NULL;
	}
}


#endif


	
